@@ -526,7 +526,8 @@ public class ArrowFieldWriters {
 
         @Override
         public void reset() {
-            fieldVector.reset();
+            super.reset();
+            elementWriter.reset();
             offset = 0;
         }
 
@@ -613,7 +614,9 @@ public class ArrowFieldWriters {
 
         @Override
         public void reset() {
-            fieldVector.reset();
+            super.reset();
+            keyWriter.reset();
+            valueWriter.reset();
             offset = 0;
         }
 
@@ -769,6 +772,14 @@ public class ArrowFieldWriters {
             this.fieldWriters = fieldWriters;
         }
 
+        @Override
+        public void reset() {
+            super.reset();
+            for (ArrowFieldWriter fieldWriter : fieldWriters) {
+                fieldWriter.reset();
+            }
+        }
+
         @Override
         protected void doWrite(
                 ColumnVector columnVector,
