@@ -29,12 +29,13 @@ import org.apache.paimon.data.variant.Variant;
 import org.apache.paimon.types.DataField;
 import org.apache.paimon.types.RowKind;
 import org.apache.paimon.types.RowType;
-import org.apache.paimon.types.VarCharType;
 
 import javax.annotation.Nullable;
 
 import java.util.List;
 
+import static org.apache.paimon.utils.DefaultValueUtils.convertDefaultValue;
+
 /**
  * An implementation of {@link InternalRow} which provides a default value for the underlying {@link
  * InternalRow}.
@@ -215,21 +216,7 @@ public class DefaultValueRow implements InternalRow {
             }
 
             containsDefaultValue = true;
-            @SuppressWarnings("unchecked")
-            CastExecutor<Object, Object> resolve =
-                    (CastExecutor<Object, Object>)
-                            CastExecutors.resolve(VarCharType.STRING_TYPE, dataField.type());
-
-            if (resolve == null) {
-                throw new RuntimeException(
-                        "Default value do not support the type of " + dataField.type());
-            }
-
-            if (defaultValueStr.startsWith("'") && defaultValueStr.endsWith("'")) {
-                defaultValueStr = defaultValueStr.substring(1, defaultValueStr.length() - 1);
-            }
-
-            Object defaultValue = resolve.cast(BinaryString.fromString(defaultValueStr));
+            Object defaultValue = convertDefaultValue(dataField.type(), defaultValueStr);
             row.setField(i, defaultValue);
         }
 
