@@ -21,23 +21,14 @@ package org.apache.paimon.data.serializer;
 import org.apache.paimon.data.AbstractPagedInputView;
 import org.apache.paimon.data.AbstractPagedOutputView;
 import org.apache.paimon.data.BinaryRow;
-import org.apache.paimon.data.BinaryRowWriter;
-import org.apache.paimon.data.BinaryWriter;
-import org.apache.paimon.data.BinaryWriter.ValueSetter;
-import org.apache.paimon.data.Decimal;
 import org.apache.paimon.data.GenericRow;
 import org.apache.paimon.data.InternalRow;
-import org.apache.paimon.data.InternalRow.FieldGetter;
 import org.apache.paimon.data.NestedRow;
-import org.apache.paimon.data.Timestamp;
+import org.apache.paimon.data.RowHelper;
 import org.apache.paimon.io.DataInputView;
 import org.apache.paimon.io.DataOutputView;
 import org.apache.paimon.types.DataType;
-import org.apache.paimon.types.DataTypeChecks;
-import org.apache.paimon.types.DecimalType;
-import org.apache.paimon.types.LocalZonedTimestampType;
 import org.apache.paimon.types.RowType;
-import org.apache.paimon.types.TimestampType;
 
 import java.io.IOException;
 import java.util.Arrays;
@@ -47,58 +38,23 @@ public class InternalRowSerializer extends AbstractRowDataSerializer<InternalRow
 
     private static final long serialVersionUID = 1L;
 
-    private final BinaryRowSerializer binarySerializer;
     private final DataType[] types;
-    private final Serializer[] fieldSerializers;
-    private final FieldGetter[] fieldGetters;
-    private final ValueSetter[] valueSetters;
-    private final boolean[] writeNulls;
-
-    private transient BinaryRow reuseRow;
-    private transient BinaryRowWriter reuseWriter;
+    private final BinaryRowSerializer binarySerializer;
+    private final RowHelper rowHelper;
 
     public InternalRowSerializer(RowType rowType) {
-        this(
-                rowType.getFieldTypes().toArray(new DataType[0]),
-                rowType.getFieldTypes().stream()
-                        .map(InternalSerializers::create)
-                        .toArray(Serializer[]::new));
+        this(rowType.getFieldTypes().toArray(new DataType[0]));
     }
 
     public InternalRowSerializer(DataType... types) {
-        this(
-                types,
-                Arrays.stream(types).map(InternalSerializers::create).toArray(Serializer[]::new));
-    }
-
-    public InternalRowSerializer(DataType[] types, Serializer<?>[] fieldSerializers) {
         this.types = types;
-        this.fieldSerializers = fieldSerializers;
         this.binarySerializer = new BinaryRowSerializer(types.length);
-        this.fieldGetters = new FieldGetter[types.length];
-        this.valueSetters = new ValueSetter[types.length];
-        this.writeNulls = new boolean[types.length];
-        for (int i = 0; i < types.length; i++) {
-            DataType type = types[i];
-            fieldGetters[i] = InternalRow.createFieldGetter(type, i);
-            // pass serializer to avoid infinite loop
-            valueSetters[i] = BinaryWriter.createValueSetter(type, fieldSerializers[i]);
-            // see reference: org.apache.paimon.codegen.GenerateUtils.binaryWriterWriteNull
-            if (type instanceof DecimalType) {
-                writeNulls[i] = !Decimal.isCompact(DataTypeChecks.getPrecision(type));
-            } else if (type instanceof TimestampType || type instanceof LocalZonedTimestampType) {
-                writeNulls[i] = !Timestamp.isCompact(DataTypeChecks.getPrecision(type));
-            }
-        }
+        this.rowHelper = new RowHelper(Arrays.asList(types));
     }
 
     @Override
     public InternalRowSerializer duplicate() {
-        Serializer<?>[] duplicateFieldSerializers = new Serializer[fieldSerializers.length];
-        for (int i = 0; i < fieldSerializers.length; i++) {
-            duplicateFieldSerializers[i] = fieldSerializers[i].duplicate();
-        }
-        return new InternalRowSerializer(types, duplicateFieldSerializers);
+        return new InternalRowSerializer(types);
     }
 
     @Override
@@ -140,7 +96,8 @@ public class InternalRowSerializer extends AbstractRowDataSerializer<InternalRow
         ret.setRowKind(from.getRowKind());
         for (int i = 0; i < from.getFieldCount(); i++) {
             if (!from.isNullAt(i)) {
-                ret.setField(i, fieldSerializers[i].copy((fieldGetters[i].getFieldOrNull(from))));
+                Object field = rowHelper.fieldGetter(i).getFieldOrNull(from);
+                ret.setField(i, rowHelper.serializer(i).copy(field));
             } else {
                 ret.setField(i, null);
             }
@@ -163,22 +120,8 @@ public class InternalRowSerializer extends AbstractRowDataSerializer<InternalRow
         if (row instanceof BinaryRow) {
             return (BinaryRow) row;
         }
-        if (reuseRow == null) {
-            reuseRow = new BinaryRow(types.length);
-            reuseWriter = new BinaryRowWriter(reuseRow);
-        }
-        reuseWriter.reset();
-        reuseWriter.writeRowKind(row.getRowKind());
-        for (int i = 0; i < types.length; i++) {
-            Object field = fieldGetters[i].getFieldOrNull(row);
-            if (field == null && !writeNulls[i]) {
-                reuseWriter.setNullAt(i);
-            } else {
-                valueSetters[i].setValue(reuseWriter, i, field);
-            }
-        }
-        reuseWriter.complete();
-        return reuseRow;
+        rowHelper.copyInto(row);
+        return rowHelper.reuseRow();
     }
 
     @Override
@@ -193,13 +136,12 @@ public class InternalRowSerializer extends AbstractRowDataSerializer<InternalRow
     }
 
     @Override
-    public InternalRow deserializeFromPages(AbstractPagedInputView source) throws IOException {
+    public InternalRow deserializeFromPages(AbstractPagedInputView source) {
         throw new UnsupportedOperationException("Not support!");
     }
 
     @Override
-    public InternalRow deserializeFromPages(InternalRow reuse, AbstractPagedInputView source)
-            throws IOException {
+    public InternalRow deserializeFromPages(InternalRow reuse, AbstractPagedInputView source) {
         throw new UnsupportedOperationException("Not support!");
     }
 
@@ -222,7 +164,7 @@ public class InternalRowSerializer extends AbstractRowDataSerializer<InternalRow
     public boolean equals(Object obj) {
         if (obj instanceof InternalRowSerializer) {
             InternalRowSerializer other = (InternalRowSerializer) obj;
-            return Arrays.equals(fieldSerializers, other.fieldSerializers);
+            return Arrays.equals(types, other.types);
         }
 
         return false;
@@ -230,6 +172,6 @@ public class InternalRowSerializer extends AbstractRowDataSerializer<InternalRow
 
     @Override
     public int hashCode() {
-        return Arrays.hashCode(fieldSerializers);
+        return Arrays.hashCode(types);
     }
 }
