@@ -53,7 +53,6 @@ import java.util.ServiceLoader;
 import java.util.Set;
 import java.util.stream.Collectors;
 
-import static org.apache.paimon.fs.FileIOUtils.checkAccess;
 import static org.apache.paimon.options.CatalogOptions.RESOLVING_FILE_IO_ENABLED;
 import static org.apache.paimon.utils.Preconditions.checkArgument;
 
@@ -73,6 +72,9 @@ public interface FileIO extends Serializable, Closeable {
     /** Configure by {@link CatalogContext}. */
     void configure(CatalogContext context);
 
+    /** Set filesystem options at runtime. Usually used for job-level settings. */
+    default void setRuntimeContext(Map<String, String> options) {}
+
     /**
      * Opens an SeekableInputStream at the indicated Path.
      *
@@ -119,10 +121,9 @@ public interface FileIO extends Serializable, Closeable {
      */
     default FileStatus[] listFiles(Path path, boolean recursive) throws IOException {
         List<FileStatus> files = new ArrayList<>();
-        try (RemoteIterator<FileStatus> iter = listFilesIterative(path, recursive)) {
-            while (iter.hasNext()) {
-                files.add(iter.next());
-            }
+        RemoteIterator<FileStatus> iter = listFilesIterative(path, recursive);
+        while (iter.hasNext()) {
+            files.add(iter.next());
         }
         return files.toArray(new FileStatus[0]);
     }
@@ -168,9 +169,6 @@ public interface FileIO extends Serializable, Closeable {
                     }
                 }
             }
-
-            @Override
-            public void close() {}
         };
     }
 
@@ -607,4 +605,17 @@ public interface FileIO extends Serializable, Closeable {
                 });
         return results;
     }
+
+    static FileIOLoader checkAccess(FileIOLoader fileIO, Path path, CatalogContext config)
+            throws IOException {
+        if (fileIO == null) {
+            return null;
+        }
+
+        // check access
+        FileIO io = fileIO.load(path);
+        io.configure(config);
+        io.exists(path);
+        return fileIO;
+    }
 }
