@@ -31,6 +31,7 @@ import org.apache.paimon.utils.FunctionWithException;
 import org.apache.paimon.utils.Pair;
 import org.apache.paimon.utils.ReflectionUtils;
 
+import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FSDataInputStream;
 import org.apache.hadoop.fs.FSDataOutputStream;
 import org.apache.hadoop.fs.FileSystem;
@@ -53,6 +54,8 @@ public class HadoopFileIO implements FileIO {
 
     protected SerializableConfiguration hadoopConf;
 
+    private org.apache.paimon.options.Options options;
+
     protected transient volatile Map<Pair<String, String>, FileSystem> fsMap;
 
     @VisibleForTesting
@@ -69,6 +72,11 @@ public class HadoopFileIO implements FileIO {
     @Override
     public void configure(CatalogContext context) {
         this.hadoopConf = new SerializableConfiguration(context.hadoopConf());
+        this.options = context.options();
+    }
+
+    public Configuration hadoopConf() {
+        return hadoopConf.get();
     }
 
     @Override
@@ -122,9 +130,6 @@ public class HadoopFileIO implements FileIO {
                 org.apache.hadoop.fs.FileStatus hadoopStatus = hadoopIter.next();
                 return new HadoopFileStatus(hadoopStatus);
             }
-
-            @Override
-            public void close() throws IOException {}
         };
     }
 
@@ -165,7 +170,8 @@ public class HadoopFileIO implements FileIO {
         return new org.apache.hadoop.fs.Path(path.toUri());
     }
 
-    private FileSystem getFileSystem(org.apache.hadoop.fs.Path path) throws IOException {
+    @VisibleForTesting
+    FileSystem getFileSystem(org.apache.hadoop.fs.Path path) throws IOException {
         return getFileSystem(path, this::createFileSystem);
     }
 
@@ -196,7 +202,10 @@ public class HadoopFileIO implements FileIO {
     }
 
     protected FileSystem createFileSystem(org.apache.hadoop.fs.Path path) throws IOException {
-        return path.getFileSystem(hadoopConf.get());
+        Configuration conf = hadoopConf.get();
+        FileSystem fileSystem = path.getFileSystem(conf);
+        fileSystem = HadoopSecuredFileSystem.trySecureFileSystem(fileSystem, options, conf);
+        return fileSystem;
     }
 
     private static class HadoopSeekableInputStream extends SeekableInputStream {
