@@ -28,6 +28,8 @@ import org.apache.paimon.data.InternalMap;
 import org.apache.paimon.data.InternalRow;
 import org.apache.paimon.data.NestedRow;
 import org.apache.paimon.data.Timestamp;
+import org.apache.paimon.data.variant.GenericVariant;
+import org.apache.paimon.data.variant.Variant;
 import org.apache.paimon.io.DataOutputView;
 import org.apache.paimon.utils.MurmurHashUtils;
 
@@ -1121,6 +1123,16 @@ public class MemorySegmentUtils {
         }
     }
 
+    public static Variant readVariant(MemorySegment[] segments, int baseOffset, long offsetAndLen) {
+        int offset = baseOffset + (int) (offsetAndLen >> 32);
+        int totalSize = (int) offsetAndLen;
+        int valueSize = getInt(segments, offset);
+        int metadataSize = totalSize - 4 - valueSize;
+        byte[] value = copyToBytes(segments, offset + 4, valueSize);
+        byte[] metadata = copyToBytes(segments, offset + 4 + valueSize, metadataSize);
+        return new GenericVariant(value, metadata);
+    }
+
     /** Gets an instance of {@link InternalMap} from underlying {@link MemorySegment}. */
     public static InternalMap readMapData(
             MemorySegment[] segments, int baseOffset, long offsetAndSize) {
