@@ -19,6 +19,7 @@
 package org.apache.paimon.format;
 
 import org.apache.paimon.data.BinaryArray;
+import org.apache.paimon.data.BinaryString;
 import org.apache.paimon.data.Decimal;
 import org.apache.paimon.data.GenericArray;
 import org.apache.paimon.data.GenericMap;
@@ -33,6 +34,7 @@ import org.apache.paimon.fs.FileIO;
 import org.apache.paimon.fs.Path;
 import org.apache.paimon.fs.PositionOutputStream;
 import org.apache.paimon.fs.local.LocalFileIO;
+import org.apache.paimon.reader.FileRecordReader;
 import org.apache.paimon.reader.RecordReader;
 import org.apache.paimon.types.ArrayType;
 import org.apache.paimon.types.DataType;
@@ -55,17 +57,21 @@ import java.util.concurrent.ThreadLocalRandom;
 import java.util.stream.IntStream;
 
 import static org.apache.paimon.data.BinaryString.fromString;
+import static org.apache.paimon.testutils.assertj.PaimonAssertions.anyCauseMatches;
 import static org.assertj.core.api.Assertions.assertThat;
+import static org.assertj.core.api.Assertions.assertThatThrownBy;
+import static org.junit.jupiter.api.Assumptions.assumeTrue;
 
 /** Test Base class for Format. */
 public abstract class FormatReadWriteTest {
 
     @TempDir java.nio.file.Path tempPath;
 
-    private final String formatType;
+    protected final String formatType;
 
     protected FileIO fileIO;
     protected Path file;
+    protected Path parent;
 
     protected FormatReadWriteTest(String formatType) {
         this.formatType = formatType;
@@ -74,6 +80,7 @@ public abstract class FormatReadWriteTest {
     @BeforeEach
     public void beforeEach() {
         this.fileIO = LocalFileIO.create();
+        this.parent = new Path(tempPath.toUri());
         this.file = new Path(new Path(tempPath.toUri()), UUID.randomUUID() + "." + formatType);
     }
 
@@ -81,49 +88,47 @@ public abstract class FormatReadWriteTest {
 
     @Test
     public void testSimpleTypes() throws IOException {
+        FileFormat format = fileFormat();
+        testSimpleTypesUtil(format, file);
+    }
+
+    protected void testSimpleTypesUtil(FileFormat format, Path file) throws IOException {
         RowType rowType = DataTypes.ROW(DataTypes.INT().notNull(), DataTypes.BIGINT());
 
         if (ThreadLocalRandom.current().nextBoolean()) {
-            rowType = (RowType) rowType.notNull();
+            rowType = rowType.notNull();
         }
 
         InternalRowSerializer serializer = new InternalRowSerializer(rowType);
-        FileFormat format = fileFormat();
-
-        PositionOutputStream out = fileIO.newOutputStream(file, false);
-        FormatWriter writer = format.createWriterFactory(rowType).create(out, "zstd");
-        writer.addElement(GenericRow.of(1, 1L));
-        writer.addElement(GenericRow.of(2, 2L));
-        writer.addElement(GenericRow.of(3, null));
-        writer.close();
-        out.close();
-
+        FormatWriterFactory factory = format.createWriterFactory(rowType);
+        write(factory, file, GenericRow.of(1, 1L), GenericRow.of(2, 2L), GenericRow.of(3, null));
         RecordReader<InternalRow> reader =
-                format.createReaderFactory(rowType)
+                format.createReaderFactory(rowType, rowType, new ArrayList<>())
                         .createReader(
                                 new FormatReaderContext(fileIO, file, fileIO.getFileSize(file)));
         List<InternalRow> result = new ArrayList<>();
         reader.forEachRemaining(row -> result.add(serializer.copy(row)));
 
-        assertThat(result)
-                .containsExactly(
-                        GenericRow.of(1, 1L), GenericRow.of(2, 2L), GenericRow.of(3, null));
+        assertThat(result.get(0)).isEqualTo(GenericRow.of(1, 1L));
+        assertThat(result.get(1)).isEqualTo(GenericRow.of(2, 2L));
+        assertThat(result.get(2).getInt(0)).isEqualTo(3);
+        assertThat(result.get(2).isNullAt(1)).isTrue();
     }
 
     @Test
     public void testFullTypes() throws IOException {
+        FileFormat format = fileFormat();
+        testFullTypesUtil(format, file);
+    }
+
+    protected void testFullTypesUtil(FileFormat format, Path file) throws IOException {
         RowType rowType = rowTypeForFullTypesTest();
         InternalRow expected = expectedRowForFullTypesTest();
-        FileFormat format = fileFormat();
-
-        PositionOutputStream out = fileIO.newOutputStream(file, false);
-        FormatWriter writer = format.createWriterFactory(rowType).create(out, "zstd");
-        writer.addElement(expected);
-        writer.close();
-        out.close();
 
+        FormatWriterFactory factory = format.createWriterFactory(rowType);
+        write(factory, file, expected);
         RecordReader<InternalRow> reader =
-                format.createReaderFactory(rowType)
+                format.createReaderFactory(rowType, rowType, new ArrayList<>())
                         .createReader(
                                 new FormatReaderContext(fileIO, file, fileIO.getFileSize(file)));
         InternalRowSerializer internalRowSerializer = new InternalRowSerializer(rowType);
@@ -134,8 +139,19 @@ public abstract class FormatReadWriteTest {
         validateFullTypesResult(result.get(0), expected);
     }
 
+    public boolean supportNestedReadPruning() {
+        return true;
+    }
+
+    public String compression() {
+        return "zstd";
+    }
+
     @Test
     public void testNestedReadPruning() throws Exception {
+        if (!supportNestedReadPruning()) {
+            return;
+        }
         FileFormat format = fileFormat();
 
         RowType writeType =
@@ -149,10 +165,8 @@ public abstract class FormatReadWriteTest {
                                         DataTypes.FIELD(3, "f1", DataTypes.INT()),
                                         DataTypes.FIELD(4, "f2", DataTypes.INT()))));
 
-        try (PositionOutputStream out = fileIO.newOutputStream(file, false);
-                FormatWriter writer = format.createWriterFactory(writeType).create(out, "zstd")) {
-            writer.addElement(GenericRow.of(0, GenericRow.of(10, 11, 12)));
-        }
+        FormatWriterFactory factory = format.createWriterFactory(writeType);
+        write(factory, file, GenericRow.of(0, GenericRow.of(10, 11, 12)));
 
         // skip read f0, f1.f1
         RowType readType =
@@ -166,14 +180,15 @@ public abstract class FormatReadWriteTest {
 
         List<InternalRow> result = new ArrayList<>();
         try (RecordReader<InternalRow> reader =
-                format.createReaderFactory(readType)
+                format.createReaderFactory(readType, readType, new ArrayList<>())
                         .createReader(
                                 new FormatReaderContext(fileIO, file, fileIO.getFileSize(file)))) {
             InternalRowSerializer serializer = new InternalRowSerializer(readType);
             reader.forEachRemaining(row -> result.add(serializer.copy(row)));
         }
 
-        assertThat(result).containsExactly(GenericRow.of(GenericRow.of(10, 12)));
+        assertThat(result.get(0).getRow(0, 2).getInt(0)).isEqualTo(10);
+        assertThat(result.get(0).getRow(0, 2).getInt(1)).isEqualTo(12);
     }
 
     @Test
@@ -186,15 +201,14 @@ public abstract class FormatReadWriteTest {
 
         RowType writeType = DataTypes.ROW(DataTypes.FIELD(0, "v", DataTypes.VARIANT()));
 
-        try (PositionOutputStream out = fileIO.newOutputStream(file, false);
-                FormatWriter writer = format.createWriterFactory(writeType).create(out, "zstd")) {
-            writer.addElement(
-                    GenericRow.of(GenericVariant.fromJson("{\"age\":35,\"city\":\"Chicago\"}")));
-        }
-
+        FormatWriterFactory factory = format.createWriterFactory(writeType);
+        write(
+                factory,
+                file,
+                GenericRow.of(GenericVariant.fromJson("{\"age\":35,\"city\":\"Chicago\"}")));
         List<InternalRow> result = new ArrayList<>();
         try (RecordReader<InternalRow> reader =
-                format.createReaderFactory(writeType)
+                format.createReaderFactory(writeType, writeType, new ArrayList<>())
                         .createReader(
                                 new FormatReaderContext(fileIO, file, fileIO.getFileSize(file)))) {
             InternalRowSerializer serializer = new InternalRowSerializer(writeType);
@@ -214,22 +228,19 @@ public abstract class FormatReadWriteTest {
         }
 
         RowType writeType = DataTypes.ROW(new ArrayType(true, new VariantType()));
-
-        try (PositionOutputStream out = fileIO.newOutputStream(file, false);
-                FormatWriter writer = format.createWriterFactory(writeType).create(out, "zstd")) {
-            writer.addElement(
-                    GenericRow.of(
-                            new GenericArray(
-                                    new Object[] {
-                                        GenericVariant.fromJson(
-                                                "{\"age\":35,\"city\":\"Chicago\"}"),
-                                        GenericVariant.fromJson("{\"age\":45,\"city\":\"Beijing\"}")
-                                    })));
-        }
+        write(
+                format.createWriterFactory(writeType),
+                file,
+                GenericRow.of(
+                        new GenericArray(
+                                new Object[] {
+                                    GenericVariant.fromJson("{\"age\":35,\"city\":\"Chicago\"}"),
+                                    GenericVariant.fromJson("{\"age\":45,\"city\":\"Beijing\"}")
+                                })));
 
         List<InternalRow> result = new ArrayList<>();
         try (RecordReader<InternalRow> reader =
-                format.createReaderFactory(writeType)
+                format.createReaderFactory(writeType, writeType, new ArrayList<>())
                         .createReader(
                                 new FormatReaderContext(fileIO, file, fileIO.getFileSize(file)))) {
             InternalRowSerializer serializer = new InternalRowSerializer(writeType);
@@ -241,7 +252,46 @@ public abstract class FormatReadWriteTest {
         assertThat(array.getVariant(1).toJson()).isEqualTo("{\"age\":45,\"city\":\"Beijing\"}");
     }
 
-    private RowType rowTypeForFullTypesTest() {
+    @Test
+    public void testWriteNullToNonNullField() {
+        FileFormat format = fileFormat();
+        String identifier = format.getFormatIdentifier();
+        // no agg for these formats now
+        assumeTrue(!identifier.equals("csv") && !identifier.equals("json"));
+
+        FormatWriterFactory factory =
+                format.createWriterFactory(
+                        RowType.builder().field("f0", DataTypes.INT().notNull()).build());
+
+        assertThatThrownBy(() -> write(factory, file, GenericRow.of((Object) null)))
+                .satisfies(
+                        anyCauseMatches(
+                                IllegalArgumentException.class,
+                                "Field 'f0' expected not null but found null value. A possible cause is "
+                                        + "that the table used partial-update or aggregation merge-engine and the aggregate "
+                                        + "function produced null value when retracting."));
+    }
+
+    protected void write(FormatWriterFactory factory, Path file, InternalRow... rows)
+            throws IOException {
+        FormatWriter writer;
+        PositionOutputStream out = null;
+        if (factory instanceof SupportsDirectWrite) {
+            writer = ((SupportsDirectWrite) factory).create(fileIO, file, this.compression());
+        } else {
+            out = fileIO.newOutputStream(file, false);
+            writer = factory.create(out, this.compression());
+        }
+        for (InternalRow row : rows) {
+            writer.addElement(row);
+        }
+        writer.close();
+        if (out != null) {
+            out.close();
+        }
+    }
+
+    protected RowType rowTypeForFullTypesTest() {
         RowType.Builder builder =
                 RowType.builder()
                         .field("id", DataTypes.INT().notNull())
@@ -292,7 +342,7 @@ public abstract class FormatReadWriteTest {
         return rowType;
     }
 
-    private GenericRow expectedRowForFullTypesTest() {
+    protected GenericRow expectedRowForFullTypesTest() {
         Object[] mapValueData = getMapValueData();
         List<Object> values =
                 Arrays.asList(
@@ -333,6 +383,94 @@ public abstract class FormatReadWriteTest {
         return GenericRow.of(values.toArray());
     }
 
+    public boolean supportDataFileWithoutExtension() {
+        return false;
+    }
+
+    @Test
+    public void testWriteAndReadFileWithoutExtension() throws IOException {
+        if (!supportDataFileWithoutExtension()) {
+            return;
+        }
+        RowType rowType =
+                RowType.of(DataTypes.INT().notNull(), DataTypes.STRING(), DataTypes.BOOLEAN());
+
+        // Create test data
+        List<InternalRow> testData = new ArrayList<>();
+        testData.add(GenericRow.of(1, BinaryString.fromString("Alice"), true));
+        testData.add(GenericRow.of(2, BinaryString.fromString("Bob"), false));
+        testData.add(GenericRow.of(3, BinaryString.fromString("Charlie"), true));
+
+        // Create file format
+        FileFormat jsonFormat = fileFormat();
+
+        // Write data
+        Path filePath = new Path(parent, UUID.randomUUID().toString());
+        FormatWriterFactory writerFactory = jsonFormat.createWriterFactory(rowType);
+        try (FormatWriter writer =
+                writerFactory.create(fileIO.newOutputStream(filePath, false), compression())) {
+            for (InternalRow row : testData) {
+                writer.addElement(row);
+            }
+        }
+
+        // Read data
+        FormatReaderFactory readerFactory = jsonFormat.createReaderFactory(rowType, rowType, null);
+        FileRecordReader<InternalRow> reader =
+                readerFactory.createReader(
+                        new FormatReaderFactory.Context() {
+                            @Override
+                            public FileIO fileIO() {
+                                return fileIO;
+                            }
+
+                            @Override
+                            public Path filePath() {
+                                return filePath;
+                            }
+
+                            @Override
+                            public long fileSize() {
+                                try {
+                                    return fileIO.getFileSize(filePath);
+                                } catch (IOException e) {
+                                    throw new RuntimeException(e);
+                                }
+                            }
+
+                            @Override
+                            public org.apache.paimon.utils.RoaringBitmap32 selection() {
+                                return null;
+                            }
+                        });
+
+        List<InternalRow> readData = new ArrayList<>();
+        RecordReader.RecordIterator<InternalRow> iterator = reader.readBatch();
+        while (iterator != null) {
+            InternalRow row;
+            while ((row = iterator.next()) != null) {
+                readData.add(GenericRow.of(row.getInt(0), row.getString(1), row.getBoolean(2)));
+            }
+            iterator.releaseBatch();
+            iterator = reader.readBatch();
+        }
+        reader.close();
+
+        // Verify data
+        assertThat(readData).hasSize(3);
+        assertThat(readData.get(0).getInt(0)).isEqualTo(1);
+        assertThat(readData.get(0).getString(1).toString()).isEqualTo("Alice");
+        assertThat(readData.get(0).getBoolean(2)).isTrue();
+
+        assertThat(readData.get(1).getInt(0)).isEqualTo(2);
+        assertThat(readData.get(1).getString(1).toString()).isEqualTo("Bob");
+        assertThat(readData.get(1).getBoolean(2)).isFalse();
+
+        assertThat(readData.get(2).getInt(0)).isEqualTo(3);
+        assertThat(readData.get(2).getString(1).toString()).isEqualTo("Charlie");
+        assertThat(readData.get(2).getBoolean(2)).isTrue();
+    }
+
     private DataType getMapValueType() {
         if (formatType.equals("avro") || formatType.equals("orc")) {
             return DataTypes.ROW(
@@ -352,7 +490,7 @@ public abstract class FormatReadWriteTest {
         }
     }
 
-    private void validateFullTypesResult(InternalRow actual, InternalRow expected) {
+    protected void validateFullTypesResult(InternalRow actual, InternalRow expected) {
         RowType rowType = rowTypeForFullTypesTest();
         InternalRow.FieldGetter[] fieldGetters =
                 IntStream.range(0, rowType.getFieldCount())
