@@ -186,7 +186,8 @@ public interface Catalog extends AutoCloseable {
             String databaseName,
             @Nullable Integer maxResults,
             @Nullable String pageToken,
-            @Nullable String tableNamePattern)
+            @Nullable String tableNamePattern,
+            @Nullable String tableType)
             throws DatabaseNotExistException;
 
     /**
@@ -203,6 +204,8 @@ public interface Catalog extends AutoCloseable {
      *     from a specific point.
      * @param tableNamePattern A sql LIKE pattern (%) for table names. All table details will be
      *     returned if not set or empty. Currently, only prefix matching is supported.
+     * @param tableType Optional parameter to filter tables by table type. All table types will be
+     *     returned if not set or empty.
      * @return a list of the table details with provided page size in this database and next page
      *     token, or a list of the details of all tables in this database if the catalog does not
      *     {@link #supportsListObjectsPaged()}.
@@ -213,7 +216,8 @@ public interface Catalog extends AutoCloseable {
             String databaseName,
             @Nullable Integer maxResults,
             @Nullable String pageToken,
-            @Nullable String tableNamePattern)
+            @Nullable String tableNamePattern,
+            @Nullable String tableType)
             throws DatabaseNotExistException;
 
     /**
@@ -561,15 +565,27 @@ public interface Catalog extends AutoCloseable {
         throw new UnsupportedOperationException();
     }
 
+    /**
+     * Register a paimon table to the catalog if it does not exist. It is an asynchronous operation
+     *
+     * @param identifier a table identifier
+     * @param path the path of the table
+     * @throws TableAlreadyExistException if the table already exists in the catalog.
+     */
+    default void registerTable(Identifier identifier, String path)
+            throws TableAlreadyExistException {
+        throw new UnsupportedOperationException();
+    }
+
     /**
      * Whether this catalog supports list objects paged. If not, corresponding methods will fall
      * back to listing all objects. For example, {@link #listTablesPaged(String, Integer, String,
-     * String)} would fall back to {@link #listTables(String)}.
+     * String, String)} would fall back to {@link #listTables(String)}.
      *
      * <ul>
      *   <li>{@link #listDatabasesPaged(Integer, String, String)}.
-     *   <li>{@link #listTablesPaged(String, Integer, String, String)}.
-     *   <li>{@link #listTableDetailsPaged(String, Integer, String, String)}.
+     *   <li>{@link #listTablesPaged(String, Integer, String, String, String)}.
+     *   <li>{@link #listTableDetailsPaged(String, Integer, String, String, String)}.
      *   <li>{@link #listViewsPaged(String, Integer, String, String)}.
      *   <li>{@link #listViewDetailsPaged(String, Integer, String, String)}.
      *   <li>{@link #listPartitionsPaged(Identifier, Integer, String, String)}.
@@ -583,8 +599,8 @@ public interface Catalog extends AutoCloseable {
      *
      * <ul>
      *   <li>{@link #listDatabasesPaged(Integer, String, String)}.
-     *   <li>{@link #listTablesPaged(String, Integer, String, String)}.
-     *   <li>{@link #listTableDetailsPaged(String, Integer, String, String)}.
+     *   <li>{@link #listTablesPaged(String, Integer, String, String, String)}.
+     *   <li>{@link #listTableDetailsPaged(String, Integer, String, String, String)}.
      *   <li>{@link #listViewsPaged(String, Integer, String, String)}.
      *   <li>{@link #listViewDetailsPaged(String, Integer, String, String)}.
      *   <li>{@link #listPartitionsPaged(Identifier, Integer, String, String)}.
@@ -594,6 +610,10 @@ public interface Catalog extends AutoCloseable {
         return false;
     }
 
+    default boolean supportsListTableByType() {
+        return false;
+    }
+
     // ==================== Version management methods ==========================
 
     /**
@@ -601,7 +621,7 @@ public interface Catalog extends AutoCloseable {
      * will throw an {@link UnsupportedOperationException}, affect the following methods:
      *
      * <ul>
-     *   <li>{@link #commitSnapshot(Identifier, Snapshot, List)}.
+     *   <li>{@link #commitSnapshot(Identifier, String, Snapshot, List)}.
      *   <li>{@link #loadSnapshot(Identifier)}.
      *   <li>{@link #rollbackTo(Identifier, Instant)}.
      *   <li>{@link #createBranch(Identifier, String, String)}.
@@ -775,6 +795,8 @@ public interface Catalog extends AutoCloseable {
     void alterPartitions(Identifier identifier, List<PartitionStatistics> partitions)
             throws TableNotExistException;
 
+    // ======================= Function methods ===============================
+
     /**
      * Get the names of all functions in this catalog.
      *
@@ -1029,12 +1051,17 @@ public interface Catalog extends AutoCloseable {
 
     /** Exception for trying to operate on the database that doesn't have permission. */
     class DatabaseNoPermissionException extends RuntimeException {
-        private static final String MSG = "Database %s has no permission.";
+        private static final String MSG = "Database %s has no permission. Cause by %s.";
 
         private final String database;
 
         public DatabaseNoPermissionException(String database, Throwable cause) {
-            super(String.format(MSG, database), cause);
+            super(
+                    String.format(
+                            MSG,
+                            database,
+                            cause != null && cause.getMessage() != null ? cause.getMessage() : ""),
+                    cause);
             this.database = database;
         }
 
@@ -1093,12 +1120,17 @@ public interface Catalog extends AutoCloseable {
     /** Exception for trying to operate on the table that doesn't have permission. */
     class TableNoPermissionException extends RuntimeException {
 
-        private static final String MSG = "Table %s has no permission.";
+        private static final String MSG = "Table %s has no permission. Cause by %s.";
 
         private final Identifier identifier;
 
         public TableNoPermissionException(Identifier identifier, Throwable cause) {
-            super(String.format(MSG, identifier.getFullName()), cause);
+            super(
+                    String.format(
+                            MSG,
+                            identifier.getFullName(),
+                            cause != null && cause.getMessage() != null ? cause.getMessage() : ""),
+                    cause);
             this.identifier = identifier;
         }
 
