@@ -25,6 +25,7 @@ import java.io.DataOutputStream;
 import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.util.Objects;
+import java.util.zip.CRC32;
 
 /**
  * A {@link DeletionVector} based on {@link RoaringBitmap32}, it only supports files with row count
@@ -33,6 +34,7 @@ import java.util.Objects;
 public class BitmapDeletionVector implements DeletionVector {
 
     public static final int MAGIC_NUMBER = 1581511376;
+    public static final int MAGIC_NUMBER_SIZE_BYTES = 4;
 
     private final RoaringBitmap32 roaringBitmap;
 
@@ -82,12 +84,17 @@ public class BitmapDeletionVector implements DeletionVector {
     }
 
     @Override
-    public byte[] serializeToBytes() {
+    public int serializeTo(DataOutputStream out) {
         try (ByteArrayOutputStream bos = new ByteArrayOutputStream();
                 DataOutputStream dos = new DataOutputStream(bos)) {
             dos.writeInt(MAGIC_NUMBER);
             roaringBitmap.serialize(dos);
-            return bos.toByteArray();
+            byte[] data = bos.toByteArray();
+            int size = data.length;
+            out.writeInt(size);
+            out.write(data);
+            out.writeInt(calculateChecksum(data));
+            return size;
         } catch (Exception e) {
             throw new RuntimeException("Unable to serialize deletion vector", e);
         }
@@ -131,4 +138,10 @@ public class BitmapDeletionVector implements DeletionVector {
     public int hashCode() {
         return Objects.hashCode(roaringBitmap);
     }
+
+    public static int calculateChecksum(byte[] bytes) {
+        CRC32 crc = new CRC32();
+        crc.update(bytes);
+        return (int) crc.getValue();
+    }
 }
