@@ -84,7 +84,6 @@ public class MergeTreeWriter implements RecordWriter<KeyValue>, MemoryOwner {
 
     private long newSequenceNumber;
     private WriteBuffer writeBuffer;
-    private boolean isInsertOnly;
 
     public MergeTreeWriter(
             boolean writeBufferSpillable,
@@ -215,7 +214,7 @@ public class MergeTreeWriter implements RecordWriter<KeyValue>, MemoryOwner {
             }
 
             final RollingFileWriter<KeyValue, DataFileMeta> changelogWriter =
-                    (changelogProducer == ChangelogProducer.INPUT && !isInsertOnly)
+                    changelogProducer == ChangelogProducer.INPUT
                             ? writerFactory.createRollingChangelogFileWriter(0)
                             : null;
             final RollingFileWriter<KeyValue, DataFileMeta> dataWriter =
@@ -235,23 +234,13 @@ public class MergeTreeWriter implements RecordWriter<KeyValue>, MemoryOwner {
                 dataWriter.close();
             }
 
-            List<DataFileMeta> dataMetas = dataWriter.result();
             if (changelogWriter != null) {
                 newFilesChangelog.addAll(changelogWriter.result());
-            } else if (changelogProducer == ChangelogProducer.INPUT && isInsertOnly) {
-                List<DataFileMeta> changelogMetas = new ArrayList<>();
-                for (DataFileMeta dataMeta : dataMetas) {
-                    String newFileName = writerFactory.newChangelogFileName(0);
-                    DataFileMeta changelogMeta = dataMeta.rename(newFileName);
-                    writerFactory.copyFile(dataMeta, changelogMeta);
-                    changelogMetas.add(changelogMeta);
-                }
-                newFilesChangelog.addAll(changelogMetas);
             }
 
-            for (DataFileMeta dataMeta : dataMetas) {
-                newFiles.add(dataMeta);
-                compactManager.addNewFile(dataMeta);
+            for (DataFileMeta fileMeta : dataWriter.result()) {
+                newFiles.add(fileMeta);
+                compactManager.addNewFile(fileMeta);
             }
         }
 
@@ -288,18 +277,6 @@ public class MergeTreeWriter implements RecordWriter<KeyValue>, MemoryOwner {
         trySyncLatestCompaction(true);
     }
 
-    @Override
-    public void withInsertOnly(boolean insertOnly) {
-        if (this.isInsertOnly == insertOnly) {
-            return;
-        }
-        if (insertOnly && writeBuffer != null && writeBuffer.size() > 0) {
-            throw new IllegalStateException(
-                    "Insert-only can only be set before any record is received.");
-        }
-        this.isInsertOnly = insertOnly;
-    }
-
     private CommitIncrement drainIncrement() {
         DataIncrement dataIncrement =
                 new DataIncrement(
