@@ -25,5 +25,5 @@ public interface Migrator {
 
     void renameTable(boolean ignoreIfNotExists) throws Exception;
 
-    public void deleteOriginTable(boolean delete) throws Exception;
+    void deleteOriginTable(boolean delete) throws Exception;
 }
