@@ -22,9 +22,9 @@ import org.apache.paimon.Snapshot;
 import org.apache.paimon.data.BinaryRow;
 import org.apache.paimon.data.InternalRow;
 import org.apache.paimon.manifest.BucketEntry;
+import org.apache.paimon.manifest.BucketFilter;
 import org.apache.paimon.manifest.FileEntry;
 import org.apache.paimon.manifest.FileEntry.Identifier;
-import org.apache.paimon.manifest.ManifestCacheFilter;
 import org.apache.paimon.manifest.ManifestEntry;
 import org.apache.paimon.manifest.ManifestEntrySerializer;
 import org.apache.paimon.manifest.ManifestFile;
@@ -62,7 +62,6 @@ import java.util.stream.Collectors;
 import static org.apache.paimon.utils.ManifestReadThreadPool.getExecutorService;
 import static org.apache.paimon.utils.ManifestReadThreadPool.randomlyExecuteSequentialReturn;
 import static org.apache.paimon.utils.ManifestReadThreadPool.sequentialBatchedExecute;
-import static org.apache.paimon.utils.Preconditions.checkArgument;
 import static org.apache.paimon.utils.ThreadPoolUtils.randomlyOnlyExecute;
 
 /** Default implementation of {@link FileStoreScan}. */
@@ -88,7 +87,6 @@ public abstract class AbstractFileStoreScan implements FileStoreScan {
     private Filter<ManifestEntry> manifestEntryFilter = null;
     private Filter<String> fileNameFilter = null;
 
-    private ManifestCacheFilter manifestCacheFilter = null;
     private ScanMetrics scanMetrics = null;
     private boolean dropStats;
 
@@ -162,13 +160,6 @@ public abstract class AbstractFileStoreScan implements FileStoreScan {
 
     @Override
     public FileStoreScan withPartitionBucket(BinaryRow partition, int bucket) {
-        if (manifestCacheFilter != null && manifestFileFactory.isCacheEnabled()) {
-            checkArgument(
-                    manifestCacheFilter.test(partition, bucket),
-                    String.format(
-                            "This is a bug! The partition %s and bucket %s is filtered!",
-                            partition, bucket));
-        }
         withPartitionFilter(Collections.singletonList(partition));
         withBucket(bucket);
         return this;
@@ -205,6 +196,12 @@ public abstract class AbstractFileStoreScan implements FileStoreScan {
         return this;
     }
 
+    @Override
+    public FileStoreScan withLevelMinMaxFilter(BiFilter<Integer, Integer> minMaxFilter) {
+        manifestsReader.withLevelMinMaxFilter(minMaxFilter);
+        return this;
+    }
+
     @Override
     public FileStoreScan enableValueFilter() {
         return this;
@@ -216,12 +213,6 @@ public abstract class AbstractFileStoreScan implements FileStoreScan {
         return this;
     }
 
-    @Override
-    public FileStoreScan withManifestCacheFilter(ManifestCacheFilter manifestFilter) {
-        this.manifestCacheFilter = manifestFilter;
-        return this;
-    }
-
     @Override
     public FileStoreScan withDataFileNameFilter(Filter<String> fileNameFilter) {
         this.fileNameFilter = fileNameFilter;
@@ -240,6 +231,12 @@ public abstract class AbstractFileStoreScan implements FileStoreScan {
         return this;
     }
 
+    @Override
+    public FileStoreScan keepStats() {
+        this.dropStats = false;
+        return this;
+    }
+
     @Nullable
     @Override
     public Integer parallelism() {
@@ -457,7 +454,8 @@ public abstract class AbstractFileStoreScan implements FileStoreScan {
                         .read(
                                 manifest.fileName(),
                                 manifest.fileSize(),
-                                createCacheRowFilter(),
+                                manifestsReader.partitionFilter(),
+                                createBucketFilter(),
                                 createEntryRowFilter().and(additionalFilter),
                                 entry ->
                                         (additionalTFilter == null || additionalTFilter.test(entry))
@@ -478,21 +476,9 @@ public abstract class AbstractFileStoreScan implements FileStoreScan {
         return entry.copyWithoutStats();
     }
 
-    /**
-     * According to the {@link ManifestCacheFilter}, entry that needs to be cached will be retained,
-     * so the entry that will not be accessed in the future will not be cached.
-     *
-     * <p>Implemented to {@link InternalRow} is for performance (No deserialization).
-     */
-    private Filter<InternalRow> createCacheRowFilter() {
-        if (manifestCacheFilter == null) {
-            return Filter.alwaysTrue();
-        }
-
-        Function<InternalRow, BinaryRow> partitionGetter =
-                ManifestEntrySerializer.partitionGetter();
-        Function<InternalRow, Integer> bucketGetter = ManifestEntrySerializer.bucketGetter();
-        return row -> manifestCacheFilter.test(partitionGetter.apply(row), bucketGetter.apply(row));
+    private BucketFilter createBucketFilter() {
+        return BucketFilter.create(
+                onlyReadRealBuckets, specifiedBucket, bucketFilter, totalAwareBucketFilter);
     }
 
     /**
@@ -509,27 +495,18 @@ public abstract class AbstractFileStoreScan implements FileStoreScan {
         Function<InternalRow, String> fileNameGetter = ManifestEntrySerializer.fileNameGetter();
         PartitionPredicate partitionFilter = manifestsReader.partitionFilter();
         Function<InternalRow, Integer> levelGetter = ManifestEntrySerializer.levelGetter();
+        BucketFilter bucketFilter = createBucketFilter();
         return row -> {
             if ((partitionFilter != null && !partitionFilter.test(partitionGetter.apply(row)))) {
                 return false;
             }
 
-            int bucket = bucketGetter.apply(row);
-            if (onlyReadRealBuckets && bucket < 0) {
-                return false;
-            }
-
-            if (specifiedBucket != null && bucket != specifiedBucket) {
-                return false;
-            }
-
-            if (bucketFilter != null && !bucketFilter.test(bucket)) {
-                return false;
-            }
-
-            if (totalAwareBucketFilter != null
-                    && !totalAwareBucketFilter.test(bucket, totalBucketGetter.apply(row))) {
-                return false;
+            if (bucketFilter != null) {
+                int bucket = bucketGetter.apply(row);
+                int totalBucket = totalBucketGetter.apply(row);
+                if (!bucketFilter.test(bucket, totalBucket)) {
+                    return false;
+                }
             }
 
             int level = levelGetter.apply(row);
