@@ -35,11 +35,13 @@ import org.apache.paimon.utils.RowDataToObjectArrayConverter;
 
 import javax.annotation.Nullable;
 
+import java.io.Serializable;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
 import java.util.Set;
 
 import static org.apache.paimon.utils.InternalRowPartitionComputer.convertSpecToInternal;
@@ -47,11 +49,26 @@ import static org.apache.paimon.utils.InternalRowPartitionComputer.convertSpecTo
 import static org.apache.paimon.utils.Preconditions.checkArgument;
 import static org.apache.paimon.utils.Preconditions.checkNotNull;
 
-/** A special predicate to filter partition only, just like {@link Predicate}. */
-public interface PartitionPredicate {
+/**
+ * A special predicate to filter partition only, just like {@link Predicate}.
+ *
+ * @since 1.3.0
+ */
+public interface PartitionPredicate extends Serializable {
 
-    boolean test(BinaryRow part);
+    /**
+     * Test based on the specific partition.
+     *
+     * @return return true when hit, false when not hit.
+     */
+    boolean test(BinaryRow partition);
 
+    /**
+     * Test based on the statistical information to determine whether a hit is possible.
+     *
+     * @return return true is likely to hit (there may also be false positives), return false is
+     *     absolutely not possible to hit.
+     */
     boolean test(
             long rowCount, InternalRow minValues, InternalRow maxValues, InternalArray nullCounts);
 
@@ -68,11 +85,13 @@ public interface PartitionPredicate {
         return new DefaultPartitionPredicate(predicate);
     }
 
+    /** Create {@link PartitionPredicate} from multiple partitions. */
     @Nullable
     static PartitionPredicate fromMultiple(RowType partitionType, List<BinaryRow> partitions) {
         return fromMultiple(partitionType, new HashSet<>(partitions));
     }
 
+    /** Create {@link PartitionPredicate} from multiple partitions. */
     @Nullable
     static PartitionPredicate fromMultiple(RowType partitionType, Set<BinaryRow> partitions) {
         if (partitionType.getFieldCount() == 0 || partitions.isEmpty()) {
@@ -122,6 +141,8 @@ public interface PartitionPredicate {
     /** A {@link PartitionPredicate} using {@link Predicate}. */
     class DefaultPartitionPredicate implements PartitionPredicate {
 
+        private static final long serialVersionUID = 1L;
+
         private final Predicate predicate;
 
         private DefaultPartitionPredicate(Predicate predicate) {
@@ -149,6 +170,8 @@ public interface PartitionPredicate {
      */
     class MultiplePartitionPredicate implements PartitionPredicate {
 
+        private static final long serialVersionUID = 1L;
+
         private final Set<BinaryRow> partitions;
         private final int fieldNum;
         private final Predicate[] min;
@@ -216,6 +239,12 @@ public interface PartitionPredicate {
             }
             return true;
         }
+
+        public Optional<BinaryRow> extractSinglePartition() {
+            return partitions.size() == 1
+                    ? Optional.of(partitions.iterator().next())
+                    : Optional.empty();
+        }
     }
 
     static Predicate createPartitionPredicate(RowType rowType, Map<String, Object> partition) {
@@ -282,4 +311,16 @@ public interface PartitionPredicate {
         }
         return result;
     }
+
+    static PartitionPredicate fromMap(
+            RowType partitionType, Map<String, String> values, String defaultPartValue) {
+        return fromPredicate(
+                partitionType, createPartitionPredicate(values, partitionType, defaultPartValue));
+    }
+
+    static PartitionPredicate fromMaps(
+            RowType partitionType, List<Map<String, String>> values, String defaultPartValue) {
+        return fromMultiple(
+                partitionType, createBinaryPartitions(values, partitionType, defaultPartValue));
+    }
 }
