@@ -35,36 +35,51 @@ import java.util.stream.Collectors;
 /** Utils to fetch partition map information from data schema and row type. */
 public class PartitionUtils {
 
-    public static Pair<Pair<int[], RowType>, List<DataField>> constructPartitionMapping(
+    public static Pair<Pair<int[], RowType>, List<DataField>> trimPartitionFields(
             TableSchema dataSchema, List<DataField> dataFields) {
         if (dataSchema.partitionKeys().isEmpty()) {
             return Pair.of(null, dataFields);
         }
+        return getPartitionMapping2fieldsWithoutPartition(
+                dataSchema.partitionKeys(),
+                dataFields,
+                dataSchema.projectedLogicalRowType(dataSchema.partitionKeys()));
+    }
+
+    public static Pair<int[], RowType> getPartitionMapping(
+            List<String> partitionKeys, List<DataField> dataFields, RowType partitionType) {
+        return getPartitionMapping2fieldsWithoutPartition(partitionKeys, dataFields, partitionType)
+                .getLeft();
+    }
+
+    public static Pair<Pair<int[], RowType>, List<DataField>>
+            getPartitionMapping2fieldsWithoutPartition(
+                    List<String> partitionKeys, List<DataField> dataFields, RowType partitionType) {
+        if (partitionKeys.isEmpty()) {
+            return Pair.of(null, dataFields);
+        }
 
-        List<String> partitionNames = dataSchema.partitionKeys();
         List<DataField> fieldsWithoutPartition = new ArrayList<>();
-
         int[] map = new int[dataFields.size() + 1];
-        int pCount = 0;
+        int partitionFieldCount = 0;
+
         for (int i = 0; i < dataFields.size(); i++) {
             DataField field = dataFields.get(i);
-            if (partitionNames.contains(field.name())) {
+            if (partitionKeys.contains(field.name())) {
                 // if the map[i] is minus, represent the related column is stored in partition row
-                map[i] = -(partitionNames.indexOf(field.name()) + 1);
-                pCount++;
+                map[i] = -(partitionKeys.indexOf(field.name()) + 1);
+                partitionFieldCount++;
             } else {
                 // else if the map[i] is positive, the related column is stored in the file-read row
-                map[i] = (i - pCount) + 1;
-                fieldsWithoutPartition.add(dataFields.get(i));
+                map[i] = (i - partitionFieldCount) + 1;
+                fieldsWithoutPartition.add(field);
             }
         }
 
         Pair<int[], RowType> partitionMapping =
                 fieldsWithoutPartition.size() == dataFields.size()
                         ? null
-                        : Pair.of(
-                                map,
-                                dataSchema.projectedLogicalRowType(dataSchema.partitionKeys()));
+                        : Pair.of(map, partitionType);
         return Pair.of(partitionMapping, fieldsWithoutPartition);
     }
 
