@@ -33,6 +33,7 @@ import org.apache.paimon.io.RollingFileWriter;
 import org.apache.paimon.manifest.FileSource;
 import org.apache.paimon.memory.MemoryOwner;
 import org.apache.paimon.memory.MemorySegmentPool;
+import org.apache.paimon.mergetree.compact.MergeFunction;
 import org.apache.paimon.options.MemorySize;
 import org.apache.paimon.reader.RecordReaderIterator;
 import org.apache.paimon.types.RowType;
@@ -55,6 +56,7 @@ public class PostponeBucketWriter implements RecordWriter<KeyValue>, MemoryOwner
 
     private final FileIO fileIO;
     private final DataFilePathFactory pathFactory;
+    private final MergeFunction<KeyValue> mergeFunction;
     private final KeyValueFileWriterFactory writerFactory;
     private final List<DataFileMeta> files;
     private final IOFunction<List<DataFileMeta>, RecordReaderIterator<KeyValue>> fileRead;
@@ -64,6 +66,7 @@ public class PostponeBucketWriter implements RecordWriter<KeyValue>, MemoryOwner
 
     private SinkWriter<KeyValue> sinkWriter;
     private MemorySegmentPool memorySegmentPool;
+    private boolean retractValidated = false;
 
     public PostponeBucketWriter(
             FileIO fileIO,
@@ -71,12 +74,14 @@ public class PostponeBucketWriter implements RecordWriter<KeyValue>, MemoryOwner
             CompressOptions spillCompression,
             MemorySize maxDiskSize,
             @Nullable IOManager ioManager,
+            MergeFunction<KeyValue> mergeFunction,
             KeyValueFileWriterFactory writerFactory,
             IOFunction<List<DataFileMeta>, RecordReaderIterator<KeyValue>> fileRead,
             boolean useWriteBuffer,
             boolean spillable,
             @Nullable CommitIncrement restoreIncrement) {
         this.ioManager = ioManager;
+        this.mergeFunction = mergeFunction;
         this.writerFactory = writerFactory;
         this.fileRead = fileRead;
         this.fileIO = fileIO;
@@ -99,6 +104,7 @@ public class PostponeBucketWriter implements RecordWriter<KeyValue>, MemoryOwner
 
     @Override
     public void write(KeyValue record) throws Exception {
+        validateRetract(record);
         boolean success = sinkWriter.write(record);
         if (!success) {
             flush();
@@ -112,6 +118,18 @@ public class PostponeBucketWriter implements RecordWriter<KeyValue>, MemoryOwner
         }
     }
 
+    private void validateRetract(KeyValue kv) {
+        if (kv.valueKind().isRetract()) {
+            if (retractValidated) {
+                return;
+            }
+            mergeFunction.reset();
+            mergeFunction.add(kv);
+            mergeFunction.getResult();
+            retractValidated = true;
+        }
+    }
+
     private void flush() throws Exception {
         files.addAll(sinkWriter.flush());
     }
