@@ -51,9 +51,11 @@ import org.apache.paimon.utils.Preconditions;
 import org.apache.paimon.utils.SnapshotManager;
 import org.apache.paimon.utils.StringUtils;
 
-import org.apache.paimon.shade.guava30.com.google.common.base.Joiner;
+import org.apache.paimon.shade.guava30.com.google.common.collect.FluentIterable;
+import org.apache.paimon.shade.guava30.com.google.common.collect.ImmutableList;
 import org.apache.paimon.shade.guava30.com.google.common.collect.Iterables;
 import org.apache.paimon.shade.guava30.com.google.common.collect.Maps;
+import org.apache.paimon.shade.guava30.com.google.common.collect.Streams;
 
 import javax.annotation.Nullable;
 import javax.annotation.concurrent.ThreadSafe;
@@ -64,7 +66,6 @@ import java.io.Serializable;
 import java.io.UncheckedIOException;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Comparator;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
@@ -72,14 +73,23 @@ import java.util.Objects;
 import java.util.Optional;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.function.BiFunction;
+import java.util.function.Function;
 import java.util.function.Predicate;
 import java.util.stream.Collectors;
-import java.util.stream.LongStream;
 
+import static org.apache.paimon.CoreOptions.AGG_FUNCTION;
 import static org.apache.paimon.CoreOptions.BUCKET_KEY;
+import static org.apache.paimon.CoreOptions.DISTINCT;
+import static org.apache.paimon.CoreOptions.FIELDS_PREFIX;
+import static org.apache.paimon.CoreOptions.IGNORE_RETRACT;
+import static org.apache.paimon.CoreOptions.LIST_AGG_DELIMITER;
+import static org.apache.paimon.CoreOptions.NESTED_KEY;
+import static org.apache.paimon.CoreOptions.SEQUENCE_FIELD;
 import static org.apache.paimon.catalog.AbstractCatalog.DB_SUFFIX;
 import static org.apache.paimon.catalog.Identifier.DEFAULT_MAIN_BRANCH;
 import static org.apache.paimon.catalog.Identifier.UNKNOWN_DATABASE;
+import static org.apache.paimon.mergetree.compact.PartialUpdateMergeFunction.SEQUENCE_GROUP;
+import static org.apache.paimon.utils.DefaultValueUtils.validateDefaultValue;
 import static org.apache.paimon.utils.FileUtils.listVersionedFiles;
 import static org.apache.paimon.utils.Preconditions.checkArgument;
 import static org.apache.paimon.utils.Preconditions.checkState;
@@ -146,53 +156,6 @@ public class SchemaManager implements Serializable {
         return listAllIds().stream().map(this::schema).collect(Collectors.toList());
     }
 
-    public List<TableSchema> schemasWithId(List<Long> schemaIds) {
-        return schemaIds.stream().map(this::schema).collect(Collectors.toList());
-    }
-
-    public List<TableSchema> listWithRange(
-            Optional<Long> optionalMaxSchemaId, Optional<Long> optionalMinSchemaId) {
-        Long lowerBoundSchemaId = 0L;
-        Long upperBoundSchematId = latest().get().id();
-
-        // null check on optionalMaxSchemaId & optionalMinSchemaId return all schemas
-        if (!optionalMaxSchemaId.isPresent() && !optionalMinSchemaId.isPresent()) {
-            return listAll();
-        }
-
-        if (optionalMaxSchemaId.isPresent()) {
-            if (optionalMaxSchemaId.get() < lowerBoundSchemaId) {
-                throw new RuntimeException(
-                        String.format(
-                                "schema id: %s should not lower than min schema id: %s",
-                                optionalMaxSchemaId.get(), lowerBoundSchemaId));
-            }
-            upperBoundSchematId =
-                    optionalMaxSchemaId.get() > upperBoundSchematId
-                            ? upperBoundSchematId
-                            : optionalMaxSchemaId.get();
-        }
-
-        if (optionalMinSchemaId.isPresent()) {
-            if (optionalMinSchemaId.get() > upperBoundSchematId) {
-                throw new RuntimeException(
-                        String.format(
-                                "schema id: %s should not greater than max schema id: %s",
-                                optionalMinSchemaId.get(), upperBoundSchematId));
-            }
-            lowerBoundSchemaId =
-                    optionalMinSchemaId.get() > lowerBoundSchemaId
-                            ? optionalMinSchemaId.get()
-                            : lowerBoundSchemaId;
-        }
-
-        // +1 here to include the upperBoundSchemaId
-        return LongStream.range(lowerBoundSchemaId, upperBoundSchematId + 1)
-                .mapToObj(this::schema)
-                .sorted(Comparator.comparingLong(TableSchema::id))
-                .collect(Collectors.toList());
-    }
-
     /** List all schema IDs. */
     public List<Long> listAllIds() {
         try {
@@ -289,7 +252,10 @@ public class SchemaManager implements Serializable {
                                             new Catalog.TableNotExistException(
                                                     identifierFromPath(
                                                             tableRoot.toString(), true, branch)));
-            TableSchema newTableSchema = generateTableSchema(oldTableSchema, changes, hasSnapshots);
+            LazyField<Identifier> lazyIdentifier =
+                    new LazyField<>(() -> identifierFromPath(tableRoot.toString(), true, branch));
+            TableSchema newTableSchema =
+                    generateTableSchema(oldTableSchema, changes, hasSnapshots, lazyIdentifier);
             try {
                 boolean success = commit(newTableSchema);
                 if (success) {
@@ -301,8 +267,11 @@ public class SchemaManager implements Serializable {
         }
     }
 
-    public TableSchema generateTableSchema(
-            TableSchema oldTableSchema, List<SchemaChange> changes, LazyField<Boolean> hasSnapshots)
+    public static TableSchema generateTableSchema(
+            TableSchema oldTableSchema,
+            List<SchemaChange> changes,
+            LazyField<Boolean> hasSnapshots,
+            LazyField<Identifier> lazyIdentifier)
             throws Catalog.ColumnAlreadyExistException, Catalog.ColumnNotExistException {
         Map<String, String> oldOptions = new HashMap<>(oldTableSchema.options());
         Map<String, String> newOptions = new HashMap<>(oldTableSchema.options());
@@ -351,16 +320,16 @@ public class SchemaManager implements Serializable {
                         addColumn.dataType().isNullable(),
                         "Column %s cannot specify NOT NULL in the %s table.",
                         String.join(".", addColumn.fieldNames()),
-                        identifierFromPath(tableRoot.toString(), true, branch).getFullName());
+                        lazyIdentifier.get().getFullName());
                 int id = highestFieldId.incrementAndGet();
                 DataType dataType = ReassignFieldId.reassign(addColumn.dataType(), highestFieldId);
-
-                new NestedColumnModifier(addColumn.fieldNames()) {
+                new NestedColumnModifier(addColumn.fieldNames(), lazyIdentifier) {
                     @Override
                     protected void updateLastColumn(
                             int depth, List<DataField> newFields, String fieldName)
-                            throws Catalog.ColumnAlreadyExistException {
-                        assertColumnNotExists(newFields, fieldName);
+                            throws Catalog.ColumnAlreadyExistException,
+                                    Catalog.ColumnNotExistException {
+                        assertColumnNotExists(newFields, fieldName, lazyIdentifier);
 
                         DataField dataField =
                                 new DataField(id, fieldName, dataType, addColumn.description());
@@ -375,8 +344,26 @@ public class SchemaManager implements Serializable {
                             if (move.type().equals(SchemaChange.Move.MoveType.FIRST)) {
                                 newFields.add(0, dataField);
                             } else if (move.type().equals(SchemaChange.Move.MoveType.AFTER)) {
-                                int fieldIndex = map.get(move.referenceFieldName());
-                                newFields.add(fieldIndex + 1, dataField);
+                                if (map.containsKey(move.referenceFieldName())) {
+                                    int fieldIndex = map.get(move.referenceFieldName());
+                                    newFields.add(fieldIndex + 1, dataField);
+                                } else {
+                                    throw new Catalog.ColumnNotExistException(
+                                            lazyIdentifier.get(), move.referenceFieldName());
+                                }
+                            } else if (move.type().equals(SchemaChange.Move.MoveType.BEFORE)) {
+                                if (map.containsKey(move.referenceFieldName())) {
+                                    int fieldIndex = map.get(move.referenceFieldName());
+                                    newFields.add(fieldIndex, dataField);
+                                } else {
+                                    throw new Catalog.ColumnNotExistException(
+                                            lazyIdentifier.get(), move.referenceFieldName());
+                                }
+                            } else if (move.type().equals(SchemaChange.Move.MoveType.LAST)) {
+                                newFields.add(dataField);
+                            } else {
+                                throw new UnsupportedOperationException(
+                                        "Unsupported move type: " + move.type());
                             }
                         } else {
                             newFields.add(dataField);
@@ -385,15 +372,15 @@ public class SchemaManager implements Serializable {
                 }.updateIntermediateColumn(newFields, 0);
             } else if (change instanceof RenameColumn) {
                 RenameColumn rename = (RenameColumn) change;
-                assertNotUpdatingPrimaryKeys(oldTableSchema, rename.fieldNames(), "rename");
-                new NestedColumnModifier(rename.fieldNames()) {
+                assertNotUpdatingPartitionKeys(oldTableSchema, rename.fieldNames(), "rename");
+                new NestedColumnModifier(rename.fieldNames(), lazyIdentifier) {
                     @Override
                     protected void updateLastColumn(
                             int depth, List<DataField> newFields, String fieldName)
                             throws Catalog.ColumnNotExistException,
                                     Catalog.ColumnAlreadyExistException {
-                        assertColumnExists(newFields, fieldName);
-                        assertColumnNotExists(newFields, rename.newName());
+                        assertColumnExists(newFields, fieldName, lazyIdentifier);
+                        assertColumnNotExists(newFields, rename.newName(), lazyIdentifier);
                         for (int i = 0; i < newFields.size(); i++) {
                             DataField field = newFields.get(i);
                             if (!field.name().equals(fieldName)) {
@@ -415,12 +402,12 @@ public class SchemaManager implements Serializable {
             } else if (change instanceof DropColumn) {
                 DropColumn drop = (DropColumn) change;
                 dropColumnValidation(oldTableSchema, drop);
-                new NestedColumnModifier(drop.fieldNames()) {
+                new NestedColumnModifier(drop.fieldNames(), lazyIdentifier) {
                     @Override
                     protected void updateLastColumn(
                             int depth, List<DataField> newFields, String fieldName)
                             throws Catalog.ColumnNotExistException {
-                        assertColumnExists(newFields, fieldName);
+                        assertColumnExists(newFields, fieldName, lazyIdentifier);
                         newFields.removeIf(f -> f.name().equals(fieldName));
                         if (newFields.isEmpty()) {
                             throw new IllegalArgumentException("Cannot drop all fields in table");
@@ -429,6 +416,7 @@ public class SchemaManager implements Serializable {
                 }.updateIntermediateColumn(newFields, 0);
             } else if (change instanceof UpdateColumnType) {
                 UpdateColumnType update = (UpdateColumnType) change;
+                assertNotUpdatingPartitionKeys(oldTableSchema, update.fieldNames(), "update");
                 assertNotUpdatingPrimaryKeys(oldTableSchema, update.fieldNames(), "update");
                 updateNestedColumn(
                         newFields,
@@ -467,14 +455,13 @@ public class SchemaManager implements Serializable {
                                             update.fieldNames().length),
                                     field.description(),
                                     field.defaultValue());
-                        });
+                        },
+                        lazyIdentifier);
             } else if (change instanceof UpdateColumnNullability) {
                 UpdateColumnNullability update = (UpdateColumnNullability) change;
-                if (update.fieldNames().length == 1
-                        && update.newNullability()
-                        && oldTableSchema.primaryKeys().contains(update.fieldNames()[0])) {
-                    throw new UnsupportedOperationException(
-                            "Cannot change nullability of primary key");
+                if (update.newNullability()) {
+                    assertNotUpdatingPrimaryKeys(
+                            oldTableSchema, update.fieldNames(), "change nullability of");
                 }
                 updateNestedColumn(
                         newFields,
@@ -499,7 +486,8 @@ public class SchemaManager implements Serializable {
                                             update.fieldNames().length),
                                     field.description(),
                                     field.defaultValue());
-                        });
+                        },
+                        lazyIdentifier);
             } else if (change instanceof UpdateColumnComment) {
                 UpdateColumnComment update = (UpdateColumnComment) change;
                 updateNestedColumn(
@@ -511,7 +499,8 @@ public class SchemaManager implements Serializable {
                                         field.name(),
                                         field.type(),
                                         update.newDescription(),
-                                        field.defaultValue()));
+                                        field.defaultValue()),
+                        lazyIdentifier);
             } else if (change instanceof UpdateColumnPosition) {
                 UpdateColumnPosition update = (UpdateColumnPosition) change;
                 SchemaChange.Move move = update.move();
@@ -521,13 +510,16 @@ public class SchemaManager implements Serializable {
                 updateNestedColumn(
                         newFields,
                         update.fieldNames(),
-                        (field, depth) ->
-                                new DataField(
-                                        field.id(),
-                                        field.name(),
-                                        field.type(),
-                                        field.description(),
-                                        update.newDefaultValue()));
+                        (field, depth) -> {
+                            validateDefaultValue(field.type(), update.newDefaultValue());
+                            return new DataField(
+                                    field.id(),
+                                    field.name(),
+                                    field.type(),
+                                    field.description(),
+                                    update.newDefaultValue());
+                        },
+                        lazyIdentifier);
             } else {
                 throw new UnsupportedOperationException("Unsupported change: " + change.getClass());
             }
@@ -542,7 +534,7 @@ public class SchemaManager implements Serializable {
                         applyNotNestedColumnRename(
                                 oldTableSchema.primaryKeys(),
                                 Iterables.filter(changes, RenameColumn.class)),
-                        applySchemaChanges(newOptions, changes),
+                        applyRenameColumnsToOptions(newOptions, changes),
                         newComment);
 
         return new TableSchema(
@@ -561,7 +553,7 @@ public class SchemaManager implements Serializable {
     // the maxDepth will be based on updateFieldNames
     // which in the case will be [v, element, value, element],
     // so maxDepth is 4 and return DataType will be INT
-    private DataType getRootType(DataType type, int currDepth, int maxDepth) {
+    private static DataType getRootType(DataType type, int currDepth, int maxDepth) {
         if (currDepth == maxDepth - 1) {
             return type;
         }
@@ -579,7 +571,7 @@ public class SchemaManager implements Serializable {
     // ex: ARRAY<MAP<STRING, ARRAY<INT>>> -> ARRAY<MAP<STRING, ARRAY<BIGINT>>>
     // here we only need to update type of ARRAY<INT> to ARRAY<BIGINT> and rest of the type
     // remains same. This function achieves this.
-    private DataType getArrayMapTypeWithTargetTypeRoot(
+    private static DataType getArrayMapTypeWithTargetTypeRoot(
             DataType source, DataType target, int currDepth, int maxDepth) {
         if (currDepth == maxDepth - 1) {
             return target;
@@ -607,7 +599,7 @@ public class SchemaManager implements Serializable {
         }
     }
 
-    private void assertNullabilityChange(
+    private static void assertNullabilityChange(
             boolean oldNullability,
             boolean newNullability,
             String fieldName,
@@ -622,7 +614,7 @@ public class SchemaManager implements Serializable {
         }
     }
 
-    public void applyMove(List<DataField> newFields, SchemaChange.Move move) {
+    public static void applyMove(List<DataField> newFields, SchemaChange.Move move) {
         Map<String, Integer> map = new HashMap<>();
         for (int i = 0; i < newFields.size(); i++) {
             map.put(newFields.get(i).name(), i);
@@ -671,7 +663,7 @@ public class SchemaManager implements Serializable {
     }
 
     // Utility method to move a field within the list, handling range checks
-    private void moveField(List<DataField> newFields, int fromIndex, int toIndex) {
+    private static void moveField(List<DataField> newFields, int fromIndex, int toIndex) {
         if (fromIndex < 0 || fromIndex >= newFields.size() || toIndex < 0) {
             return;
         }
@@ -704,19 +696,106 @@ public class SchemaManager implements Serializable {
         }
     }
 
-    private static Map<String, String> applySchemaChanges(
+    private static Map<String, String> applyRenameColumnsToOptions(
             Map<String, String> options, Iterable<SchemaChange> changes) {
+        Iterable<RenameColumn> renameColumns =
+                FluentIterable.from(changes).filter(RenameColumn.class);
+
+        if (Iterables.isEmpty(renameColumns)) {
+            return options;
+        }
+
         Map<String, String> newOptions = Maps.newHashMap(options);
+
+        Map<String, String> renameMappings =
+                Streams.stream(renameColumns)
+                        .collect(
+                                Collectors.toMap(
+                                        // currently only non-nested columns are supported
+                                        rename -> rename.fieldNames()[0],
+                                        RenameColumn::newName));
+
+        // case 1: the option key is fixed and only value may contain field names
+
+        // bucket key rename
         String bucketKeysStr = options.get(BUCKET_KEY.key());
         if (!StringUtils.isNullOrWhitespaceOnly(bucketKeysStr)) {
             List<String> bucketColumns = Arrays.asList(bucketKeysStr.split(","));
             List<String> newBucketColumns =
-                    applyNotNestedColumnRename(
-                            bucketColumns, Iterables.filter(changes, RenameColumn.class));
-            newOptions.put(BUCKET_KEY.key(), Joiner.on(',').join(newBucketColumns));
+                    applyNotNestedColumnRename(bucketColumns, renameMappings);
+            newOptions.put(BUCKET_KEY.key(), String.join(",", newBucketColumns));
+        }
+
+        // sequence field rename
+        String sequenceFieldsStr = options.get(SEQUENCE_FIELD.key());
+        if (!StringUtils.isNullOrWhitespaceOnly(sequenceFieldsStr)) {
+            List<String> sequenceFields = Arrays.asList(sequenceFieldsStr.split(","));
+            List<String> newSequenceFields =
+                    applyNotNestedColumnRename(sequenceFields, renameMappings);
+            newOptions.put(SEQUENCE_FIELD.key(), String.join(",", newSequenceFields));
+        }
+
+        // case 2: the option key is composed of certain fixed prefixes, suffixes, and the field
+        // name, while the option value doesn't contain field names.
+        List<Function<String, String>> fieldNameToOptionKeys =
+                ImmutableList.of(
+                        // NESTED_KEY is not added since renaming nested columns is not supported
+                        // currently
+                        fieldName -> FIELDS_PREFIX + "." + fieldName + "." + AGG_FUNCTION,
+                        fieldName -> FIELDS_PREFIX + "." + fieldName + "." + IGNORE_RETRACT,
+                        fieldName -> FIELDS_PREFIX + "." + fieldName + "." + DISTINCT,
+                        fieldName -> FIELDS_PREFIX + "." + fieldName + "." + LIST_AGG_DELIMITER);
+
+        for (RenameColumn rename : renameColumns) {
+            String fieldName = rename.fieldNames()[0];
+            String newFieldName = rename.newName();
+
+            for (Function<String, String> fieldNameToKey : fieldNameToOptionKeys) {
+                String key = fieldNameToKey.apply(fieldName);
+                if (newOptions.containsKey(key)) {
+                    String value = newOptions.remove(key);
+                    newOptions.put(fieldNameToKey.apply(newFieldName), value);
+                }
+            }
+        }
+
+        // case 3: both option key and option value may contain field names
+        for (String key : options.keySet()) {
+            if (key.startsWith(FIELDS_PREFIX)) {
+                String matchedSuffix = null;
+                if (key.endsWith(SEQUENCE_GROUP)) {
+                    matchedSuffix = SEQUENCE_GROUP;
+                } else if (key.endsWith(NESTED_KEY)) {
+                    matchedSuffix = NESTED_KEY;
+                }
+
+                if (matchedSuffix != null) {
+                    // Both the key and value may contain field names. If we were to perform a
+                    // "match then replace" operation, the conditions would become quite complex.
+                    // Instead, we directly make a replacement across all instances
+                    String keyFieldsStr =
+                            key.substring(
+                                    FIELDS_PREFIX.length() + 1,
+                                    key.length() - matchedSuffix.length() - 1);
+                    List<String> keyFields = Arrays.asList(keyFieldsStr.split(","));
+                    List<String> newKeyFields =
+                            applyNotNestedColumnRename(keyFields, renameMappings);
+
+                    String valueFieldsStr = newOptions.remove(key);
+                    List<String> valueFields = Arrays.asList(valueFieldsStr.split(","));
+                    List<String> newValueFields =
+                            applyNotNestedColumnRename(valueFields, renameMappings);
+                    newOptions.put(
+                            FIELDS_PREFIX
+                                    + "."
+                                    + String.join(",", newKeyFields)
+                                    + "."
+                                    + matchedSuffix,
+                            String.join(",", newValueFields));
+                }
+            }
         }
 
-        // TODO: Apply changes to other options that contain column names, such as `sequence.field`
         return newOptions;
     }
 
@@ -734,10 +813,15 @@ public class SchemaManager implements Serializable {
                 columnNames.put(renameColumn.fieldNames()[0], renameColumn.newName());
             }
         }
+        return applyNotNestedColumnRename(columns, columnNames);
+    }
+
+    private static List<String> applyNotNestedColumnRename(
+            List<String> columns, Map<String, String> renameMapping) {
 
         // The order of the column names will be preserved, as a non-parallel stream is used here.
         return columns.stream()
-                .map(column -> columnNames.getOrDefault(column, column))
+                .map(column -> renameMapping.getOrDefault(column, column))
                 .collect(Collectors.toList());
     }
 
@@ -754,26 +838,40 @@ public class SchemaManager implements Serializable {
         }
     }
 
-    private static void assertNotUpdatingPrimaryKeys(
+    private static void assertNotUpdatingPartitionKeys(
             TableSchema schema, String[] fieldNames, String operation) {
         // partition keys can't be nested columns
         if (fieldNames.length > 1) {
             return;
         }
-        String columnToRename = fieldNames[0];
-        if (schema.partitionKeys().contains(columnToRename)) {
+        String fieldName = fieldNames[0];
+        if (schema.partitionKeys().contains(fieldName)) {
             throw new UnsupportedOperationException(
-                    String.format(
-                            "Cannot " + operation + " partition column: [%s]", columnToRename));
+                    String.format("Cannot %s partition column: [%s]", operation, fieldName));
         }
     }
 
-    private abstract class NestedColumnModifier {
+    private static void assertNotUpdatingPrimaryKeys(
+            TableSchema schema, String[] fieldNames, String operation) {
+        // primary keys can't be nested columns
+        if (fieldNames.length > 1) {
+            return;
+        }
+        String fieldName = fieldNames[0];
+        if (schema.primaryKeys().contains(fieldName)) {
+            throw new UnsupportedOperationException(
+                    String.format("Cannot %s primary key", operation));
+        }
+    }
+
+    private abstract static class NestedColumnModifier {
 
         private final String[] updateFieldNames;
+        private final LazyField<Identifier> identifier;
 
-        private NestedColumnModifier(String[] updateFieldNames) {
+        private NestedColumnModifier(String[] updateFieldNames, LazyField<Identifier> identifier) {
             this.updateFieldNames = updateFieldNames;
+            this.identifier = identifier;
         }
 
         private void updateIntermediateColumn(
@@ -816,7 +914,7 @@ public class SchemaManager implements Serializable {
             }
 
             throw new Catalog.ColumnNotExistException(
-                    identifierFromPath(tableRoot.toString(), true, branch),
+                    identifier.get(),
                     String.join(".", Arrays.asList(updateFieldNames).subList(0, depth + 1)));
         }
 
@@ -863,7 +961,8 @@ public class SchemaManager implements Serializable {
                 int depth, List<DataField> newFields, String fieldName)
                 throws Catalog.ColumnNotExistException, Catalog.ColumnAlreadyExistException;
 
-        protected void assertColumnExists(List<DataField> newFields, String fieldName)
+        protected void assertColumnExists(
+                List<DataField> newFields, String fieldName, LazyField<Identifier> lazyIdentifier)
                 throws Catalog.ColumnNotExistException {
             for (DataField field : newFields) {
                 if (field.name().equals(fieldName)) {
@@ -871,17 +970,16 @@ public class SchemaManager implements Serializable {
                 }
             }
             throw new Catalog.ColumnNotExistException(
-                    identifierFromPath(tableRoot.toString(), true, branch),
-                    getLastFieldName(fieldName));
+                    lazyIdentifier.get(), getLastFieldName(fieldName));
         }
 
-        protected void assertColumnNotExists(List<DataField> newFields, String fieldName)
+        protected void assertColumnNotExists(
+                List<DataField> newFields, String fieldName, LazyField<Identifier> lazyIdentifier)
                 throws Catalog.ColumnAlreadyExistException {
             for (DataField field : newFields) {
                 if (field.name().equals(fieldName)) {
                     throw new Catalog.ColumnAlreadyExistException(
-                            identifierFromPath(tableRoot.toString(), true, branch),
-                            getLastFieldName(fieldName));
+                            lazyIdentifier.get(), getLastFieldName(fieldName));
                 }
             }
         }
@@ -896,12 +994,13 @@ public class SchemaManager implements Serializable {
         }
     }
 
-    private void updateNestedColumn(
+    private static void updateNestedColumn(
             List<DataField> newFields,
             String[] updateFieldNames,
-            BiFunction<DataField, Integer, DataField> updateFunc)
+            BiFunction<DataField, Integer, DataField> updateFunc,
+            LazyField<Identifier> lazyIdentifier)
             throws Catalog.ColumnNotExistException, Catalog.ColumnAlreadyExistException {
-        new NestedColumnModifier(updateFieldNames) {
+        new NestedColumnModifier(updateFieldNames, lazyIdentifier) {
             @Override
             protected void updateLastColumn(int depth, List<DataField> newFields, String fieldName)
                     throws Catalog.ColumnNotExistException {
@@ -916,8 +1015,7 @@ public class SchemaManager implements Serializable {
                 }
 
                 throw new Catalog.ColumnNotExistException(
-                        identifierFromPath(tableRoot.toString(), true, branch),
-                        String.join(".", updateFieldNames));
+                        lazyIdentifier.get(), String.join(".", updateFieldNames));
             }
         }.updateIntermediateColumn(newFields, 0);
     }
