@@ -27,7 +27,6 @@ import org.apache.paimon.fs.Path;
 import org.apache.paimon.manifest.IndexManifestEntry;
 import org.apache.paimon.manifest.ManifestEntry;
 import org.apache.paimon.manifest.ManifestFileMeta;
-import org.apache.paimon.operation.DefaultValueAssigner;
 import org.apache.paimon.operation.FileStoreScan;
 import org.apache.paimon.options.ExpireConfig;
 import org.apache.paimon.options.Options;
@@ -52,6 +51,7 @@ import org.apache.paimon.table.source.StreamDataTableScan;
 import org.apache.paimon.table.source.snapshot.SnapshotReader;
 import org.apache.paimon.table.source.snapshot.SnapshotReaderImpl;
 import org.apache.paimon.table.source.snapshot.TimeTravelUtil;
+import org.apache.paimon.tag.TagAutoManager;
 import org.apache.paimon.utils.BranchManager;
 import org.apache.paimon.utils.CatalogBranchManager;
 import org.apache.paimon.utils.ChangelogManager;
@@ -121,6 +121,12 @@ abstract class AbstractFileStoreTable implements FileStoreTable {
         store().setManifestCache(manifestCache);
     }
 
+    @Nullable
+    @Override
+    public SegmentsCache<Path> getManifestCache() {
+        return manifestCache;
+    }
+
     @Override
     public void setSnapshotCache(Cache<Path, Snapshot> cache) {
         this.snapshotCache = cache;
@@ -232,7 +238,7 @@ abstract class AbstractFileStoreTable implements FileStoreTable {
             case HASH_FIXED:
                 return new FixedBucketRowKeyExtractor(schema());
             case HASH_DYNAMIC:
-            case CROSS_PARTITION:
+            case KEY_DYNAMIC:
                 return new DynamicBucketRowKeyExtractor(schema());
             case BUCKET_UNAWARE:
                 return new AppendTableRowKeyExtractor(schema());
@@ -253,7 +259,6 @@ abstract class AbstractFileStoreTable implements FileStoreTable {
                 changelogManager(),
                 splitGenerator(),
                 nonPartitionFilterConsumer(),
-                DefaultValueAssigner.create(tableSchema),
                 store().pathFactory(),
                 name(),
                 store().newIndexFileHandler());
@@ -263,6 +268,7 @@ abstract class AbstractFileStoreTable implements FileStoreTable {
     public DataTableBatchScan newScan() {
         return new DataTableBatchScan(
                 tableSchema,
+                schemaManager(),
                 coreOptions(),
                 newSnapshotReader(),
                 catalogEnvironment.tableQueryAuth(coreOptions()));
@@ -444,12 +450,13 @@ abstract class AbstractFileStoreTable implements FileStoreTable {
                 store().newCommit(commitUser, this),
                 newExpireRunnable(),
                 options.writeOnly() ? null : store().newPartitionExpire(commitUser, this),
-                options.writeOnly() ? null : store().newTagCreationManager(this),
+                options.writeOnly() ? null : store().newTagAutoManager(this),
                 CoreOptions.fromMap(options()).consumerExpireTime(),
                 new ConsumerManager(fileIO, path, snapshotManager().branch()),
                 options.snapshotExpireExecutionMode(),
                 name(),
-                options.forceCreatingSnapshot());
+                options.forceCreatingSnapshot(),
+                options.fileOperationThreadNum());
     }
 
     @Override
@@ -558,6 +565,11 @@ abstract class AbstractFileStoreTable implements FileStoreTable {
         return snapshot;
     }
 
+    @Override
+    public TagAutoManager newTagAutoManager() {
+        return store().newTagAutoManager(this);
+    }
+
     @Override
     public void createTag(String tagName, long fromSnapshotId) {
         createTag(tagName, findSnapshot(fromSnapshotId), coreOptions().tagDefaultTimeRetained());
@@ -666,7 +678,7 @@ abstract class AbstractFileStoreTable implements FileStoreTable {
 
     @Override
     public TagManager tagManager() {
-        return new TagManager(fileIO, path, currentBranch());
+        return new TagManager(fileIO, path, currentBranch(), coreOptions());
     }
 
     @Override
