@@ -22,6 +22,8 @@ import org.apache.paimon.Snapshot;
 import org.apache.paimon.manifest.IndexManifestEntry;
 import org.apache.paimon.manifest.ManifestCommittable;
 import org.apache.paimon.manifest.ManifestEntry;
+import org.apache.paimon.manifest.SimpleFileEntry;
+import org.apache.paimon.table.FileStoreTable;
 
 import java.util.List;
 
@@ -39,9 +41,12 @@ import java.util.List;
 public interface CommitCallback extends AutoCloseable {
 
     void call(
-            List<ManifestEntry> committedEntries,
+            List<SimpleFileEntry> baseFiles,
+            List<ManifestEntry> deltaFiles,
             List<IndexManifestEntry> indexFiles,
             Snapshot snapshot);
 
     void retry(ManifestCommittable committable);
+
+    default void setTable(FileStoreTable table) {}
 }
