@@ -22,10 +22,10 @@ import org.apache.paimon.CoreOptions;
 import org.apache.paimon.CoreOptions.StreamScanMode;
 import org.apache.paimon.Snapshot;
 import org.apache.paimon.consumer.Consumer;
-import org.apache.paimon.lookup.LookupStrategy;
 import org.apache.paimon.manifest.PartitionEntry;
-import org.apache.paimon.operation.DefaultValueAssigner;
 import org.apache.paimon.predicate.Predicate;
+import org.apache.paimon.schema.TableSchema;
+import org.apache.paimon.table.BucketMode;
 import org.apache.paimon.table.source.snapshot.AllDeltaFollowUpScanner;
 import org.apache.paimon.table.source.snapshot.BoundedChecker;
 import org.apache.paimon.table.source.snapshot.ChangelogFollowUpScanner;
@@ -49,6 +49,7 @@ import javax.annotation.Nullable;
 import java.util.List;
 
 import static org.apache.paimon.CoreOptions.ChangelogProducer.FULL_COMPACTION;
+import static org.apache.paimon.CoreOptions.ChangelogProducer.LOOKUP;
 import static org.apache.paimon.CoreOptions.StreamScanMode.FILE_MONITOR;
 
 /** {@link StreamTableScan} implementation for streaming planning. */
@@ -60,8 +61,8 @@ public class DataTableStreamScan extends AbstractDataTableScan implements Stream
     private final StreamScanMode scanMode;
     private final SnapshotManager snapshotManager;
     private final boolean supportStreamingReadOverwrite;
-    private final DefaultValueAssigner defaultValueAssigner;
     private final NextSnapshotFetcher nextSnapshotProvider;
+    private final boolean hasPk;
 
     private boolean initialized = false;
     private StartingScanner startingScanner;
@@ -75,26 +76,35 @@ public class DataTableStreamScan extends AbstractDataTableScan implements Stream
     @Nullable private Long scanDelayMillis;
 
     public DataTableStreamScan(
+            TableSchema schema,
             CoreOptions options,
             SnapshotReader snapshotReader,
             SnapshotManager snapshotManager,
             ChangelogManager changelogManager,
             boolean supportStreamingReadOverwrite,
-            DefaultValueAssigner defaultValueAssigner) {
-        super(options, snapshotReader);
+            TableQueryAuth queryAuth,
+            boolean hasPk) {
+        super(schema, options, snapshotReader, queryAuth);
+
         this.options = options;
         this.scanMode = options.toConfiguration().get(CoreOptions.STREAM_SCAN_MODE);
         this.snapshotManager = snapshotManager;
         this.supportStreamingReadOverwrite = supportStreamingReadOverwrite;
-        this.defaultValueAssigner = defaultValueAssigner;
         this.nextSnapshotProvider =
                 new NextSnapshotFetcher(
                         snapshotManager, changelogManager, options.changelogLifecycleDecoupled());
+        this.hasPk = hasPk;
+
+        if (options.bucket() == BucketMode.POSTPONE_BUCKET
+                && options.changelogProducer() != CoreOptions.ChangelogProducer.NONE) {
+            snapshotReader.onlyReadRealBuckets();
+        }
     }
 
     @Override
     public DataTableStreamScan withFilter(Predicate predicate) {
-        snapshotReader.withFilter(defaultValueAssigner.handlePredicate(predicate));
+        super.withFilter(predicate);
+        snapshotReader.withFilter(predicate);
         return this;
     }
 
@@ -108,6 +118,8 @@ public class DataTableStreamScan extends AbstractDataTableScan implements Stream
 
     @Override
     public Plan plan() {
+        authQuery();
+
         if (!initialized) {
             initScanner();
         }
@@ -145,7 +157,8 @@ public class DataTableStreamScan extends AbstractDataTableScan implements Stream
         StartingScanner.Result result;
         if (scanMode == FILE_MONITOR) {
             result = startingScanner.scan(snapshotReader);
-        } else if (options.needLookup()) {
+        } else if (options.changelogProducer().equals(LOOKUP)) {
+            // level0 data will be compacted to produce changelog in the future
             result = startingScanner.scan(snapshotReader.withLevelFilter(level -> level > 0));
             snapshotReader.withLevelFilter(Filter.alwaysTrue());
         } else if (options.changelogProducer().equals(FULL_COMPACTION)) {
@@ -162,16 +175,7 @@ public class DataTableStreamScan extends AbstractDataTableScan implements Stream
             ScannedResult scannedResult = (ScannedResult) result;
             currentWatermark = scannedResult.currentWatermark();
             long currentSnapshotId = scannedResult.currentSnapshotId();
-            LookupStrategy lookupStrategy = options.lookupStrategy();
-            if (scanMode == FILE_MONITOR) {
-                nextSnapshotId = currentSnapshotId + 1;
-            } else if (!lookupStrategy.produceChangelog && lookupStrategy.deletionVector) {
-                // For DELETION_VECTOR_ONLY mode, we need to return the remaining data from level 0
-                // in the subsequent plan.
-                nextSnapshotId = currentSnapshotId;
-            } else {
-                nextSnapshotId = currentSnapshotId + 1;
-            }
+            nextSnapshotId = currentSnapshotId + 1;
             isFullPhaseEnd =
                     boundedChecker.shouldEndInput(snapshotManager.snapshot(currentSnapshotId));
             LOG.debug(
@@ -207,7 +211,7 @@ public class DataTableStreamScan extends AbstractDataTableScan implements Stream
                 throw new EndOfScanException();
             }
 
-            if (shouldDelaySnapshot(nextSnapshotId)) {
+            if (shouldDelaySnapshot(snapshot)) {
                 return SnapshotNotExistPlan.INSTANCE;
             }
 
@@ -216,6 +220,9 @@ public class DataTableStreamScan extends AbstractDataTableScan implements Stream
                 SnapshotReader.Plan overwritePlan = handleOverwriteSnapshot(snapshot);
                 if (overwritePlan != null) {
                     nextSnapshotId++;
+                    if (overwritePlan.splits().isEmpty()) {
+                        continue;
+                    }
                     return overwritePlan;
                 }
             }
@@ -225,6 +232,9 @@ public class DataTableStreamScan extends AbstractDataTableScan implements Stream
                 SnapshotReader.Plan plan = followUpScanner.scan(snapshot, snapshotReader);
                 currentWatermark = plan.watermark();
                 nextSnapshotId++;
+                if (plan.splits().isEmpty()) {
+                    continue;
+                }
                 return plan;
             } else {
                 nextSnapshotId++;
@@ -232,17 +242,13 @@ public class DataTableStreamScan extends AbstractDataTableScan implements Stream
         }
     }
 
-    private boolean shouldDelaySnapshot(long snapshotId) {
+    private boolean shouldDelaySnapshot(Snapshot snapshot) {
         if (scanDelayMillis == null) {
             return false;
         }
 
         long snapshotMills = System.currentTimeMillis() - scanDelayMillis;
-        if (snapshotManager.snapshotExists(snapshotId)
-                && snapshotManager.snapshot(snapshotId).timeMillis() > snapshotMills) {
-            return true;
-        }
-        return false;
+        return snapshot.timeMillis() > snapshotMills;
     }
 
     @Nullable
@@ -250,7 +256,7 @@ public class DataTableStreamScan extends AbstractDataTableScan implements Stream
         if (supportStreamingReadOverwrite) {
             LOG.debug("Find overwrite snapshot id {}.", nextSnapshotId);
             SnapshotReader.Plan overwritePlan =
-                    followUpScanner.getOverwriteChangesPlan(snapshot, snapshotReader);
+                    followUpScanner.getOverwriteChangesPlan(snapshot, snapshotReader, !hasPk);
             currentWatermark = overwritePlan.watermark();
             return overwritePlan;
         }
