@@ -21,6 +21,7 @@ package org.apache.paimon.table.source;
 import org.apache.paimon.metrics.MetricRegistry;
 import org.apache.paimon.predicate.Predicate;
 import org.apache.paimon.predicate.PredicateBuilder;
+import org.apache.paimon.predicate.TopN;
 import org.apache.paimon.types.RowType;
 
 import java.util.List;
@@ -50,6 +51,14 @@ public interface InnerTableRead extends TableRead {
         throw new UnsupportedOperationException();
     }
 
+    default InnerTableRead withTopN(TopN topN) {
+        return this;
+    }
+
+    default InnerTableRead withLimit(int limit) {
+        return this;
+    }
+
     default InnerTableRead forceKeepDelete() {
         return this;
     }
