@@ -24,6 +24,7 @@ import org.apache.paimon.codegen.CodeGenUtils;
 import org.apache.paimon.codegen.RecordComparator;
 import org.apache.paimon.consumer.ConsumerManager;
 import org.apache.paimon.data.BinaryRow;
+import org.apache.paimon.deletionvectors.DeletionVectorsIndexFile;
 import org.apache.paimon.index.DeletionVectorMeta;
 import org.apache.paimon.index.IndexFileHandler;
 import org.apache.paimon.index.IndexFileMeta;
@@ -34,10 +35,10 @@ import org.apache.paimon.manifest.ManifestEntry;
 import org.apache.paimon.manifest.ManifestFileMeta;
 import org.apache.paimon.manifest.PartitionEntry;
 import org.apache.paimon.metrics.MetricRegistry;
-import org.apache.paimon.operation.DefaultValueAssigner;
 import org.apache.paimon.operation.FileStoreScan;
 import org.apache.paimon.operation.ManifestsReader;
 import org.apache.paimon.operation.metrics.ScanMetrics;
+import org.apache.paimon.partition.PartitionPredicate;
 import org.apache.paimon.predicate.Predicate;
 import org.apache.paimon.predicate.PredicateBuilder;
 import org.apache.paimon.schema.TableSchema;
@@ -46,9 +47,11 @@ import org.apache.paimon.table.source.DeletionFile;
 import org.apache.paimon.table.source.PlanImpl;
 import org.apache.paimon.table.source.ScanMode;
 import org.apache.paimon.table.source.SplitGenerator;
+import org.apache.paimon.utils.BiFilter;
 import org.apache.paimon.utils.ChangelogManager;
 import org.apache.paimon.utils.FileStorePathFactory;
 import org.apache.paimon.utils.Filter;
+import org.apache.paimon.utils.LazyField;
 import org.apache.paimon.utils.Pair;
 import org.apache.paimon.utils.SnapshotManager;
 
@@ -62,7 +65,6 @@ import java.util.Iterator;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
-import java.util.Optional;
 import java.util.Set;
 import java.util.function.BiConsumer;
 import java.util.stream.Collectors;
@@ -71,7 +73,7 @@ import static org.apache.paimon.Snapshot.FIRST_SNAPSHOT_ID;
 import static org.apache.paimon.deletionvectors.DeletionVectorsIndexFile.DELETION_VECTORS_INDEX;
 import static org.apache.paimon.operation.FileStoreScan.Plan.groupByPartFiles;
 import static org.apache.paimon.partition.PartitionPredicate.createPartitionPredicate;
-import static org.apache.paimon.predicate.PredicateBuilder.transformFieldMapping;
+import static org.apache.paimon.predicate.PredicateBuilder.splitAndByPartition;
 
 /** Implementation of {@link SnapshotReader}. */
 public class SnapshotReaderImpl implements SnapshotReader {
@@ -85,7 +87,6 @@ public class SnapshotReaderImpl implements SnapshotReader {
     private final ConsumerManager consumerManager;
     private final SplitGenerator splitGenerator;
     private final BiConsumer<FileStoreScan, Predicate> nonPartitionFilterConsumer;
-    private final DefaultValueAssigner defaultValueAssigner;
     private final FileStorePathFactory pathFactory;
     private final String tableName;
     private final IndexFileHandler indexFileHandler;
@@ -101,7 +102,6 @@ public class SnapshotReaderImpl implements SnapshotReader {
             ChangelogManager changelogManager,
             SplitGenerator splitGenerator,
             BiConsumer<FileStoreScan, Predicate> nonPartitionFilterConsumer,
-            DefaultValueAssigner defaultValueAssigner,
             FileStorePathFactory pathFactory,
             String tableName,
             IndexFileHandler indexFileHandler) {
@@ -118,7 +118,6 @@ public class SnapshotReaderImpl implements SnapshotReader {
                         snapshotManager.branch());
         this.splitGenerator = splitGenerator;
         this.nonPartitionFilterConsumer = nonPartitionFilterConsumer;
-        this.defaultValueAssigner = defaultValueAssigner;
         this.pathFactory = pathFactory;
 
         this.tableName = tableName;
@@ -202,32 +201,31 @@ public class SnapshotReaderImpl implements SnapshotReader {
         return this;
     }
 
+    @Override
+    public SnapshotReader withPartitionFilter(PartitionPredicate partitionPredicate) {
+        if (partitionPredicate != null) {
+            scan.withPartitionFilter(partitionPredicate);
+        }
+        return this;
+    }
+
     @Override
     public SnapshotReader withPartitionsFilter(List<Map<String, String>> partitions) {
-        scan.withPartitionsFilter(partitions);
+        if (partitions != null) {
+            scan.withPartitionsFilter(partitions);
+        }
         return this;
     }
 
     @Override
     public SnapshotReader withFilter(Predicate predicate) {
-        List<String> partitionKeys = tableSchema.partitionKeys();
         int[] fieldIdxToPartitionIdx =
-                tableSchema.fields().stream()
-                        .mapToInt(f -> partitionKeys.indexOf(f.name()))
-                        .toArray();
-
-        List<Predicate> partitionFilters = new ArrayList<>();
-        List<Predicate> nonPartitionFilters = new ArrayList<>();
-        for (Predicate p :
-                PredicateBuilder.splitAnd(defaultValueAssigner.handlePredicate(predicate))) {
-            Optional<Predicate> mapped = transformFieldMapping(p, fieldIdxToPartitionIdx);
-            if (mapped.isPresent()) {
-                partitionFilters.add(mapped.get());
-            } else {
-                nonPartitionFilters.add(p);
-            }
-        }
-
+                PredicateBuilder.fieldIdxToPartitionIdx(
+                        tableSchema.logicalRowType(), tableSchema.partitionKeys());
+        Pair<List<Predicate>, List<Predicate>> partitionAndNonPartitionFilter =
+                splitAndByPartition(predicate, fieldIdxToPartitionIdx);
+        List<Predicate> partitionFilters = partitionAndNonPartitionFilter.getLeft();
+        List<Predicate> nonPartitionFilters = partitionAndNonPartitionFilter.getRight();
         if (partitionFilters.size() > 0) {
             scan.withPartitionFilter(PredicateBuilder.and(partitionFilters));
         }
@@ -257,6 +255,12 @@ public class SnapshotReaderImpl implements SnapshotReader {
         return this;
     }
 
+    @Override
+    public SnapshotReader withLevelMinMaxFilter(BiFilter<Integer, Integer> minMaxFilter) {
+        scan.withLevelMinMaxFilter(minMaxFilter);
+        return this;
+    }
+
     @Override
     public SnapshotReader enableValueFilter() {
         scan.enableValueFilter();
@@ -305,6 +309,12 @@ public class SnapshotReaderImpl implements SnapshotReader {
         return this;
     }
 
+    @Override
+    public SnapshotReader keepStats() {
+        scan.keepStats();
+        return this;
+    }
+
     @Override
     public SnapshotReader withShard(int indexOfThisSubtask, int numberOfParallelSubtasks) {
         if (splitGenerator.alwaysRawConvertible()) {
@@ -387,6 +397,7 @@ public class SnapshotReaderImpl implements SnapshotReader {
                     if (deletionVectors && deletionIndexFilesMap != null) {
                         builder.withDataDeletionFiles(
                                 getDeletionFiles(
+                                        indexFileHandler.dvIndex(partition, bucket),
                                         dataFiles,
                                         deletionIndexFilesMap.getOrDefault(
                                                 Pair.of(partition, bucket),
@@ -429,14 +440,15 @@ public class SnapshotReaderImpl implements SnapshotReader {
                 groupByPartFiles(plan.files(FileKind.DELETE));
         Map<BinaryRow, Map<Integer, List<ManifestEntry>>> dataFiles =
                 groupByPartFiles(plan.files(FileKind.ADD));
-        Snapshot beforeSnapshot = snapshotManager.snapshot(plan.snapshot().id() - 1);
+        LazyField<Snapshot> beforeSnapshot =
+                new LazyField<>(() -> snapshotManager.snapshot(plan.snapshot().id() - 1));
         return toChangesPlan(true, plan, beforeSnapshot, beforeFiles, dataFiles);
     }
 
     private Plan toChangesPlan(
             boolean isStreaming,
             FileStoreScan.Plan plan,
-            Snapshot beforeSnapshot,
+            LazyField<Snapshot> beforeSnapshot,
             Map<BinaryRow, Map<Integer, List<ManifestEntry>>> beforeFiles,
             Map<BinaryRow, Map<Integer, List<ManifestEntry>>> dataFiles) {
         Snapshot snapshot = plan.snapshot();
@@ -457,7 +469,9 @@ public class SnapshotReaderImpl implements SnapshotReader {
             beforDeletionIndexFilesMap =
                     deletionVectors
                             ? indexFileHandler.scan(
-                                    beforeSnapshot, DELETION_VECTORS_INDEX, beforeFiles.keySet())
+                                    beforeSnapshot.get(),
+                                    DELETION_VECTORS_INDEX,
+                                    beforeFiles.keySet())
                             : Collections.emptyMap();
             deletionIndexFilesMap =
                     deletionVectors
@@ -510,11 +524,13 @@ public class SnapshotReaderImpl implements SnapshotReader {
                         && deletionIndexFilesMap != null) {
                     builder.withBeforeDeletionFiles(
                             getDeletionFiles(
+                                    indexFileHandler.dvIndex(part, bucket),
                                     before,
                                     beforDeletionIndexFilesMap.getOrDefault(
                                             Pair.of(part, bucket), Collections.emptyList())));
                     builder.withDataDeletionFiles(
                             getDeletionFiles(
+                                    indexFileHandler.dvIndex(part, bucket),
                                     data,
                                     deletionIndexFilesMap.getOrDefault(
                                             Pair.of(part, bucket), Collections.emptyList())));
@@ -535,7 +551,7 @@ public class SnapshotReaderImpl implements SnapshotReader {
                 groupByPartFiles(plan.files(FileKind.ADD));
         Map<BinaryRow, Map<Integer, List<ManifestEntry>>> beforeFiles =
                 groupByPartFiles(scan.withSnapshot(before).plan().files(FileKind.ADD));
-        return toChangesPlan(false, plan, before, beforeFiles, dataFiles);
+        return toChangesPlan(false, plan, new LazyField<>(() -> before), beforeFiles, dataFiles);
     }
 
     private RecordComparator partitionComparator() {
@@ -548,12 +564,14 @@ public class SnapshotReaderImpl implements SnapshotReader {
     }
 
     private List<DeletionFile> getDeletionFiles(
-            List<DataFileMeta> dataFiles, List<IndexFileMeta> indexFileMetas) {
+            DeletionVectorsIndexFile indexFile,
+            List<DataFileMeta> dataFiles,
+            List<IndexFileMeta> indexFileMetas) {
         List<DeletionFile> deletionFiles = new ArrayList<>(dataFiles.size());
         Map<String, IndexFileMeta> dataFileToIndexFileMeta = new HashMap<>();
         for (IndexFileMeta indexFileMeta : indexFileMetas) {
-            if (indexFileMeta.deletionVectorMetas() != null) {
-                for (DeletionVectorMeta dvMeta : indexFileMeta.deletionVectorMetas().values()) {
+            if (indexFileMeta.dvRanges() != null) {
+                for (DeletionVectorMeta dvMeta : indexFileMeta.dvRanges().values()) {
                     dataFileToIndexFileMeta.put(dvMeta.dataFileName(), indexFileMeta);
                 }
             }
@@ -561,12 +579,11 @@ public class SnapshotReaderImpl implements SnapshotReader {
         for (DataFileMeta file : dataFiles) {
             IndexFileMeta indexFileMeta = dataFileToIndexFileMeta.get(file.fileName());
             if (indexFileMeta != null) {
-                LinkedHashMap<String, DeletionVectorMeta> dvMetas =
-                        indexFileMeta.deletionVectorMetas();
+                LinkedHashMap<String, DeletionVectorMeta> dvMetas = indexFileMeta.dvRanges();
                 if (dvMetas != null && dvMetas.containsKey(file.fileName())) {
                     deletionFiles.add(
                             new DeletionFile(
-                                    indexFileHandler.filePath(indexFileMeta).toString(),
+                                    indexFile.path(indexFileMeta).toString(),
                                     dvMetas.get(file.fileName()).offset(),
                                     dvMetas.get(file.fileName()).length(),
                                     dvMetas.get(file.fileName()).cardinality()));
