@@ -34,6 +34,7 @@ import org.apache.paimon.manifest.ManifestFileMeta;
 import org.apache.paimon.manifest.PartitionEntry;
 import org.apache.paimon.metrics.MetricRegistry;
 import org.apache.paimon.operation.ManifestsReader;
+import org.apache.paimon.partition.PartitionPredicate;
 import org.apache.paimon.predicate.LeafPredicate;
 import org.apache.paimon.predicate.Predicate;
 import org.apache.paimon.predicate.PredicateBuilder;
@@ -58,6 +59,7 @@ import org.apache.paimon.table.source.snapshot.StartingContext;
 import org.apache.paimon.types.DataField;
 import org.apache.paimon.types.RowKind;
 import org.apache.paimon.types.RowType;
+import org.apache.paimon.utils.BiFilter;
 import org.apache.paimon.utils.BranchManager;
 import org.apache.paimon.utils.ChangelogManager;
 import org.apache.paimon.utils.FileStorePathFactory;
@@ -328,6 +330,12 @@ public class AuditLogTable implements DataTable, ReadonlyTable {
             return this;
         }
 
+        @Override
+        public SnapshotReader withPartitionFilter(PartitionPredicate partitionPredicate) {
+            wrapped.withPartitionFilter(partitionPredicate);
+            return this;
+        }
+
         @Override
         public SnapshotReader withPartitionsFilter(List<Map<String, String>> partitions) {
             wrapped.withPartitionsFilter(partitions);
@@ -352,6 +360,12 @@ public class AuditLogTable implements DataTable, ReadonlyTable {
             return this;
         }
 
+        @Override
+        public SnapshotReader withLevelMinMaxFilter(BiFilter<Integer, Integer> minMaxFilter) {
+            wrapped.withLevelMinMaxFilter(minMaxFilter);
+            return this;
+        }
+
         @Override
         public SnapshotReader enableValueFilter() {
             wrapped.enableValueFilter();
@@ -393,6 +407,12 @@ public class AuditLogTable implements DataTable, ReadonlyTable {
             return this;
         }
 
+        @Override
+        public SnapshotReader keepStats() {
+            wrapped.keepStats();
+            return this;
+        }
+
         @Override
         public SnapshotReader withShard(int indexOfThisSubtask, int numberOfParallelSubtasks) {
             wrapped.withShard(indexOfThisSubtask, numberOfParallelSubtasks);
@@ -485,6 +505,12 @@ public class AuditLogTable implements DataTable, ReadonlyTable {
             return this;
         }
 
+        @Override
+        public InnerTableScan withPartitionFilter(PartitionPredicate partitionPredicate) {
+            batchScan.withPartitionFilter(partitionPredicate);
+            return this;
+        }
+
         @Override
         public InnerTableScan withBucketFilter(Filter<Integer> bucketFilter) {
             batchScan.withBucketFilter(bucketFilter);
