@@ -140,7 +140,7 @@ public class IndexBootstrapTest extends TableTestBase {
     }
 
     private static DataFileMeta newFile(long timeMillis) {
-        return new DataFileMeta(
+        return DataFileMeta.create(
                 "",
                 1,
                 1,
@@ -161,6 +161,8 @@ public class IndexBootstrapTest extends TableTestBase {
                 null,
                 FileSource.APPEND,
                 null,
+                null,
+                null,
                 null);
     }
 
