@@ -119,7 +119,7 @@ public class ManifestListTest {
                 LocalFileIO.create(),
                 new LegacyManifestFileMetaSerializerPaimon10(),
                 legacyMetaType,
-                avro.createReaderFactory(legacyMetaType),
+                avro.createReaderFactory(legacyMetaType, legacyMetaType, new ArrayList<>()),
                 avro.createWriterFactory(legacyMetaType),
                 "zstd",
                 pathFactory.manifestListFactory(),
@@ -168,11 +168,12 @@ public class ManifestListTest {
                 CoreOptions.FILE_FORMAT.defaultValue().toString(),
                 CoreOptions.DATA_FILE_PREFIX.defaultValue(),
                 CoreOptions.CHANGELOG_FILE_PREFIX.defaultValue(),
-                CoreOptions.PARTITION_GENERATE_LEGCY_NAME.defaultValue(),
+                CoreOptions.PARTITION_GENERATE_LEGACY_NAME.defaultValue(),
                 CoreOptions.FILE_SUFFIX_INCLUDE_COMPRESSION.defaultValue(),
                 CoreOptions.FILE_COMPRESSION.defaultValue(),
                 null,
-                null);
+                null,
+                false);
     }
 
     private ManifestList createManifestList(String pathStr) {
