@@ -25,8 +25,8 @@ import org.apache.paimon.TestAppendFileStore;
 import org.apache.paimon.TestFileStore;
 import org.apache.paimon.TestKeyValueGenerator;
 import org.apache.paimon.data.BinaryRow;
+import org.apache.paimon.deletionvectors.BucketedDvMaintainer;
 import org.apache.paimon.deletionvectors.DeletionVector;
-import org.apache.paimon.deletionvectors.DeletionVectorsMaintainer;
 import org.apache.paimon.fs.Path;
 import org.apache.paimon.fs.local.LocalFileIO;
 import org.apache.paimon.index.IndexFileHandler;
@@ -729,11 +729,19 @@ public class FileStoreCommitTest {
                 record1,
                 gen::getPartition,
                 0,
-                indexFileHandler.writeHashIndex(new int[] {1, 2, 5}));
+                indexFileHandler
+                        .hashIndex(gen.getPartition(record1), 0)
+                        .write(new int[] {1, 2, 5}));
         store.commitDataIndex(
-                record1, gen::getPartition, 1, indexFileHandler.writeHashIndex(new int[] {6, 8}));
+                record1,
+                gen::getPartition,
+                1,
+                indexFileHandler.hashIndex(gen.getPartition(record1), 1).write(new int[] {6, 8}));
         store.commitDataIndex(
-                record2, gen::getPartition, 2, indexFileHandler.writeHashIndex(new int[] {3, 5}));
+                record2,
+                gen::getPartition,
+                2,
+                indexFileHandler.hashIndex(gen.getPartition(record2), 2).write(new int[] {3, 5}));
 
         Snapshot snapshot = store.snapshotManager().latestSnapshot();
 
@@ -744,12 +752,12 @@ public class FileStoreCommitTest {
 
         IndexManifestEntry indexManifestEntry =
                 part1Index.stream().filter(entry -> entry.bucket() == 0).findAny().get();
-        assertThat(indexFileHandler.readHashIndexList(indexManifestEntry.indexFile()))
+        assertThat(indexFileHandler.hashIndex(part1, 0).readList(indexManifestEntry.indexFile()))
                 .containsExactlyInAnyOrder(1, 2, 5);
 
         indexManifestEntry =
                 part1Index.stream().filter(entry -> entry.bucket() == 1).findAny().get();
-        assertThat(indexFileHandler.readHashIndexList(indexManifestEntry.indexFile()))
+        assertThat(indexFileHandler.hashIndex(part1, 1).readList(indexManifestEntry.indexFile()))
                 .containsExactlyInAnyOrder(6, 8);
 
         // assert part2
@@ -757,12 +765,15 @@ public class FileStoreCommitTest {
                 indexFileHandler.scanEntries(snapshot, HASH_INDEX, part2);
         assertThat(part2Index.size()).isEqualTo(1);
         assertThat(part2Index.get(0).bucket()).isEqualTo(2);
-        assertThat(indexFileHandler.readHashIndexList(part2Index.get(0).indexFile()))
+        assertThat(indexFileHandler.hashIndex(part2, 2).readList(part2Index.get(0).indexFile()))
                 .containsExactlyInAnyOrder(3, 5);
 
         // update part1
         store.commitDataIndex(
-                record1, gen::getPartition, 0, indexFileHandler.writeHashIndex(new int[] {1, 4}));
+                record1,
+                gen::getPartition,
+                0,
+                indexFileHandler.hashIndex(gen.getPartition(record1), 0).write(new int[] {1, 4}));
         snapshot = store.snapshotManager().latestSnapshot();
 
         // assert update part1
@@ -771,18 +782,19 @@ public class FileStoreCommitTest {
 
         indexManifestEntry =
                 part1Index.stream().filter(entry -> entry.bucket() == 0).findAny().get();
-        assertThat(indexFileHandler.readHashIndexList(indexManifestEntry.indexFile()))
+        assertThat(indexFileHandler.hashIndex(part1, 0).readList(indexManifestEntry.indexFile()))
                 .containsExactlyInAnyOrder(1, 4);
 
         indexManifestEntry =
                 part1Index.stream().filter(entry -> entry.bucket() == 1).findAny().get();
-        assertThat(indexFileHandler.readHashIndexList(indexManifestEntry.indexFile()))
+        assertThat(indexFileHandler.hashIndex(part1, 1).readList(indexManifestEntry.indexFile()))
                 .containsExactlyInAnyOrder(6, 8);
 
         // assert scan one bucket
         Optional<IndexFileMeta> file = indexFileHandler.scanHashIndex(snapshot, part1, 0);
         assertThat(file).isPresent();
-        assertThat(indexFileHandler.readHashIndexList(file.get())).containsExactlyInAnyOrder(1, 4);
+        assertThat(indexFileHandler.hashIndex(part1, 0).readList(file.get()))
+                .containsExactlyInAnyOrder(1, 4);
 
         // overwrite one partition
         store.options().toConfiguration().set(CoreOptions.DYNAMIC_PARTITION_OVERWRITE, true);
@@ -874,24 +886,25 @@ public class FileStoreCommitTest {
         Map<String, String> options = new HashMap<>();
         options.put(CoreOptions.DELETION_VECTOR_BITMAP64.key(), String.valueOf(bitmap64));
         TestAppendFileStore store = TestAppendFileStore.createAppendStore(tempDir, options);
+        BinaryRow partition = gen.getPartition(gen.next());
+
+        // create files
+        CommitMessageImpl commitMessage0 =
+                store.writeDataFiles(partition, 0, Arrays.asList("f1", "f2"));
+        store.commit(commitMessage0);
 
         // commit 1
         CommitMessageImpl commitMessage1 =
                 store.writeDVIndexFiles(
-                        BinaryRow.EMPTY_ROW,
-                        0,
-                        Collections.singletonMap("f1", Arrays.asList(1, 3)));
+                        partition, 0, Collections.singletonMap("f1", Arrays.asList(1, 3)));
         CommitMessageImpl commitMessage2 =
                 store.writeDVIndexFiles(
-                        BinaryRow.EMPTY_ROW,
-                        0,
-                        Collections.singletonMap("f2", Arrays.asList(2, 4)));
+                        partition, 0, Collections.singletonMap("f2", Arrays.asList(2, 4)));
         store.commit(commitMessage1, commitMessage2);
 
         // assert 1
-        assertThat(store.scanDVIndexFiles(BinaryRow.EMPTY_ROW, 0).size()).isEqualTo(2);
-        DeletionVectorsMaintainer maintainer =
-                store.createOrRestoreDVMaintainer(BinaryRow.EMPTY_ROW, 0);
+        assertThat(store.scanDVIndexFiles(partition, 0).size()).isEqualTo(2);
+        BucketedDvMaintainer maintainer = store.createOrRestoreDVMaintainer(partition, 0);
         Map<String, DeletionVector> dvs = maintainer.deletionVectors();
         assertThat(dvs.size()).isEqualTo(2);
         assertThat(dvs.get("f2").isDeleted(2)).isTrue();
@@ -901,16 +914,16 @@ public class FileStoreCommitTest {
         // commit 2
         CommitMessage commitMessage3 =
                 store.writeDVIndexFiles(
-                        BinaryRow.EMPTY_ROW, 0, Collections.singletonMap("f2", Arrays.asList(3)));
+                        partition, 0, Collections.singletonMap("f2", Arrays.asList(3)));
         List<IndexFileMeta> deleted =
-                new ArrayList<>(commitMessage1.indexIncrement().newIndexFiles());
-        deleted.addAll(commitMessage2.indexIncrement().newIndexFiles());
-        CommitMessage commitMessage4 = store.removeIndexFiles(BinaryRow.EMPTY_ROW, 0, deleted);
+                new ArrayList<>(commitMessage1.newFilesIncrement().newIndexFiles());
+        deleted.addAll(commitMessage2.newFilesIncrement().newIndexFiles());
+        CommitMessage commitMessage4 = store.removeIndexFiles(partition, 0, deleted);
         store.commit(commitMessage3, commitMessage4);
 
         // assert 2
-        assertThat(store.scanDVIndexFiles(BinaryRow.EMPTY_ROW, 0).size()).isEqualTo(1);
-        maintainer = store.createOrRestoreDVMaintainer(BinaryRow.EMPTY_ROW, 0);
+        assertThat(store.scanDVIndexFiles(partition, 0).size()).isEqualTo(1);
+        maintainer = store.createOrRestoreDVMaintainer(partition, 0);
         dvs = maintainer.deletionVectors();
         assertThat(dvs.size()).isEqualTo(2);
         assertThat(dvs.get("f1").isDeleted(3)).isTrue();
