@@ -48,6 +48,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.UUID;
 
+import static org.apache.paimon.rest.RESTApi.HEADER_PREFIX;
 import static org.assertj.core.api.Assertions.assertThatThrownBy;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 
@@ -83,7 +84,9 @@ class MockRESTCatalogTest extends RESTCatalogTest {
 
     @AfterEach
     public void tearDown() throws Exception {
-        restCatalogServer.shutdown();
+        if (restCatalogServer != null) {
+            restCatalogServer.shutdown();
+        }
     }
 
     @Test
@@ -151,6 +154,22 @@ class MockRESTCatalogTest extends RESTCatalogTest {
         assertEquals(headers.get(serverDefineHeaderName), serverDefineHeaderValue);
     }
 
+    @Test
+    void testHeaderOptions() throws Exception {
+        options.set(HEADER_PREFIX + "User-Agent", "test");
+        RESTCatalog restCatalog = initCatalog(false);
+
+        Map<String, String> parameters = new HashMap<>();
+        RESTAuthParameter restAuthParameter =
+                new RESTAuthParameter("/path", parameters, "method", "data");
+        Map<String, String> headers = restCatalog.api().authFunction().apply(restAuthParameter);
+        assertEquals(headers.get("User-Agent"), "test");
+
+        RESTCatalog restCatalog2 = restCatalog.catalogLoader().load();
+        Map<String, String> headers2 = restCatalog2.api().authFunction().apply(restAuthParameter);
+        assertEquals(headers2.get("User-Agent"), "test");
+    }
+
     private void testDlfAuth(RESTCatalog restCatalog) throws Exception {
         String databaseName = "db1";
         restCatalog.createDatabase(databaseName, true);
@@ -160,10 +179,10 @@ class MockRESTCatalogTest extends RESTCatalogTest {
                     Identifier.create(databaseName, tableName), DEFAULT_TABLE_SCHEMA, false);
         }
         PagedList<String> listTablesPaged =
-                restCatalog.listTablesPaged(databaseName, 1, "dt=20230101", null);
+                restCatalog.listTablesPaged(databaseName, 1, "dt=20230101", null, null);
         PagedList<String> listTablesPaged2 =
                 restCatalog.listTablesPaged(
-                        databaseName, 1, listTablesPaged.getNextPageToken(), null);
+                        databaseName, 1, listTablesPaged.getNextPageToken(), null, null);
         assertEquals(listTablesPaged.getElements().get(0), "dt=20230102");
         assertEquals(listTablesPaged2.getElements().get(0), "dt=20230103");
     }
