@@ -21,7 +21,7 @@ package org.apache.paimon.table;
 import org.apache.paimon.data.BinaryRow;
 import org.apache.paimon.data.GenericRow;
 import org.apache.paimon.data.InternalRow;
-import org.apache.paimon.index.HashIndexMaintainer;
+import org.apache.paimon.index.DynamicBucketIndexMaintainer;
 import org.apache.paimon.operation.AbstractFileStoreWrite;
 import org.apache.paimon.schema.Schema;
 import org.apache.paimon.table.sink.BatchTableWrite;
@@ -51,18 +51,17 @@ public class DynamicBucketTableTest extends TableTestBase {
         Table table = getTableDefault();
         BatchWriteBuilderImpl builder = (BatchWriteBuilderImpl) table.newBatchWriteBuilder();
         TableWriteImpl batchTableWrite = (TableWriteImpl) builder.withOverwrite().newWrite();
-        HashIndexMaintainer indexMaintainer =
-                (HashIndexMaintainer)
-                        ((AbstractFileStoreWrite<?>) (batchTableWrite.getWrite()))
-                                .createWriterContainer(BinaryRow.EMPTY_ROW, 0, true)
-                                .indexMaintainer;
+        AbstractFileStoreWrite<?> write = (AbstractFileStoreWrite<?>) (batchTableWrite.getWrite());
+        write.withIgnorePreviousFiles(true);
+        DynamicBucketIndexMaintainer indexMaintainer =
+                write.createWriterContainer(BinaryRow.EMPTY_ROW, 0).dynamicBucketMaintainer;
 
         assertThat(indexMaintainer.isEmpty()).isTrue();
         Pair<InternalRow, Integer> rowWithBucket = data(0);
         batchTableWrite.write(rowWithBucket.getKey(), rowWithBucket.getValue());
         assertThat(
                         ((CommitMessageImpl) batchTableWrite.prepareCommit().get(0))
-                                .indexIncrement()
+                                .newFilesIncrement()
                                 .newIndexFiles()
                                 .get(0)
                                 .rowCount())
