@@ -21,6 +21,7 @@ package org.apache.paimon.table;
 import org.apache.paimon.CoreOptions;
 import org.apache.paimon.catalog.Catalog;
 import org.apache.paimon.catalog.Identifier;
+import org.apache.paimon.data.BinaryString;
 import org.apache.paimon.data.DataFormatTestUtil;
 import org.apache.paimon.data.GenericRow;
 import org.apache.paimon.data.InternalRow;
@@ -34,9 +35,11 @@ import org.apache.paimon.schema.SchemaManager;
 import org.apache.paimon.schema.TableSchema;
 import org.apache.paimon.table.sink.StreamTableWrite;
 import org.apache.paimon.table.sink.TableCommitImpl;
+import org.apache.paimon.table.source.DataSplit;
 import org.apache.paimon.table.source.InnerTableRead;
 import org.apache.paimon.table.source.Split;
 import org.apache.paimon.table.source.snapshot.SnapshotReader;
+import org.apache.paimon.types.DataField;
 import org.apache.paimon.types.DataType;
 import org.apache.paimon.types.DataTypes;
 import org.apache.paimon.types.RowType;
@@ -56,9 +59,17 @@ import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 import java.util.UUID;
 import java.util.function.Consumer;
 
+import static org.apache.paimon.CoreOptions.AGG_FUNCTION;
+import static org.apache.paimon.CoreOptions.DISTINCT;
+import static org.apache.paimon.CoreOptions.FIELDS_PREFIX;
+import static org.apache.paimon.CoreOptions.IGNORE_RETRACT;
+import static org.apache.paimon.CoreOptions.NESTED_KEY;
+import static org.apache.paimon.CoreOptions.SEQUENCE_FIELD;
+import static org.apache.paimon.mergetree.compact.PartialUpdateMergeFunction.SEQUENCE_GROUP;
 import static org.apache.paimon.table.SpecialFields.KEY_FIELD_PREFIX;
 import static org.apache.paimon.table.SpecialFields.SYSTEM_FIELD_NAMES;
 import static org.apache.paimon.testutils.assertj.PaimonAssertions.anyCauseMatches;
@@ -216,6 +227,22 @@ public class SchemaEvolutionTest {
         assertThat(tableSchema.fields().get(0).type()).isEqualTo(DataTypes.STRING());
     }
 
+    @Test
+    public void testUpdatePrimaryKeyType() throws Exception {
+        Schema schema =
+                Schema.newBuilder()
+                        .column("k", DataTypes.INT())
+                        .column("v", DataTypes.BIGINT())
+                        .primaryKey("k")
+                        .build();
+        schemaManager.createTable(schema);
+
+        List<SchemaChange> changes =
+                Collections.singletonList(SchemaChange.updateColumnType("k", DataTypes.STRING()));
+        assertThatThrownBy(() -> schemaManager.commitChanges(changes))
+                .hasMessageContaining("Cannot update primary key");
+    }
+
     @Test
     public void testRenameField() throws Exception {
         Schema schema =
@@ -405,6 +432,182 @@ public class SchemaEvolutionTest {
                                         "_VALUE_KIND", SYSTEM_FIELD_NAMES)));
     }
 
+    @Test
+    public void testPushDownEvolutionSafeFilter() throws Exception {
+        Schema schema =
+                new Schema(
+                        RowType.of(DataTypes.INT(), DataTypes.BIGINT()).getFields(),
+                        Collections.emptyList(),
+                        Collections.emptyList(),
+                        Collections.singletonMap("write-only", "true"),
+                        "");
+        schemaManager.createTable(schema);
+
+        FileStoreTable table = FileStoreTableFactory.create(LocalFileIO.create(), tablePath);
+
+        try (StreamTableWrite write = table.newWrite(commitUser);
+                TableCommitImpl commit = table.newCommit(commitUser)) {
+            // file 1
+            write.write(GenericRow.of(1, 1L));
+            commit.commit(1, write.prepareCommit(false, 1));
+
+            // file 2
+            write.write(GenericRow.of(2, 2L));
+            commit.commit(2, write.prepareCommit(false, 2));
+        }
+
+        schemaManager.commitChanges(
+                Collections.singletonList(SchemaChange.updateColumnType("f0", DataTypes.STRING())));
+        table = FileStoreTableFactory.create(LocalFileIO.create(), tablePath);
+
+        try (StreamTableWrite write = table.newWrite(commitUser);
+                TableCommitImpl commit = table.newCommit(commitUser)) {
+            // file 3
+            write.write(GenericRow.of(BinaryString.fromString("0"), 3L));
+            commit.commit(3, write.prepareCommit(false, 3));
+
+            // file 4
+            write.write(GenericRow.of(BinaryString.fromString("3"), 3L));
+            commit.commit(4, write.prepareCommit(false, 4));
+        }
+
+        PredicateBuilder builder = new PredicateBuilder(table.schema().logicalRowType());
+        // p: f0 >= '3' && f1 >= 2L
+        Predicate p =
+                PredicateBuilder.and(
+                        builder.greaterOrEqual(0, BinaryString.fromString("3")),
+                        builder.greaterOrEqual(1, 2L));
+        // file 1 will be filtered by f1 >= 2L
+        // file 2 won't be filtered because f0 >= '3' is not safe
+        // file 3 will be filtered by f0 >= '3'
+        // file 4 won't be filtered
+        List<String> rows = readRecords(table, p);
+        assertThat(rows).containsExactlyInAnyOrder("2, 2", "3, 3");
+    }
+
+    @Test
+    public void testRenameFieldReferencedByOptions() throws Exception {
+        ImmutableMap.Builder<String, String> mapBuilder = ImmutableMap.builder();
+
+        Schema schema =
+                new Schema(
+                        ImmutableList.of(
+                                new DataField(0, "f0", DataTypes.INT()),
+                                new DataField(1, "f1", DataTypes.INT()),
+                                new DataField(2, "f2", DataTypes.INT()),
+                                new DataField(3, "f3", DataTypes.INT()),
+                                new DataField(
+                                        4,
+                                        "f4",
+                                        DataTypes.ARRAY(
+                                                DataTypes.ROW(
+                                                        new DataField(5, "f5", DataTypes.INT())))),
+                                new DataField(6, "f6", DataTypes.ARRAY(DataTypes.INT()))),
+                        ImmutableList.of("f0"),
+                        ImmutableList.of(),
+                        mapBuilder
+                                .put(SEQUENCE_FIELD.key(), "f1,f2")
+                                .put(FIELDS_PREFIX + "." + "f3" + "." + IGNORE_RETRACT, "true")
+                                .put(
+                                        FIELDS_PREFIX + "." + "f4" + "." + AGG_FUNCTION,
+                                        "nested_update")
+                                .put(FIELDS_PREFIX + "." + "f4" + "." + NESTED_KEY, "f5")
+                                .put(FIELDS_PREFIX + "." + "f6" + "." + AGG_FUNCTION, "collect")
+                                .put(FIELDS_PREFIX + "." + "f6" + "." + DISTINCT, "true")
+                                .build(),
+                        "");
+
+        schemaManager.createTable(schema);
+
+        TableSchema newSchema =
+                schemaManager.commitChanges(
+                        SchemaChange.renameColumn("f1", "f1_"),
+                        SchemaChange.renameColumn("f2", "f2_"),
+                        SchemaChange.renameColumn("f3", "f3_"),
+                        SchemaChange.renameColumn("f4", "f4_"),
+                        // doesn't support rename nested columns currently
+                        // SchemaChange.renameColumn("f5", "f5_"),
+                        SchemaChange.renameColumn("f6", "f6_"));
+
+        assertThat(newSchema.fieldNames()).containsExactly("f0", "f1_", "f2_", "f3_", "f4_", "f6_");
+
+        assertThat(newSchema.options())
+                .doesNotContainKeys(
+                        FIELDS_PREFIX + "." + "f3" + "." + IGNORE_RETRACT,
+                        FIELDS_PREFIX + "." + "f4" + "." + AGG_FUNCTION,
+                        FIELDS_PREFIX + "." + "f4" + "." + NESTED_KEY,
+                        FIELDS_PREFIX + "." + "f6" + "." + AGG_FUNCTION,
+                        FIELDS_PREFIX + "." + "f6" + "." + DISTINCT);
+
+        Map.Entry[] entries =
+                ImmutableMap.of(
+                                SEQUENCE_FIELD.key(),
+                                "f1_,f2_",
+                                FIELDS_PREFIX + "." + "f3_" + "." + IGNORE_RETRACT,
+                                "true",
+                                FIELDS_PREFIX + "." + "f4_" + "." + AGG_FUNCTION,
+                                "nested_update",
+                                FIELDS_PREFIX + "." + "f6_" + "." + AGG_FUNCTION,
+                                "collect",
+                                FIELDS_PREFIX + "." + "f6_" + "." + DISTINCT,
+                                "true")
+                        .entrySet()
+                        .toArray(new Map.Entry[0]);
+
+        assertThat(newSchema.options()).contains(entries);
+    }
+
+    @Test
+    public void testRenameSeqGroupFields() throws Exception {
+        ImmutableMap.Builder<String, String> mapBuilder = ImmutableMap.builder();
+
+        Schema schema =
+                new Schema(
+                        ImmutableList.of(
+                                new DataField(0, "f0", DataTypes.INT()),
+                                new DataField(1, "f1", DataTypes.INT()),
+                                new DataField(2, "f2", DataTypes.INT()),
+                                new DataField(3, "f3", DataTypes.INT()),
+                                new DataField(4, "f4", DataTypes.INT()),
+                                new DataField(5, "f5", DataTypes.INT()),
+                                new DataField(6, "f6", DataTypes.INT())),
+                        ImmutableList.of("f0"),
+                        ImmutableList.of(),
+                        mapBuilder
+                                .put(FIELDS_PREFIX + "." + "f1,f2" + "." + SEQUENCE_GROUP, "f3")
+                                .put(FIELDS_PREFIX + "." + "f4" + "." + SEQUENCE_GROUP, "f5,f6")
+                                .build(),
+                        "");
+
+        schemaManager.createTable(schema);
+
+        TableSchema newSchema =
+                schemaManager.commitChanges(
+                        SchemaChange.renameColumn("f1", "f1_"),
+                        SchemaChange.renameColumn("f2", "f2_"),
+                        SchemaChange.renameColumn("f3", "f3_"),
+                        SchemaChange.renameColumn("f4", "f4_"),
+                        SchemaChange.renameColumn("f5", "f5_"),
+                        SchemaChange.renameColumn("f6", "f6_"));
+
+        assertThat(newSchema.fieldNames())
+                .containsExactly("f0", "f1_", "f2_", "f3_", "f4_", "f5_", "f6_");
+
+        assertThat(newSchema.options())
+                .doesNotContainKeys(
+                        FIELDS_PREFIX + "." + "f1,f2" + "." + SEQUENCE_GROUP,
+                        FIELDS_PREFIX + "." + "f4" + "." + SEQUENCE_GROUP);
+
+        Map.Entry[] entries =
+                ImmutableMap.of(
+                                FIELDS_PREFIX + "." + "f1_,f2_" + "." + SEQUENCE_GROUP, "f3_",
+                                FIELDS_PREFIX + "." + "f4_" + "." + SEQUENCE_GROUP, "f5_,f6_")
+                        .entrySet()
+                        .toArray(new Map.Entry[0]);
+
+        assertThat(newSchema.options()).contains(entries);
+    }
+
     private List<String> readRecords(FileStoreTable table, Predicate filter) throws IOException {
         List<String> results = new ArrayList<>();
         forEachRemaining(
@@ -423,7 +626,9 @@ public class SchemaEvolutionTest {
         if (filter != null) {
             snapshotReader.withFilter(filter);
         }
-        for (Split split : snapshotReader.read().dataSplits()) {
+        List<DataSplit> dataSplits = snapshotReader.read().dataSplits();
+
+        for (Split split : dataSplits) {
             InnerTableRead read = table.newRead();
             if (filter != null) {
                 read.withFilter(filter);
