@@ -20,21 +20,19 @@ package org.apache.paimon.oss;
 
 import org.apache.paimon.catalog.CatalogContext;
 import org.apache.paimon.fs.FileIO;
-import org.apache.paimon.fs.FileStatus;
-import org.apache.paimon.fs.Path;
-import org.apache.paimon.fs.RemoteIterator;
 import org.apache.paimon.options.Options;
 import org.apache.paimon.utils.IOUtils;
+import org.apache.paimon.utils.ReflectionUtils;
 
-import com.aliyun.oss.model.ObjectMetadata;
+import com.aliyun.oss.OSSClient;
+import com.aliyun.oss.common.comm.ServiceClient;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.aliyun.oss.AliyunOSSFileSystem;
+import org.apache.hadoop.fs.aliyun.oss.AliyunOSSFileSystemStore;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import javax.annotation.Nullable;
-
 import java.io.IOException;
 import java.io.UncheckedIOException;
 import java.net.URI;
@@ -45,7 +43,6 @@ import java.util.concurrent.ConcurrentHashMap;
 import java.util.function.Supplier;
 
 import static org.apache.paimon.options.CatalogOptions.FILE_IO_ALLOW_CACHE;
-import static org.apache.paimon.options.CatalogOptions.FILE_IO_POPULATE_META;
 
 /** OSS {@link FileIO}. */
 public class OSSFileIO extends HadoopCompliantFileIO {
@@ -63,6 +60,7 @@ public class OSSFileIO extends HadoopCompliantFileIO {
     private static final String OSS_ACCESS_KEY_ID = "fs.oss.accessKeyId";
     private static final String OSS_ACCESS_KEY_SECRET = "fs.oss.accessKeySecret";
     private static final String OSS_SECURITY_TOKEN = "fs.oss.securityToken";
+    private static final String OSS_SECOND_LEVEL_DOMAIN_ENABLED = "fs.oss.sld.enabled";
 
     private static final Map<String, String> CASE_SENSITIVE_KEYS =
             new HashMap<String, String>() {
@@ -84,7 +82,6 @@ public class OSSFileIO extends HadoopCompliantFileIO {
 
     private Options hadoopOptions;
     private boolean allowCache = true;
-    private boolean populateMeta = false;
 
     @Override
     public boolean isObjectStore() {
@@ -94,7 +91,6 @@ public class OSSFileIO extends HadoopCompliantFileIO {
     @Override
     public void configure(CatalogContext context) {
         allowCache = context.options().get(FILE_IO_ALLOW_CACHE);
-        populateMeta = context.options().get(FILE_IO_POPULATE_META);
         hadoopOptions = new Options();
         // read all configuration with prefix 'CONFIG_PREFIXES'
         for (String key : context.options().keySet()) {
@@ -115,6 +111,10 @@ public class OSSFileIO extends HadoopCompliantFileIO {
         }
     }
 
+    public Options hadoopOptions() {
+        return hadoopOptions;
+    }
+
     @Override
     protected AliyunOSSFileSystem createFileSystem(org.apache.hadoop.fs.Path path) {
         final String scheme = path.toUri().getScheme();
@@ -142,6 +142,11 @@ public class OSSFileIO extends HadoopCompliantFileIO {
                     } catch (IOException e) {
                         throw new UncheckedIOException(e);
                     }
+
+                    if (hadoopOptions.getBoolean(OSS_SECOND_LEVEL_DOMAIN_ENABLED, false)) {
+                        enableSecondLevelDomain(fs);
+                    }
+
                     return fs;
                 };
 
@@ -161,6 +166,19 @@ public class OSSFileIO extends HadoopCompliantFileIO {
         }
     }
 
+    public void enableSecondLevelDomain(AliyunOSSFileSystem fs) {
+        AliyunOSSFileSystemStore store = fs.getStore();
+        try {
+            OSSClient ossClient = ReflectionUtils.getPrivateFieldValue(store, "ossClient");
+            ServiceClient serviceClient =
+                    ReflectionUtils.getPrivateFieldValue(ossClient, "serviceClient");
+            serviceClient.getClientConfiguration().setSLDEnabled(true);
+        } catch (Exception e) {
+            LOG.error("Failed to enable second level domain.", e);
+            throw new RuntimeException("Failed to enable second level domain.", e);
+        }
+    }
+
     private static class CacheKey {
 
         private final Options options;
@@ -192,156 +210,4 @@ public class OSSFileIO extends HadoopCompliantFileIO {
             return Objects.hash(options, scheme, authority);
         }
     }
-
-    @Override
-    public FileStatus getFileStatus(Path path) throws IOException {
-        FileStatus basic = super.getFileStatus(path);
-        if (!populateMeta) {
-            return basic;
-        }
-        AliyunOSSFileSystem fs = (AliyunOSSFileSystem) getFileSystem(path(path));
-        return getExtendedFileStatus(fs, basic);
-    }
-
-    @Override
-    public FileStatus[] listStatus(Path path) throws IOException {
-        FileStatus[] basic = super.listStatus(path);
-        if (!populateMeta) {
-            return basic;
-        }
-        AliyunOSSFileSystem fs = (AliyunOSSFileSystem) getFileSystem(path(path));
-        FileStatus[] extended = new FileStatus[basic.length];
-        for (int i = 0; i < basic.length; i++) {
-            extended[i] = getExtendedFileStatus(fs, basic[i]);
-        }
-        return extended;
-    }
-
-    @Override
-    public RemoteIterator<FileStatus> listFilesIterative(Path path, boolean recursive)
-            throws IOException {
-        RemoteIterator<FileStatus> basicIter = super.listFilesIterative(path, recursive);
-        if (!populateMeta) {
-            return basicIter;
-        }
-        AliyunOSSFileSystem fs = (AliyunOSSFileSystem) getFileSystem(path(path));
-        return new RemoteIterator<FileStatus>() {
-            @Override
-            public boolean hasNext() throws IOException {
-                return basicIter.hasNext();
-            }
-
-            @Override
-            public FileStatus next() throws IOException {
-                FileStatus basic = basicIter.next();
-                return getExtendedFileStatus(fs, basic);
-            }
-
-            @Override
-            public void close() throws IOException {
-                basicIter.close();
-            }
-        };
-    }
-
-    private ExtendedFileStatus getExtendedFileStatus(AliyunOSSFileSystem fs, FileStatus status) {
-        org.apache.hadoop.fs.Path path = path(status.getPath());
-        if (!path.isAbsolute()) {
-            path = new org.apache.hadoop.fs.Path(fs.getWorkingDirectory(), path);
-        }
-        String objKey = path.toUri().getPath().substring(1);
-        ObjectMetadata meta = fs.getStore().getObjectMetadata(objKey);
-        return new ExtendedFileStatus(status, meta);
-    }
-
-    private static class ExtendedFileStatus implements FileStatus {
-
-        private final FileStatus basic;
-        @Nullable private final com.aliyun.oss.model.ObjectMetadata meta;
-
-        private ExtendedFileStatus(
-                FileStatus basic, @Nullable com.aliyun.oss.model.ObjectMetadata meta) {
-            this.basic = basic;
-            this.meta = meta;
-        }
-
-        @Override
-        public long getLen() {
-            return basic.getLen();
-        }
-
-        @Override
-        public boolean isDir() {
-            return basic.isDir();
-        }
-
-        @Override
-        public Path getPath() {
-            return basic.getPath();
-        }
-
-        @Override
-        public long getModificationTime() {
-            return basic.getModificationTime();
-        }
-
-        @Override
-        public long getAccessTime() {
-            return basic.getAccessTime();
-        }
-
-        @Nullable
-        @Override
-        public String getOwner() {
-            return basic.getOwner();
-        }
-
-        @Nullable
-        @Override
-        public Integer getGeneration() {
-            return basic.getGeneration();
-        }
-
-        @Nullable
-        @Override
-        public String getContentType() {
-            if (meta == null) {
-                return basic.getContentType();
-            }
-            return meta.getContentType();
-        }
-
-        @Nullable
-        @Override
-        public String getStorageClass() {
-            return basic.getStorageClass();
-        }
-
-        @Nullable
-        @Override
-        public String getMd5Hash() {
-            if (meta == null) {
-                return basic.getMd5Hash();
-            }
-            return meta.getContentMD5();
-        }
-
-        @Nullable
-        @Override
-        public Long getMetadataModificationTime() {
-            if (meta == null) {
-                return basic.getMetadataModificationTime();
-            }
-            return meta.getLastModified().getTime();
-        }
-
-        @Nullable
-        @Override
-        public Map<String, String> getMetadata() {
-            if (meta == null) {
-                return basic.getMetadata();
-            }
-            return meta.getUserMetadata();
-        }
-    }
 }
