@@ -130,6 +130,10 @@ public class MySqlTypeUtils {
     private static final String GEOMETRYCOLLECTION = "GEOMETRYCOLLECTION";
     private static final String UNKNOWN = "UNKNOWN";
 
+    // Special types for AliyunJson format
+    private static final String LONG = "LONG";
+    private static final String STRING = "STRING";
+
     // This length is from JDBC.
     // It returns the number of characters when converting this timestamp to string.
     // The base length of a timestamp is 19, for example "2023-03-23 17:20:00".
@@ -221,6 +225,7 @@ public class MySqlTypeUtils {
             case MEDIUMINT_UNSIGNED:
             case MEDIUMINT_UNSIGNED_ZEROFILL:
             case BIGINT:
+            case LONG:
                 return DataTypes.BIGINT();
             case BIGINT_UNSIGNED:
             case BIGINT_UNSIGNED_ZEROFILL:
@@ -303,6 +308,7 @@ public class MySqlTypeUtils {
             case MULTILINESTRING:
             case MULTIPOLYGON:
             case GEOMETRYCOLLECTION:
+            case STRING:
                 return DataTypes.STRING();
                 // MySQL BINARY and VARBINARY are stored as bytes in JSON. We convert them to
                 // DataTypes.VARBINARY to retain the length information
