@@ -23,7 +23,6 @@ import org.apache.paimon.table.FormatTable;
 import org.apache.flink.table.api.Schema;
 import org.apache.flink.table.catalog.CatalogTable;
 import org.apache.flink.table.connector.sink.DynamicTableSink;
-import org.apache.flink.table.connector.source.DynamicTableSource;
 import org.apache.flink.table.factories.DynamicTableFactory;
 import org.apache.flink.table.factories.FactoryUtil;
 import org.apache.flink.table.types.logical.RowType;
@@ -39,7 +38,6 @@ import static org.apache.flink.table.factories.FactoryUtil.CONNECTOR;
 import static org.apache.flink.table.factories.FactoryUtil.FORMAT;
 import static org.apache.flink.table.types.utils.TypeConversions.fromLogicalToDataType;
 import static org.apache.paimon.flink.LogicalTypeConversion.toLogicalType;
-import static org.apache.paimon.table.FormatTableOptions.FIELD_DELIMITER;
 
 /** A {@link CatalogTable} to represent format table. */
 public class FormatCatalogTable implements CatalogTable {
@@ -99,8 +97,8 @@ public class FormatCatalogTable implements CatalogTable {
                             cachedOptions.put(k, v);
                         }
                     });
-            if (options.containsKey(FIELD_DELIMITER.key())) {
-                cachedOptions.put("csv.field-delimiter", options.get(FIELD_DELIMITER.key()));
+            if (options.containsKey("field-delimiter")) {
+                cachedOptions.put("csv.field-delimiter", "field-delimiter");
             }
             cachedOptions.put(CONNECTOR.key(), "filesystem");
             cachedOptions.put(PATH.key(), table.location());
@@ -129,17 +127,6 @@ public class FormatCatalogTable implements CatalogTable {
         return getDescription();
     }
 
-    public DynamicTableSource createTableSource(DynamicTableFactory.Context context) {
-        return FactoryUtil.createDynamicTableSource(
-                null,
-                context.getObjectIdentifier(),
-                context.getCatalogTable(),
-                new HashMap<>(),
-                context.getConfiguration(),
-                context.getClassLoader(),
-                context.isTemporary());
-    }
-
     public DynamicTableSink createTableSink(DynamicTableFactory.Context context) {
         return FactoryUtil.createDynamicTableSink(
                 null,
