@@ -56,6 +56,7 @@ public class ChangelogCompactCoordinateOperator
 
     private transient long checkpointId;
     private transient Map<BinaryRow, PartitionChangelog> partitionChangelogs;
+    private transient Map<BinaryRow, Integer> numBuckets;
 
     public ChangelogCompactCoordinateOperator(CoreOptions options) {
         this.options = options;
@@ -67,6 +68,7 @@ public class ChangelogCompactCoordinateOperator
 
         checkpointId = Long.MIN_VALUE;
         partitionChangelogs = new LinkedHashMap<>();
+        numBuckets = new LinkedHashMap<>();
     }
 
     public void processElement(StreamRecord<Committable> record) {
@@ -84,6 +86,8 @@ public class ChangelogCompactCoordinateOperator
             return;
         }
 
+        numBuckets.put(message.partition(), message.totalBuckets());
+
         // Changelog files are not stored in an LSM tree,
         // so we can regard them as files without primary keys.
         long targetFileSize = options.targetFileSize(false);
@@ -134,21 +138,26 @@ public class ChangelogCompactCoordinateOperator
                         new DataIncrement(
                                 message.newFilesIncrement().newFiles(),
                                 message.newFilesIncrement().deletedFiles(),
-                                skippedNewChangelogs),
+                                skippedNewChangelogs,
+                                message.newFilesIncrement().newIndexFiles(),
+                                message.newFilesIncrement().deletedIndexFiles()),
                         new CompactIncrement(
                                 message.compactIncrement().compactBefore(),
                                 message.compactIncrement().compactAfter(),
-                                skippedCompactChangelogs),
-                        message.indexIncrement());
+                                skippedCompactChangelogs,
+                                message.compactIncrement().newIndexFiles(),
+                                message.compactIncrement().deletedIndexFiles()));
         Committable newCommittable =
                 new Committable(committable.checkpointId(), Committable.Kind.FILE, newMessage);
         output.collect(new StreamRecord<>(Either.Left(newCommittable)));
     }
 
+    @Override
     public void prepareSnapshotPreBarrier(long checkpointId) {
         emitAllPartitionsChangelogCompactTask();
     }
 
+    @Override
     public void endInput() {
         emitAllPartitionsChangelogCompactTask();
     }
@@ -174,7 +183,7 @@ public class ChangelogCompactCoordinateOperator
                         new CommitMessageImpl(
                                 partition,
                                 entry.getKey(),
-                                options.bucket(),
+                                numBuckets.get(partition),
                                 new DataIncrement(
                                         Collections.emptyList(),
                                         Collections.emptyList(),
@@ -187,7 +196,7 @@ public class ChangelogCompactCoordinateOperator
                         new CommitMessageImpl(
                                 partition,
                                 entry.getKey(),
-                                options.bucket(),
+                                numBuckets.get(partition),
                                 DataIncrement.emptyIncrement(),
                                 new CompactIncrement(
                                         Collections.emptyList(),
@@ -204,7 +213,7 @@ public class ChangelogCompactCoordinateOperator
                                     new ChangelogCompactTask(
                                             checkpointId,
                                             partition,
-                                            options.bucket(),
+                                            numBuckets.get(partition),
                                             partitionChangelog.newFileChangelogFiles,
                                             partitionChangelog.compactChangelogFiles))));
         }
@@ -216,6 +225,7 @@ public class ChangelogCompactCoordinateOperator
         for (BinaryRow partition : partitions) {
             emitPartitionChangelogCompactTask(partition);
         }
+        numBuckets.clear();
     }
 
     private static class PartitionChangelog {
