@@ -91,7 +91,7 @@ public abstract class FlinkWriteSink<T> extends FlinkSink<T> {
                             StoreSinkWriteState.StateValueFilter stateFilter) {
                         // No conflicts will occur in append only unaware bucket writer, so no state
                         // is needed.
-                        return new NoopStoreSinkWriteState(subtaskId, stateFilter);
+                        return new NoopStoreSinkWriteState(subtaskId);
                     }
 
                     @Override
@@ -99,7 +99,7 @@ public abstract class FlinkWriteSink<T> extends FlinkSink<T> {
                             throws Exception {
                         // No conflicts will occur in append only unaware bucket writer, so
                         // commitUser does not matter.
-                        return commitUser;
+                        return commitUser == null ? initialCommitUser : commitUser;
                     }
                 };
             }
