@@ -21,6 +21,7 @@ package org.apache.paimon.flink.source;
 import org.apache.paimon.CoreOptions;
 import org.apache.paimon.CoreOptions.StartupMode;
 import org.apache.paimon.CoreOptions.StreamingReadMode;
+import org.apache.paimon.annotation.VisibleForTesting;
 import org.apache.paimon.flink.FlinkConnectorOptions;
 import org.apache.paimon.flink.NestedProjectedRowData;
 import org.apache.paimon.flink.Projection;
@@ -30,6 +31,7 @@ import org.apache.paimon.flink.source.align.AlignedContinuousFileStoreSource;
 import org.apache.paimon.flink.source.operator.MonitorSource;
 import org.apache.paimon.flink.utils.TableScanUtils;
 import org.apache.paimon.options.Options;
+import org.apache.paimon.partition.PartitionPredicate;
 import org.apache.paimon.predicate.Predicate;
 import org.apache.paimon.table.BucketMode;
 import org.apache.paimon.table.FileStoreTable;
@@ -83,12 +85,13 @@ public class FlinkSourceBuilder {
 
     private final Table table;
     private final Options conf;
-    private final boolean unawareBucket;
+    private final boolean unordered;
     private String sourceName;
     private Boolean sourceBounded;
     private StreamExecutionEnvironment env;
     @Nullable private int[][] projectedFields;
     @Nullable private Predicate predicate;
+    @Nullable private PartitionPredicate partitionPredicate;
     @Nullable private LogSourceProvider logSourceProvider;
     @Nullable private Integer parallelism;
     @Nullable private Long limit;
@@ -97,11 +100,28 @@ public class FlinkSourceBuilder {
 
     public FlinkSourceBuilder(Table table) {
         this.table = table;
-        this.unawareBucket =
-                table instanceof FileStoreTable
-                        && ((FileStoreTable) table).bucketMode() == BucketMode.BUCKET_UNAWARE;
         this.sourceName = table.name();
         this.conf = Options.fromMap(table.options());
+        this.unordered = unordered(table);
+    }
+
+    private static boolean unordered(Table table) {
+        if (!(table instanceof FileStoreTable)) {
+            return false;
+        }
+
+        if (!table.primaryKeys().isEmpty()) {
+            return false;
+        }
+
+        BucketMode bucketMode = ((FileStoreTable) table).bucketMode();
+        if (bucketMode == BucketMode.BUCKET_UNAWARE) {
+            return true;
+        } else if (bucketMode == BucketMode.HASH_FIXED) {
+            return !Options.fromMap(table.options()).get(CoreOptions.BUCKET_APPEND_ORDERED);
+        }
+
+        return false;
     }
 
     public FlinkSourceBuilder env(StreamExecutionEnvironment env) {
@@ -136,6 +156,11 @@ public class FlinkSourceBuilder {
         return this;
     }
 
+    public FlinkSourceBuilder partitionPredicate(PartitionPredicate partitionPredicate) {
+        this.partitionPredicate = partitionPredicate;
+        return this;
+    }
+
     public FlinkSourceBuilder limit(@Nullable Long limit) {
         this.limit = limit;
         return this;
@@ -179,7 +204,12 @@ public class FlinkSourceBuilder {
         if (readType != null) {
             readBuilder.withReadType(readType);
         }
-        readBuilder.withFilter(predicate);
+        if (predicate != null) {
+            readBuilder.withFilter(predicate);
+        }
+        if (partitionPredicate != null) {
+            readBuilder.withPartitionFilter(partitionPredicate);
+        }
         if (limit != null) {
             readBuilder.withLimit(limit.intValue());
         }
@@ -204,7 +234,7 @@ public class FlinkSourceBuilder {
                         createReadBuilder(projectedRowType()),
                         table.options(),
                         limit,
-                        unawareBucket,
+                        unordered,
                         outerProject()));
     }
 
@@ -215,7 +245,7 @@ public class FlinkSourceBuilder {
                         createReadBuilder(projectedRowType()),
                         table.options(),
                         limit,
-                        unawareBucket,
+                        unordered,
                         outerProject()));
     }
 
@@ -266,6 +296,11 @@ public class FlinkSourceBuilder {
                 .orElse(null);
     }
 
+    @VisibleForTesting
+    public boolean isUnordered() {
+        return unordered;
+    }
+
     /** Build source {@link DataStream} with {@link RowData}. */
     public DataStream<Row> buildForRow() {
         DataType rowType = fromLogicalToDataType(toLogicalType(table.rowType()));
@@ -317,6 +352,7 @@ public class FlinkSourceBuilder {
                                                 table,
                                                 projectedRowType(),
                                                 predicate,
+                                                partitionPredicate,
                                                 outerProject()))
                                 .addSource(
                                         new LogHybridSourceFactory(logSourceProvider),
@@ -351,7 +387,7 @@ public class FlinkSourceBuilder {
                         conf.get(CoreOptions.CONTINUOUS_DISCOVERY_INTERVAL).toMillis(),
                         watermarkStrategy == null,
                         conf.get(FlinkConnectorOptions.READ_SHUFFLE_BUCKET_WITH_PARTITION),
-                        unawareBucket,
+                        unordered,
                         outerProject(),
                         isBounded,
                         limit);
