@@ -23,9 +23,12 @@ import org.apache.paimon.table.FileStoreTable;
 import org.apache.paimon.utils.BlockingIterator;
 import org.apache.paimon.utils.SnapshotManager;
 
-import org.apache.commons.lang3.StringUtils;
+import org.apache.paimon.shade.org.apache.commons.lang3.StringUtils;
+
 import org.apache.flink.types.Row;
+import org.apache.flink.types.RowKind;
 import org.apache.flink.util.CloseableIterator;
+import org.assertj.core.api.AssertionsForInterfaceTypes;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.Timeout;
 
@@ -49,6 +52,125 @@ public class BranchSqlITCase extends CatalogITCaseBase {
                 .containsExactlyInAnyOrder("+I[1, 5]", "+I[2, 5]");
     }
 
+    @Test
+    public void testUnsupportedDefaultValue() {
+        sql("CREATE TABLE T (a INT, b INT)");
+        assertThatThrownBy(
+                        () -> sql("CALL sys.alter_column_default_value('default.T', 'b', 'ddd')"))
+                .hasMessageContaining("Unsupported default value `ddd` for type INT");
+    }
+
+    @Test
+    public void testArrayDefaultValue() throws Exception {
+        sql("CREATE TABLE T_ARRAY (id INT, tags ARRAY<STRING>, numbers ARRAY<INT>)");
+        sql("CALL sys.alter_column_default_value('default.T_ARRAY', 'tags', '[tag1, tag2]')");
+        sql("CALL sys.alter_column_default_value('default.T_ARRAY', 'numbers', '[1, 2, 3]')");
+
+        sql("INSERT INTO T_ARRAY (id) VALUES (1), (2)");
+        assertThat(collectResult("SELECT * FROM T_ARRAY"))
+                .containsExactlyInAnyOrder(
+                        "+I[1, [tag1, tag2], [1, 2, 3]]", "+I[2, [tag1, tag2], [1, 2, 3]]");
+
+        sql("CREATE TABLE T_EMPTY_ARRAY (id INT, empty_tags ARRAY<STRING>)");
+        sql("CALL sys.alter_column_default_value('default.T_EMPTY_ARRAY', 'empty_tags', '[]')");
+        sql("INSERT INTO T_EMPTY_ARRAY (id) VALUES (1)");
+        assertThat(collectResult("SELECT * FROM T_EMPTY_ARRAY"))
+                .containsExactlyInAnyOrder("+I[1, []]");
+    }
+
+    @Test
+    public void testMapDefaultValue() throws Exception {
+        sql("CREATE TABLE T_MAP (id INT, properties MAP<STRING, STRING>)");
+        sql(
+                "CALL sys.alter_column_default_value('default.T_MAP', 'properties', '{key1 -> value1, key2 -> value2}')");
+
+        sql("INSERT INTO T_MAP (id) VALUES (1), (2)");
+        assertThat(collectResult("SELECT * FROM T_MAP"))
+                .containsExactlyInAnyOrder(
+                        "+I[1, {key1=value1, key2=value2}]", "+I[2, {key1=value1, key2=value2}]");
+
+        sql("CREATE TABLE T_EMPTY_MAP (id INT, empty_map MAP<STRING, INT>)");
+        sql("CALL sys.alter_column_default_value('default.T_EMPTY_MAP', 'empty_map', '{}')");
+        sql("INSERT INTO T_EMPTY_MAP (id) VALUES (1)");
+        assertThat(collectResult("SELECT * FROM T_EMPTY_MAP"))
+                .containsExactlyInAnyOrder("+I[1, {}]");
+    }
+
+    @Test
+    public void testRowDefaultValue() throws Exception {
+        sql("CREATE TABLE T_STRUCT (id INT, nested ROW<x INT, y STRING>)");
+        sql(
+                "CALL sys.alter_column_default_value('default.T_STRUCT', 'nested', '{42, default_value}')");
+
+        sql("INSERT INTO T_STRUCT (id) VALUES (1), (2)");
+        assertThat(collectResult("SELECT * FROM T_STRUCT"))
+                .containsExactlyInAnyOrder(
+                        "+I[1, +I[42, default_value]]", "+I[2, +I[42, default_value]]");
+
+        sql(
+                "CREATE TABLE T_COMPLEX_STRUCT (id INT, config ROW<enabled BOOLEAN, timeout INT, name STRING>)");
+        sql(
+                "CALL sys.alter_column_default_value('default.T_COMPLEX_STRUCT', 'config', '{true, 30, config_name}')");
+        sql("INSERT INTO T_COMPLEX_STRUCT (id) VALUES (1)");
+        assertThat(collectResult("SELECT * FROM T_COMPLEX_STRUCT"))
+                .containsExactlyInAnyOrder("+I[1, +I[true, 30, config_name]]");
+    }
+
+    @Test
+    public void testMixedComplexDefaultValue() throws Exception {
+        sql(
+                "CREATE TABLE T_MIXED (id INT, name STRING, tags ARRAY<STRING>, metadata MAP<STRING, STRING>, config ROW<enabled BOOLEAN, timeout INT>)");
+        sql("CALL sys.alter_column_default_value('default.T_MIXED', 'name', 'default_name')");
+        sql("CALL sys.alter_column_default_value('default.T_MIXED', 'tags', '[default_tag]')");
+        sql(
+                "CALL sys.alter_column_default_value('default.T_MIXED', 'metadata', '{created_by -> system}')");
+        sql("CALL sys.alter_column_default_value('default.T_MIXED', 'config', '{true, 30}')");
+
+        sql("INSERT INTO T_MIXED (id) VALUES (1)");
+        assertThat(collectResult("SELECT * FROM T_MIXED"))
+                .containsExactlyInAnyOrder(
+                        "+I[1, default_name, [default_tag], {created_by=system}, +I[true, 30]]");
+
+        sql("INSERT INTO T_MIXED (id, name) VALUES (2, 'custom_name')");
+        assertThat(collectResult("SELECT * FROM T_MIXED WHERE id = 2"))
+                .containsExactlyInAnyOrder(
+                        "+I[2, custom_name, [default_tag], {created_by=system}, +I[true, 30]]");
+    }
+
+    @Test
+    public void testNestedComplexDefaultValue() throws Exception {
+        sql(
+                "CREATE TABLE T_NESTED ("
+                        + "id INT, "
+                        + "nested_array ARRAY<ROW<name STRING, val INT>>, "
+                        + "nested_struct ROW<info ROW<x INT, y STRING>, enabled BOOLEAN>"
+                        + ")");
+
+        sql(
+                "CALL sys.alter_column_default_value('default.T_NESTED', 'nested_array', '[{item1, 10}, {item2, 20}]')");
+        sql(
+                "CALL sys.alter_column_default_value('default.T_NESTED', 'nested_struct', '{{42, nested_value}, true}')");
+
+        sql("INSERT INTO T_NESTED (id) VALUES (1)");
+        assertThat(collectResult("SELECT * FROM T_NESTED"))
+                .containsExactlyInAnyOrder(
+                        "+I[1, [+I[item1, 10], +I[item2, 20]], +I[+I[42, nested_value], true]]");
+
+        sql("INSERT INTO T_NESTED (id, nested_array) VALUES (2, ARRAY[ROW('custom', 99)])");
+        assertThat(collectResult("SELECT * FROM T_NESTED WHERE id = 2"))
+                .containsExactlyInAnyOrder(
+                        "+I[2, [+I[custom, 99]], +I[+I[42, nested_value], true]]");
+    }
+
+    @Test
+    public void testDefaultValueForPkTableDynamicBucket() throws Exception {
+        sql("CREATE TABLE T (a INT PRIMARY KEY NOT ENFORCED, b INT)");
+        sql("CALL sys.alter_column_default_value('default.T', 'b', '5')");
+        sql("INSERT INTO T (a) VALUES (1), (2)");
+        assertThat(collectResult("SELECT * FROM T"))
+                .containsExactlyInAnyOrder("+I[1, 5]", "+I[2, 5]");
+    }
+
     @Test
     public void testAlterBranchTable() throws Exception {
         sql(
@@ -353,6 +475,32 @@ public class BranchSqlITCase extends CatalogITCaseBase {
                         "+I[cat, 20]", "+I[dog, 30]", "+I[lion, 10]", "+I[wolf, 20]");
     }
 
+    @Test
+    public void testFallbackBranchStreamRead() throws Exception {
+        sql(
+                "CREATE TABLE t ( pt INT NOT NULL, k INT NOT NULL, v STRING ) PARTITIONED BY (pt) WITH ( 'bucket' = '-1' )");
+        sql("INSERT INTO t VALUES (1, 10, 'apple'), (1, 20, 'banana')");
+
+        sql("CALL sys.create_branch('default.t', 'pk')");
+        sql("ALTER TABLE `t$branch_pk` SET ( 'primary-key' = 'pt, k', 'bucket' = '2' )");
+        sql("ALTER TABLE t SET ( 'scan.fallback-branch' = 'pk' )");
+
+        sql("INSERT INTO `t$branch_pk` VALUES (2, 20, 'cat'), (2, 30, 'dog')");
+
+        // read main branch in batch
+        assertThat(collectResult("SELECT v, k FROM t"))
+                .containsExactlyInAnyOrder(
+                        "+I[apple, 10]", "+I[banana, 20]", "+I[cat, 20]", "+I[dog, 30]");
+
+        // read main branch streamingly
+        try (BlockingIterator<Row, Row> iter = streamSqlBlockIter("SELECT * FROM t")) {
+            AssertionsForInterfaceTypes.assertThat(iter.collect(2))
+                    .containsExactlyInAnyOrder(
+                            Row.ofKind(RowKind.INSERT, 1, 10, "apple"),
+                            Row.ofKind(RowKind.INSERT, 1, 20, "banana"));
+        }
+    }
+
     @Test
     public void testCrossPartitionFallbackBranchBatchRead() throws Exception {
         sql(
@@ -686,6 +834,41 @@ public class BranchSqlITCase extends CatalogITCaseBase {
                         "+I[2, 220, 2200]");
     }
 
+    @Test
+    public void testBranchesTableFilter() throws Exception {
+        sql("CREATE TABLE T (a INT, b INT)");
+        sql("INSERT INTO T VALUES (1, 1)");
+        sql("CALL sys.create_branch('default.T', 'b1')");
+        sql("CALL sys.create_branch('default.T', 'b2')");
+        sql("CALL sys.create_branch('default.T', 'b3')");
+
+        // no filter
+        assertThat(collectResult("SELECT branch_name FROM `T$branches`"))
+                .containsExactlyInAnyOrder("+I[b1]", "+I[b2]", "+I[b3]");
+
+        // equals
+        assertThat(collectResult("SELECT branch_name FROM `T$branches` WHERE branch_name = 'b2'"))
+                .containsExactlyInAnyOrder("+I[b2]");
+
+        // in
+        assertThat(
+                        collectResult(
+                                "SELECT branch_name FROM `T$branches` WHERE branch_name IN ('b1', 'b3')"))
+                .containsExactlyInAnyOrder("+I[b1]", "+I[b3]");
+
+        // in with non-existent branch
+        assertThat(
+                        collectResult(
+                                "SELECT branch_name FROM `T$branches` WHERE branch_name IN ('b1', 'non_existent')"))
+                .containsExactlyInAnyOrder("+I[b1]");
+
+        // equals with non-existent branch
+        assertThat(
+                        collectResult(
+                                "SELECT branch_name FROM `T$branches` WHERE branch_name = 'non_existent'"))
+                .isEmpty();
+    }
+
     private List<String> collectResult(String sql) throws Exception {
         List<String> result = new ArrayList<>();
         try (CloseableIterator<Row> it = tEnv.executeSql(sql).collect()) {
