@@ -23,15 +23,19 @@ import org.apache.paimon.table.system.AllTableOptionsTable;
 import org.apache.paimon.table.system.CatalogOptionsTable;
 import org.apache.paimon.utils.BlockingIterator;
 
-import org.apache.commons.lang3.StringUtils;
+import org.apache.paimon.shade.org.apache.commons.lang3.StringUtils;
+
 import org.apache.flink.table.catalog.CatalogPartition;
 import org.apache.flink.table.catalog.CatalogPartitionSpec;
 import org.apache.flink.table.catalog.ObjectPath;
 import org.apache.flink.table.catalog.exceptions.PartitionNotExistException;
 import org.apache.flink.table.catalog.exceptions.TableNotPartitionedException;
 import org.apache.flink.types.Row;
+import org.apache.flink.types.variant.Variant;
+import org.apache.flink.types.variant.VariantBuilder;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.Timeout;
+import org.junit.jupiter.api.condition.EnabledIf;
 
 import javax.annotation.Nonnull;
 
@@ -954,9 +958,9 @@ public class CatalogTableITCase extends CatalogITCaseBase {
                 BlockingIterator.of(
                         streamSqlIter(
                                 "SELECT * FROM T /*+ OPTIONS('consumer-id'='my1','consumer.expiration-time'='3h') */"));
+        assertThat(iterator.collect(2)).containsExactlyInAnyOrder(Row.of(1, 2), Row.of(3, 4));
 
         batchSql("INSERT INTO T VALUES (5, 6), (7, 8)");
-        assertThat(iterator.collect(2)).containsExactlyInAnyOrder(Row.of(1, 2), Row.of(3, 4));
 
         List<Row> result;
         do {
@@ -1168,6 +1172,26 @@ public class CatalogTableITCase extends CatalogITCaseBase {
         iterator.close();
     }
 
+    @Test
+    public void testBinlogTableStreamReadWithProjection() throws Exception {
+        sql(
+                "CREATE TABLE T (a INT, b INT, primary key (a) NOT ENFORCED) with ('changelog-producer' = 'lookup', "
+                        + "'bucket' = '2')");
+        BlockingIterator<Row, Row> iterator =
+                streamSqlBlockIter(
+                        "SELECT rowkind, a FROM T$binlog /*+ OPTIONS('scan.mode' = 'latest') */");
+        sql("INSERT INTO T VALUES (1, 2)");
+        sql("INSERT INTO T VALUES (1, 3)");
+        sql("INSERT INTO T VALUES (2, 2)");
+        List<Row> rows = iterator.collect(3);
+        assertThat(rows)
+                .containsExactly(
+                        Row.of("+I", new Integer[] {1}),
+                        Row.of("+U", new Integer[] {1, 1}),
+                        Row.of("+I", new Integer[] {2}));
+        iterator.close();
+    }
+
     @Test
     public void testBinlogTableBatchRead() throws Exception {
         sql(
@@ -1218,6 +1242,24 @@ public class CatalogTableITCase extends CatalogITCaseBase {
         assertThat(row.getField(6)).isNotNull();
     }
 
+    @Test
+    @EnabledIf("isFlink2_1OrAbove")
+    void testReadWriteVariant() {
+        sql("CREATE TABLE t (v VARIANT)");
+
+        sql(
+                "INSERT INTO t SELECT PARSE_JSON(s) FROM (VALUES ('{\"a\":1}'), ('{\"a\":2}'), ('\"hello\"')) AS T(s)");
+
+        List<Row> rows = sql("SELECT * FROM t");
+
+        VariantBuilder builder = Variant.newBuilder();
+        assertThat(rows)
+                .containsExactlyInAnyOrder(
+                        Row.of(builder.object().add("a", builder.of((byte) 1)).build()),
+                        Row.of(builder.object().add("a", builder.of((byte) 2)).build()),
+                        Row.of(builder.of("hello")));
+    }
+
     private void innerTestReadOptimizedTableAndCheckData(String insertTableName) {
         // full compaction will always be performed at the end of batch jobs, as long as
         // full-compaction.delta-commits is set, regardless of its value
