@@ -18,16 +18,22 @@
 
 package org.apache.paimon.flink;
 
+import org.apache.paimon.fs.Path;
+import org.apache.paimon.fs.local.LocalFileIO;
 import org.apache.paimon.utils.BlockingIterator;
+import org.apache.paimon.utils.TraceableFileIO;
 
 import org.apache.flink.types.Row;
 import org.apache.flink.types.RowKind;
 import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.io.TempDir;
 import org.junit.jupiter.params.ParameterizedTest;
 import org.junit.jupiter.params.provider.Arguments;
 import org.junit.jupiter.params.provider.MethodSource;
 import org.junit.jupiter.params.provider.ValueSource;
 
+import java.io.IOException;
+import java.util.Arrays;
 import java.util.stream.Stream;
 
 import static org.assertj.core.api.AssertionsForInterfaceTypes.assertThat;
@@ -35,6 +41,8 @@ import static org.assertj.core.api.AssertionsForInterfaceTypes.assertThat;
 /** ITCase for deletion vector table. */
 public class DeletionVectorITCase extends CatalogITCaseBase {
 
+    @TempDir java.nio.file.Path tempExternalPath;
+
     private static Stream<Arguments> parameters1() {
         // parameters: changelogProducer, dvBitmap64
         return Stream.of(
@@ -70,14 +78,14 @@ public class DeletionVectorITCase extends CatalogITCaseBase {
         try (BlockingIterator<Row, Row> iter =
                 streamSqlBlockIter(
                         "SELECT * FROM T /*+ OPTIONS('scan.mode'='from-snapshot-full','scan.snapshot-id' = '3') */")) {
-            assertThat(iter.collect(8))
+
+            // the first two values will be merged
+            assertThat(iter.collect(6))
                     .containsExactlyInAnyOrder(
                             Row.ofKind(RowKind.INSERT, 1, "111111111"),
-                            Row.ofKind(RowKind.INSERT, 2, "2"),
-                            Row.ofKind(RowKind.INSERT, 3, "3"),
-                            Row.ofKind(RowKind.INSERT, 4, "4"),
                             Row.ofKind(RowKind.INSERT, 2, "2_1"),
                             Row.ofKind(RowKind.INSERT, 3, "3_1"),
+                            Row.ofKind(RowKind.INSERT, 4, "4"),
                             Row.ofKind(RowKind.INSERT, 2, "2_2"),
                             Row.ofKind(RowKind.INSERT, 4, "4_1"));
         }
@@ -118,20 +126,34 @@ public class DeletionVectorITCase extends CatalogITCaseBase {
         try (BlockingIterator<Row, Row> iter =
                 streamSqlBlockIter(
                         "SELECT * FROM T /*+ OPTIONS('scan.mode'='from-snapshot-full','scan.snapshot-id' = '3') */")) {
-            assertThat(iter.collect(12))
-                    .containsExactlyInAnyOrder(
-                            Row.ofKind(RowKind.INSERT, 1, "111111111"),
-                            Row.ofKind(RowKind.INSERT, 2, "2"),
-                            Row.ofKind(RowKind.INSERT, 3, "3"),
-                            Row.ofKind(RowKind.INSERT, 4, "4"),
-                            Row.ofKind(RowKind.UPDATE_BEFORE, 2, "2"),
-                            Row.ofKind(RowKind.UPDATE_AFTER, 2, "2_1"),
-                            Row.ofKind(RowKind.UPDATE_BEFORE, 3, "3"),
-                            Row.ofKind(RowKind.UPDATE_AFTER, 3, "3_1"),
-                            Row.ofKind(RowKind.UPDATE_BEFORE, 2, "2_1"),
-                            Row.ofKind(RowKind.UPDATE_AFTER, 2, "2_2"),
-                            Row.ofKind(RowKind.UPDATE_BEFORE, 4, "4"),
-                            Row.ofKind(RowKind.UPDATE_AFTER, 4, "4_1"));
+            if (changelogProducer.equals("none")) {
+                // the first two values will be merged
+                assertThat(iter.collect(8))
+                        .containsExactlyInAnyOrder(
+                                Row.ofKind(RowKind.INSERT, 1, "111111111"),
+                                Row.ofKind(RowKind.INSERT, 2, "2_1"),
+                                Row.ofKind(RowKind.INSERT, 3, "3_1"),
+                                Row.ofKind(RowKind.INSERT, 4, "4"),
+                                Row.ofKind(RowKind.UPDATE_BEFORE, 2, "2_1"),
+                                Row.ofKind(RowKind.UPDATE_AFTER, 2, "2_2"),
+                                Row.ofKind(RowKind.UPDATE_BEFORE, 4, "4"),
+                                Row.ofKind(RowKind.UPDATE_AFTER, 4, "4_1"));
+            } else {
+                assertThat(iter.collect(12))
+                        .containsExactlyInAnyOrder(
+                                Row.ofKind(RowKind.INSERT, 1, "111111111"),
+                                Row.ofKind(RowKind.INSERT, 2, "2"),
+                                Row.ofKind(RowKind.INSERT, 3, "3"),
+                                Row.ofKind(RowKind.INSERT, 4, "4"),
+                                Row.ofKind(RowKind.UPDATE_BEFORE, 2, "2"),
+                                Row.ofKind(RowKind.UPDATE_AFTER, 2, "2_1"),
+                                Row.ofKind(RowKind.UPDATE_BEFORE, 3, "3"),
+                                Row.ofKind(RowKind.UPDATE_AFTER, 3, "3_1"),
+                                Row.ofKind(RowKind.UPDATE_BEFORE, 2, "2_1"),
+                                Row.ofKind(RowKind.UPDATE_AFTER, 2, "2_2"),
+                                Row.ofKind(RowKind.UPDATE_BEFORE, 4, "4"),
+                                Row.ofKind(RowKind.UPDATE_AFTER, 4, "4_1"));
+            }
         }
 
         // test read from COMPACT snapshot
@@ -380,4 +402,96 @@ public class DeletionVectorITCase extends CatalogITCaseBase {
         sql("ALTER TABLE TT SET('deletion-vectors.enabled' = 'false')");
         assertThat(sql("SELECT * FROM TT").size()).isEqualTo(5);
     }
+
+    // No compaction to verify that level0 data can be read at full phase
+    @ParameterizedTest
+    @ValueSource(booleans = {true, false})
+    public void testStreamingReadFullWithoutCompact(boolean isPk) throws Exception {
+        if (isPk) {
+            sql(
+                    "CREATE TABLE T (a INT PRIMARY KEY NOT ENFORCED, b INT) "
+                            + "WITH ('deletion-vectors.enabled' = 'true', 'changelog-producer' = 'none', 'write-only' = 'true')");
+        } else {
+            sql(
+                    "CREATE TABLE T (a INT, b INT) WITH ('deletion-vectors.enabled' = 'true', 'write-only' = 'true')");
+        }
+
+        sql("INSERT INTO T VALUES (1, 1)");
+        sql("INSERT INTO T VALUES (2, 2)");
+        sql("INSERT INTO T VALUES (3, 3)");
+
+        try (BlockingIterator<Row, Row> iter =
+                streamSqlBlockIter(
+                        "SELECT * FROM T /*+ OPTIONS('scan.mode' = 'from-snapshot-full', 'scan.snapshot-id' = '2') */")) {
+            assertThat(iter.collect(3))
+                    .containsExactlyInAnyOrder(Row.of(1, 1), Row.of(2, 2), Row.of(3, 3));
+        }
+    }
+
+    @Test
+    public void testIndexFileInDataFileDir() throws IOException {
+        sql(
+                "CREATE TABLE IT (a INT PRIMARY KEY NOT ENFORCED, b INT) WITH ("
+                        + "'deletion-vectors.enabled' = 'true', "
+                        + "'index-file-in-data-file-dir' = 'true')");
+        sql("INSERT INTO IT VALUES (1, 1)");
+        assertThat(sql("SELECT * FROM IT")).containsExactly(Row.of(1, 1));
+        Path path = getTableDirectory("IT");
+        LocalFileIO fileIO = LocalFileIO.create();
+        String result = Arrays.asList(fileIO.listFiles(path, true)).toString();
+        assertThat(result).contains("default.db/IT/bucket-0/index-");
+        assertThat(result).doesNotContain("default.db/IT/index/index-");
+    }
+
+    @Test
+    public void testIndexFileInIndexDir() throws IOException {
+        sql(
+                "CREATE TABLE IT (a INT PRIMARY KEY NOT ENFORCED, b INT) WITH ("
+                        + "'deletion-vectors.enabled' = 'true')");
+        sql("INSERT INTO IT (a, b) VALUES (1, 1)");
+        assertThat(sql("SELECT * FROM IT")).containsExactly(Row.of(1, 1));
+        Path path = getTableDirectory("IT");
+        LocalFileIO fileIO = LocalFileIO.create();
+        String result = Arrays.asList(fileIO.listFiles(path, true)).toString();
+        assertThat(result).doesNotContain("default.db/IT/bucket-0/index-");
+        assertThat(result).contains("default.db/IT/index/index-");
+    }
+
+    @Test
+    public void testIndexFileInDataFileDirWithExternalPath() throws IOException {
+        String externalPaths = TraceableFileIO.SCHEME + "://" + tempExternalPath.toString();
+        sql(
+                "CREATE TABLE IT (a INT PRIMARY KEY NOT ENFORCED, b INT) WITH ("
+                        + "'deletion-vectors.enabled' = 'true', "
+                        + "'index-file-in-data-file-dir' = 'true', "
+                        + "'data-file.external-paths.strategy' = 'round-robin', "
+                        + String.format("'data-file.external-paths' = '%s')", externalPaths));
+        sql("INSERT INTO IT (a, b) VALUES (1, 1)");
+        assertThat(sql("SELECT * FROM IT")).containsExactly(Row.of(1, 1));
+        LocalFileIO fileIO = LocalFileIO.create();
+
+        Path path = getTableDirectory("IT");
+        String inTablePath = Arrays.asList(fileIO.listFiles(path, true)).toString();
+        assertThat(inTablePath).doesNotContain("bucket-0/index-");
+        assertThat(inTablePath).doesNotContain("index/index-");
+
+        Path externalPath = new Path(externalPaths);
+        String inExternalPath = Arrays.asList(fileIO.listFiles(externalPath, true)).toString();
+        assertThat(inExternalPath).contains("bucket-0/index-");
+        assertThat(inExternalPath).doesNotContain("index/index-");
+    }
+
+    @Test
+    public void testLookupMergeBufferSize() {
+        sql(
+                "CREATE TABLE T (id INT PRIMARY KEY NOT ENFORCED, name STRING) "
+                        + "WITH ('deletion-vectors.enabled' = 'true', 'lookup.merge-records-threshold' = '2')");
+        for (int i = 0; i < 5; i++) {
+            sql(
+                    String.format(
+                            "INSERT INTO T /*+ OPTIONS('write-only' = '%s') */ VALUES (1, '%s')",
+                            i != 4, i));
+        }
+        assertThat(sql("SELECT * FROM T")).containsExactly(Row.of(1, String.valueOf(4)));
+    }
 }
