@@ -22,12 +22,16 @@ import org.apache.paimon.CoreOptions;
 import org.apache.paimon.Snapshot;
 import org.apache.paimon.data.BinaryString;
 import org.apache.paimon.flink.FlinkConnectorOptions;
+import org.apache.paimon.fs.Path;
+import org.apache.paimon.fs.local.LocalFileIO;
+import org.apache.paimon.schema.SchemaChange;
 import org.apache.paimon.table.FileStoreTable;
 import org.apache.paimon.table.source.DataSplit;
 import org.apache.paimon.table.source.StreamTableScan;
 import org.apache.paimon.table.source.TableScan;
 import org.apache.paimon.utils.CommonTestUtils;
 import org.apache.paimon.utils.SnapshotManager;
+import org.apache.paimon.utils.TraceableFileIO;
 
 import org.apache.paimon.shade.guava30.com.google.common.collect.Lists;
 
@@ -91,6 +95,56 @@ public class CompactActionITCase extends CompactActionITCaseBase {
         }
     }
 
+    @Test
+    @Timeout(60)
+    public void testCompactWhenSkipLevel0() throws Exception {
+        Map<String, String> tableOptions = new HashMap<>();
+        tableOptions.put(CoreOptions.WRITE_ONLY.key(), "true");
+        // in dv mode or merge-engine = first-row, batch read will skip level-0
+        if (ThreadLocalRandom.current().nextBoolean()) {
+            tableOptions.put(CoreOptions.DELETION_VECTORS_ENABLED.key(), "true");
+        } else {
+            tableOptions.put(CoreOptions.MERGE_ENGINE.key(), "first-row");
+        }
+        tableOptions.put(CoreOptions.CHANGELOG_PRODUCER.key(), "lookup");
+
+        FileStoreTable table =
+                prepareTable(
+                        Arrays.asList("dt", "hh"),
+                        Arrays.asList("dt", "hh", "k"),
+                        Collections.emptyList(),
+                        tableOptions);
+
+        writeData(
+                rowData(1, 100, 15, BinaryString.fromString("20221208")),
+                rowData(1, 100, 16, BinaryString.fromString("20221208")),
+                rowData(1, 100, 15, BinaryString.fromString("20221209")));
+
+        writeData(
+                rowData(2, 100, 15, BinaryString.fromString("20221208")),
+                rowData(2, 100, 16, BinaryString.fromString("20221208")),
+                rowData(2, 100, 15, BinaryString.fromString("20221209")));
+
+        checkLatestSnapshot(table, 2, Snapshot.CommitKind.APPEND);
+        assertThat(table.newScan().plan().splits().size()).isEqualTo(0);
+
+        runAction(false);
+
+        checkLatestSnapshot(table, 3, Snapshot.CommitKind.COMPACT);
+
+        List<DataSplit> splits = table.newSnapshotReader().read().dataSplits();
+        assertThat(splits.size()).isEqualTo(3);
+        for (DataSplit split : splits) {
+            if (split.partition().getInt(1) == 15) {
+                // compacted
+                assertThat(split.dataFiles().size()).isEqualTo(1);
+            } else {
+                // not compacted
+                assertThat(split.dataFiles().size()).isEqualTo(2);
+            }
+        }
+    }
+
     @Test
     public void testStreamingCompact() throws Exception {
         Map<String, String> tableOptions = new HashMap<>();
@@ -233,6 +287,86 @@ public class CompactActionITCase extends CompactActionITCaseBase {
         }
     }
 
+    @Test
+    public void testStreamingCompactWithChangedExternalPath() throws Exception {
+        String externalPath1 = getTempDirPath();
+        String externalPath2 = getTempDirPath();
+
+        Map<String, String> tableOptions = new HashMap<>();
+        tableOptions.put(CoreOptions.CHANGELOG_PRODUCER.key(), "full-compaction");
+        tableOptions.put(CoreOptions.FULL_COMPACTION_DELTA_COMMITS.key(), "1");
+        tableOptions.put(CoreOptions.CONTINUOUS_DISCOVERY_INTERVAL.key(), "1s");
+        tableOptions.put(
+                CoreOptions.DATA_FILE_EXTERNAL_PATHS.key(),
+                TraceableFileIO.SCHEME + "://" + externalPath1);
+        tableOptions.put(CoreOptions.DATA_FILE_EXTERNAL_PATHS_STRATEGY.key(), "round-robin");
+        tableOptions.put(
+                FlinkConnectorOptions.SINK_WRITER_REFRESH_DETECTORS.key(), "external-paths");
+        tableOptions.put(CoreOptions.WRITE_ONLY.key(), "true");
+
+        FileStoreTable table =
+                prepareTable(
+                        Arrays.asList("dt", "hh"),
+                        Arrays.asList("dt", "hh", "k"),
+                        Collections.emptyList(),
+                        tableOptions);
+
+        // base records
+        writeData(
+                rowData(1, 100, 15, BinaryString.fromString("20221208")),
+                rowData(1, 100, 16, BinaryString.fromString("20221208")),
+                rowData(1, 100, 15, BinaryString.fromString("20221209")));
+
+        checkLatestSnapshot(table, 1, Snapshot.CommitKind.APPEND);
+
+        // no full compaction has happened, so plan should be empty
+        StreamTableScan scan = table.newReadBuilder().newStreamScan();
+        TableScan.Plan plan = scan.plan();
+        assertThat(plan.splits()).isEmpty();
+
+        runAction(true);
+
+        // first full compaction
+        validateResult(
+                table,
+                ROW_TYPE,
+                scan,
+                Arrays.asList("+I[1, 100, 15, 20221208]", "+I[1, 100, 15, 20221209]"),
+                60_000);
+        LocalFileIO fileIO = LocalFileIO.create();
+        assertThat(fileIO.exists(new Path(externalPath2))).isFalse();
+        assertThat(fileIO.listStatus(new Path(externalPath1)).length).isGreaterThanOrEqualTo(1);
+
+        SchemaChange schemaChange =
+                SchemaChange.setOption(
+                        CoreOptions.DATA_FILE_EXTERNAL_PATHS.key(),
+                        TraceableFileIO.SCHEME + "://" + externalPath2);
+        table.schemaManager().commitChanges(schemaChange);
+        // wait a checkpoint at least to wait the writer refresh
+        Thread.sleep(1000);
+
+        // incremental records
+        writeData(
+                rowData(1, 101, 15, BinaryString.fromString("20221208")),
+                rowData(1, 101, 16, BinaryString.fromString("20221208")),
+                rowData(1, 101, 15, BinaryString.fromString("20221209")));
+
+        // second full compaction
+        validateResult(
+                table,
+                ROW_TYPE,
+                scan,
+                Arrays.asList(
+                        "+U[1, 101, 15, 20221208]",
+                        "+U[1, 101, 15, 20221209]",
+                        "-U[1, 100, 15, 20221208]",
+                        "-U[1, 100, 15, 20221209]"),
+                60_000);
+
+        assertThat(fileIO.exists(new Path(externalPath2))).isTrue();
+        assertThat(fileIO.listStatus(new Path(externalPath2)).length).isGreaterThanOrEqualTo(1);
+    }
+
     @Test
     public void testUnawareBucketStreamingCompact() throws Exception {
         Map<String, String> tableOptions = new HashMap<>();
@@ -275,6 +409,71 @@ public class CompactActionITCase extends CompactActionITCaseBase {
         checkFileAndRowSize(table, 5L, 30_000L, 1, 9);
     }
 
+    @Test
+    public void testUnawareBucketStreamingCompactWithChangedExternalPath() throws Exception {
+        String externalPath1 = getTempDirPath();
+        String externalPath2 = getTempDirPath();
+
+        Map<String, String> tableOptions = new HashMap<>();
+        tableOptions.put(CoreOptions.CONTINUOUS_DISCOVERY_INTERVAL.key(), "1s");
+        tableOptions.put(CoreOptions.BUCKET.key(), "-1");
+        tableOptions.put(CoreOptions.COMPACTION_MIN_FILE_NUM.key(), "2");
+        tableOptions.put(CoreOptions.WRITE_ONLY.key(), "true");
+        tableOptions.put(
+                CoreOptions.DATA_FILE_EXTERNAL_PATHS.key(),
+                TraceableFileIO.SCHEME + "://" + externalPath1);
+        tableOptions.put(CoreOptions.DATA_FILE_EXTERNAL_PATHS_STRATEGY.key(), "round-robin");
+        tableOptions.put(
+                FlinkConnectorOptions.SINK_WRITER_REFRESH_DETECTORS.key(), "external-paths");
+
+        FileStoreTable table =
+                prepareTable(
+                        Collections.singletonList("k"),
+                        Collections.emptyList(),
+                        Collections.emptyList(),
+                        tableOptions);
+
+        // base records
+        writeData(
+                rowData(1, 100, 15, BinaryString.fromString("20221208")),
+                rowData(1, 100, 16, BinaryString.fromString("20221208")),
+                rowData(1, 100, 15, BinaryString.fromString("20221209")));
+
+        writeData(
+                rowData(1, 100, 15, BinaryString.fromString("20221208")),
+                rowData(1, 100, 16, BinaryString.fromString("20221208")),
+                rowData(1, 100, 15, BinaryString.fromString("20221209")));
+
+        checkLatestSnapshot(table, 2, Snapshot.CommitKind.APPEND);
+
+        // repairing that the ut don't specify the real partition of table
+        runActionForUnawareTable(true);
+
+        // first compaction, snapshot will be 3
+        checkFileAndRowSize(table, 3L, 30_000L, 1, 6);
+        LocalFileIO fileIO = LocalFileIO.create();
+        assertThat(fileIO.exists(new Path(externalPath2))).isFalse();
+        assertThat(fileIO.listStatus(new Path(externalPath1)).length).isGreaterThanOrEqualTo(1);
+
+        SchemaChange schemaChange =
+                SchemaChange.setOption(
+                        CoreOptions.DATA_FILE_EXTERNAL_PATHS.key(),
+                        TraceableFileIO.SCHEME + "://" + externalPath2);
+        table.schemaManager().commitChanges(schemaChange);
+        Thread.sleep(1000);
+
+        writeData(
+                rowData(1, 101, 15, BinaryString.fromString("20221208")),
+                rowData(1, 101, 16, BinaryString.fromString("20221208")),
+                rowData(1, 101, 15, BinaryString.fromString("20221209")));
+
+        // second compaction, snapshot will be 5
+        checkFileAndRowSize(table, 5L, 30_000L, 1, 9);
+
+        assertThat(fileIO.exists(new Path(externalPath2))).isTrue();
+        assertThat(fileIO.listStatus(new Path(externalPath2)).length).isGreaterThanOrEqualTo(1);
+    }
+
     @Test
     public void testUnawareBucketBatchCompact() throws Exception {
         Map<String, String> tableOptions = new HashMap<>();
@@ -308,6 +507,45 @@ public class CompactActionITCase extends CompactActionITCaseBase {
         checkFileAndRowSize(table, 3L, 0L, 1, 6);
     }
 
+    @Test
+    public void testLotsOfPartitionsCompact() throws Exception {
+        Map<String, String> tableOptions = new HashMap<>();
+        tableOptions.put(CoreOptions.BUCKET.key(), "-1");
+        tableOptions.put(CoreOptions.COMPACTION_MIN_FILE_NUM.key(), "2");
+
+        FileStoreTable table =
+                prepareTable(
+                        Collections.singletonList("k"),
+                        Collections.emptyList(),
+                        Collections.emptyList(),
+                        tableOptions);
+
+        // base records
+        writeData(
+                rowData(1, 100, 15, BinaryString.fromString("20221208")),
+                rowData(1, 100, 16, BinaryString.fromString("20221208")),
+                rowData(1, 100, 15, BinaryString.fromString("20221209")));
+
+        writeData(
+                rowData(1, 100, 15, BinaryString.fromString("20221208")),
+                rowData(1, 100, 16, BinaryString.fromString("20221208")),
+                rowData(1, 100, 15, BinaryString.fromString("20221209")));
+
+        checkLatestSnapshot(table, 2, Snapshot.CommitKind.APPEND);
+
+        List<String> partitions = new ArrayList<>();
+        for (int i = 0; i < 1000; i++) {
+            partitions.add("--partition");
+            partitions.add("k=" + i);
+        }
+
+        // repairing that the ut don't specify the real partition of table
+        runActionForUnawareTable(false, partitions);
+
+        // first compaction, snapshot will be 3.
+        checkFileAndRowSize(table, 3L, 0L, 1, 6);
+    }
+
     @Test
     public void testTableConf() throws Exception {
         prepareTable(
@@ -390,14 +628,20 @@ public class CompactActionITCase extends CompactActionITCaseBase {
     }
 
     private void runAction(boolean isStreaming) throws Exception {
-        runAction(isStreaming, false);
+        runAction(isStreaming, false, Collections.emptyList());
+    }
+
+    private void runActionForUnawareTable(boolean isStreaming, List<String> extra)
+            throws Exception {
+        runAction(isStreaming, true, extra);
     }
 
     private void runActionForUnawareTable(boolean isStreaming) throws Exception {
-        runAction(isStreaming, true);
+        runAction(isStreaming, true, Collections.emptyList());
     }
 
-    private void runAction(boolean isStreaming, boolean unawareBucket) throws Exception {
+    private void runAction(boolean isStreaming, boolean unawareBucket, List<String> extra)
+            throws Exception {
         StreamExecutionEnvironment env;
         if (isStreaming) {
             env = streamExecutionEnvironmentBuilder().streamingMode().build();
@@ -436,6 +680,8 @@ public class CompactActionITCase extends CompactActionITCaseBase {
             }
         }
 
+        baseArgs.addAll(extra);
+
         CompactAction action = createAction(CompactAction.class, baseArgs.toArray(new String[0]));
 
         action.withStreamExecutionEnvironment(env).build();
