@@ -20,6 +20,7 @@ package org.apache.parquet.hadoop;
 
 import org.apache.paimon.format.parquet.ParquetInputFile;
 import org.apache.paimon.format.parquet.ParquetInputStream;
+import org.apache.paimon.fs.FileRange;
 import org.apache.paimon.fs.VectoredReadable;
 import org.apache.paimon.utils.RoaringBitmap32;
 
@@ -70,11 +71,11 @@ import org.apache.parquet.internal.column.columnindex.ColumnIndex;
 import org.apache.parquet.internal.column.columnindex.OffsetIndex;
 import org.apache.parquet.internal.filter2.columnindex.ColumnIndexFilter;
 import org.apache.parquet.internal.filter2.columnindex.ColumnIndexStore;
+import org.apache.parquet.internal.filter2.columnindex.ColumnIndexStore.MissingOffsetIndexException;
 import org.apache.parquet.internal.filter2.columnindex.RowRanges;
 import org.apache.parquet.internal.hadoop.metadata.IndexReference;
 import org.apache.parquet.io.InputFile;
 import org.apache.parquet.io.ParquetDecodingException;
-import org.apache.parquet.io.ParquetFileRange;
 import org.apache.parquet.io.SeekableInputStream;
 import org.apache.parquet.schema.MessageType;
 import org.apache.parquet.schema.PrimitiveType;
@@ -329,7 +330,8 @@ public class ParquetFileReader implements Closeable {
 
     public long getFilteredRecordCount() {
         if (!options.useColumnIndexFilter()
-                || !FilterCompat.isFilteringRequired(options.getRecordFilter())) {
+                || (!FilterCompat.isFilteringRequired(options.getRecordFilter())
+                        && selection == null)) {
             return getRecordCount();
         }
         long total = 0L;
@@ -514,7 +516,8 @@ public class ParquetFileReader implements Closeable {
 
         // Filtering not required -> fall back to the non-filtering path
         if (!options.useColumnIndexFilter()
-                || !FilterCompat.isFilteringRequired(options.getRecordFilter())) {
+                || (!FilterCompat.isFilteringRequired(options.getRecordFilter())
+                        && selection == null)) {
             return internalReadRowGroup(blockIndex);
         }
 
@@ -585,14 +588,8 @@ public class ParquetFileReader implements Closeable {
             List<ConsecutivePartList> allParts, ChunkListBuilder builder) throws IOException {
 
         if (shouldUseVectoredIo(allParts)) {
-            try {
-                readVectored(allParts, builder);
-                return;
-            } catch (IllegalArgumentException | UnsupportedOperationException e) {
-                // Either the arguments are wrong or somehow this is being invoked against
-                // a hadoop release which doesn't have the API and yet somehow it got here.
-                LOG.warn("readVectored() failed; falling back to normal IO against {}", f, e);
-            }
+            readVectored(allParts, builder);
+            return;
         }
         for (ConsecutivePartList consecutiveChunks : allParts) {
             consecutiveChunks.readAll(f, builder);
@@ -650,7 +647,7 @@ public class ParquetFileReader implements Closeable {
     private void readVectored(List<ConsecutivePartList> allParts, ChunkListBuilder builder)
             throws IOException {
 
-        List<ParquetFileRange> ranges = new ArrayList<>(allParts.size());
+        List<FileRange> ranges = new ArrayList<>(allParts.size());
         long totalSize = 0;
         for (ConsecutivePartList consecutiveChunks : allParts) {
             final long len = consecutiveChunks.length;
@@ -658,16 +655,16 @@ public class ParquetFileReader implements Closeable {
                     len < Integer.MAX_VALUE,
                     "Invalid length %s for vectored read operation. It must be less than max integer value.",
                     len);
-            ranges.add(new ParquetFileRange(consecutiveChunks.offset, (int) len));
+            ranges.add(FileRange.createFileRange(consecutiveChunks.offset, (int) len));
             totalSize += len;
         }
         LOG.debug(
                 "Reading {} bytes of data with vectored IO in {} ranges", totalSize, ranges.size());
         // Request a vectored read;
-        f.readVectored(ranges, options.getAllocator());
+        ((VectoredReadable) f.in()).readVectored(ranges);
         int k = 0;
         for (ConsecutivePartList consecutivePart : allParts) {
-            ParquetFileRange currRange = ranges.get(k++);
+            FileRange currRange = ranges.get(k++);
             consecutivePart.readFromVectoredRange(currRange, builder);
         }
     }
@@ -687,7 +684,8 @@ public class ParquetFileReader implements Closeable {
         }
         // Filtering not required -> fall back to the non-filtering path
         if (!options.useColumnIndexFilter()
-                || !FilterCompat.isFilteringRequired(options.getRecordFilter())) {
+                || (!FilterCompat.isFilteringRequired(options.getRecordFilter())
+                        && selection == null)) {
             return readNextRowGroup();
         }
         BlockMetaData block = blocks.get(currentBlock);
@@ -798,23 +796,60 @@ public class ParquetFileReader implements Closeable {
     }
 
     private RowRanges getRowRanges(int blockIndex) {
-        assert FilterCompat.isFilteringRequired(options.getRecordFilter())
-                : "Should not be invoked if filter is null or NOOP";
+        boolean filteringRequired = FilterCompat.isFilteringRequired(options.getRecordFilter());
+        if (!filteringRequired && selection == null) {
+            throw new IllegalArgumentException("Should not be invoked if filter is null or NOOP");
+        }
+
         RowRanges rowRanges = blockRowRanges.get(blockIndex);
         if (rowRanges == null) {
-            rowRanges =
-                    ColumnIndexFilter.calculateRowRanges(
-                            options.getRecordFilter(),
-                            getColumnIndexStore(blockIndex),
-                            paths.keySet(),
-                            blocks.get(blockIndex).getRowCount(),
-                            blocks.get(blockIndex).getRowIndexOffset(),
-                            selection);
+            rowRanges = calculateRowRanges(blockIndex);
             blockRowRanges.set(blockIndex, rowRanges);
         }
         return rowRanges;
     }
 
+    private RowRanges calculateRowRanges(int blockIndex) {
+        BlockMetaData block = blocks.get(blockIndex);
+        RowRanges rowRanges = RowRanges.createSingle(block.getRowCount());
+        FilterCompat.Filter recordFilter = options.getRecordFilter();
+        boolean filteringRequired = FilterCompat.isFilteringRequired(recordFilter);
+        if (selection != null || filteringRequired) {
+            ColumnIndexStore store = getColumnIndexStore(blockIndex);
+
+            // do filter by selection
+            if (selection != null) {
+                List<OffsetIndex> offsets = new ArrayList<>();
+                for (ColumnChunkMetaData mc : block.getColumns()) {
+                    ColumnPath pathKey = mc.getPath();
+                    if (paths.containsKey(pathKey)) {
+                        try {
+                            offsets.add(store.getOffsetIndex(pathKey));
+                        } catch (MissingOffsetIndexException ignored) {
+                        }
+                    }
+                }
+                long rowCount = block.getRowCount();
+                long rowIndexOffset = block.getRowIndexOffset();
+                for (OffsetIndex offset : offsets) {
+                    // avoiding creating too many ranges, just filter columns pages
+                    RowRanges result =
+                            RowRanges.create(rowCount, rowIndexOffset, offset, selection);
+                    rowRanges = RowRanges.intersection(result, rowRanges);
+                }
+            }
+
+            // do filter by push down filter
+            if (filteringRequired) {
+                RowRanges result =
+                        ColumnIndexFilter.calculateRowRanges(
+                                recordFilter, store, paths.keySet(), block.getRowCount());
+                rowRanges = RowRanges.intersection(result, rowRanges);
+            }
+        }
+        return rowRanges;
+    }
+
     public boolean skipNextRowGroup() {
         return advanceToNextBlock();
     }
@@ -1667,9 +1702,9 @@ public class ParquetFileReader implements Closeable {
          * @throws IOException if there is an error while reading from the stream, including a
          *     timeout.
          */
-        public void readFromVectoredRange(ParquetFileRange currRange, ChunkListBuilder builder)
+        public void readFromVectoredRange(FileRange currRange, ChunkListBuilder builder)
                 throws IOException {
-            ByteBuffer buffer;
+            byte[] buffer;
             final long timeoutSeconds = HADOOP_VECTORED_READ_TIMEOUT_SECONDS;
             long readStart = System.nanoTime();
             try {
@@ -1678,8 +1713,7 @@ public class ParquetFileReader implements Closeable {
                         currRange,
                         timeoutSeconds);
                 buffer =
-                        FutureIO.awaitFuture(
-                                currRange.getDataReadFuture(), timeoutSeconds, TimeUnit.SECONDS);
+                        FutureIO.awaitFuture(currRange.getData(), timeoutSeconds, TimeUnit.SECONDS);
                 setReadMetrics(readStart, currRange.getLength());
                 // report in a counter the data we just scanned
                 BenchmarkCounter.incrementBytesRead(currRange.getLength());
@@ -1691,7 +1725,7 @@ public class ParquetFileReader implements Closeable {
                 LOG.error(error, e);
                 throw new IOException(error, e);
             }
-            ByteBufferInputStream stream = ByteBufferInputStream.wrap(buffer);
+            ByteBufferInputStream stream = ByteBufferInputStream.wrap(ByteBuffer.wrap(buffer));
             for (ChunkDescriptor descriptor : chunks) {
                 builder.add(descriptor, stream.sliceBuffers(descriptor.size), f);
             }
