@@ -114,6 +114,7 @@ import static org.apache.paimon.catalog.CatalogUtils.checkNotSystemTable;
 import static org.apache.paimon.catalog.CatalogUtils.isSystemDatabase;
 import static org.apache.paimon.catalog.CatalogUtils.listPartitionsFromFileSystem;
 import static org.apache.paimon.catalog.Identifier.DEFAULT_MAIN_BRANCH;
+import static org.apache.paimon.format.csv.CsvOptions.FIELD_DELIMITER;
 import static org.apache.paimon.hive.HiveCatalogOptions.HADOOP_CONF_DIR;
 import static org.apache.paimon.hive.HiveCatalogOptions.HIVE_CONF_DIR;
 import static org.apache.paimon.hive.HiveCatalogOptions.IDENTIFIER;
@@ -124,7 +125,6 @@ import static org.apache.paimon.options.CatalogOptions.FORMAT_TABLE_ENABLED;
 import static org.apache.paimon.options.CatalogOptions.SYNC_ALL_PROPERTIES;
 import static org.apache.paimon.options.CatalogOptions.TABLE_TYPE;
 import static org.apache.paimon.options.OptionsUtils.convertToPropertiesPrefixKey;
-import static org.apache.paimon.table.FormatTableOptions.FIELD_DELIMITER;
 import static org.apache.paimon.utils.HadoopUtils.addHadoopConfIfFound;
 import static org.apache.paimon.utils.Preconditions.checkArgument;
 import static org.apache.paimon.utils.StringUtils.isNullOrWhitespaceOnly;
@@ -152,6 +152,8 @@ public class HiveCatalog extends AbstractCatalog {
     private static final String HIVE_EXTERNAL_TABLE_PROP = "EXTERNAL";
     private static final int DEFAULT_TABLE_BATCH_SIZE = 300;
     private static final String HIVE_LAST_UPDATE_TIME_PROP = "transient_lastDdlTime";
+    // hive default field delimiter is '\u0001'
+    public static final String HIVE_FIELD_DELIM_DEFAULT = "\u0001";
 
     private final HiveConf hiveConf;
     private final String clientClassName;
@@ -994,6 +996,23 @@ public class HiveCatalog extends AbstractCatalog {
 
     @Override
     protected void createTableImpl(Identifier identifier, Schema schema) {
+        try {
+            boolean tableExists =
+                    clients.run(
+                            (client ->
+                                    client.tableExists(
+                                            identifier.getDatabaseName(),
+                                            identifier.getTableName())));
+            if (tableExists) {
+                throw new RuntimeException(
+                        "Table "
+                                + identifier.getFullName()
+                                + " already exists, but this table is not a paimon table.");
+            }
+        } catch (TException | InterruptedException e) {
+            throw new RuntimeException(e);
+        }
+
         Pair<Path, Boolean> pair = initialTableLocation(schema.options(), identifier);
         Path location = pair.getLeft();
         boolean externalTable = pair.getRight();
@@ -1465,10 +1484,14 @@ public class HiveCatalog extends AbstractCatalog {
         return OUTPUT_FORMAT_CLASS_NAME;
     }
 
-    private Map<String, String> setSerDeInfoParam(@Nullable FormatTable.Format provider) {
+    private Map<String, String> setSerDeInfoParam(
+            @Nullable FormatTable.Format provider, Map<String, String> tableParameters) {
         Map<String, String> param = new HashMap<>();
         if (provider == FormatTable.Format.CSV) {
-            param.put(FIELD_DELIM, options.get(FIELD_DELIMITER));
+            param.put(
+                    FIELD_DELIM,
+                    tableParameters.getOrDefault(
+                            FIELD_DELIMITER.key(), options.get(FIELD_DELIMITER)));
         }
         return param;
     }
@@ -1485,7 +1508,7 @@ public class HiveCatalog extends AbstractCatalog {
         sd.setOutputFormat(getOutputFormatClassName(provider));
 
         SerDeInfo serDeInfo = sd.getSerdeInfo() != null ? sd.getSerdeInfo() : new SerDeInfo();
-        serDeInfo.setParameters(setSerDeInfoParam(provider));
+        serDeInfo.setParameters(setSerDeInfoParam(provider, schema.options()));
         serDeInfo.setSerializationLib(getSerdeClassName(provider));
         sd.setSerdeInfo(serDeInfo);
 
