@@ -18,4 +18,4 @@
 
 package org.apache.paimon.spark.sql
 
-class TagDdlTest extends PaimonTagDdlTestBase {}
+class TagDdlTest extends PaimonTagDdlTestBase
