@@ -18,12 +18,13 @@
 
 package org.apache.paimon.spark.sql
 
-import org.apache.spark.sql.catalyst.dsl.expressions._
 import org.apache.spark.sql.catalyst.expressions.{Attribute, GetStructField, NamedExpression, ScalarSubquery}
 import org.apache.spark.sql.paimon.shims.SparkShimLoader
 
 class PaimonOptimizationTest extends PaimonOptimizationTestBase {
 
+  import org.apache.spark.sql.catalyst.dsl.expressions._
+
   override def extractorExpression(
       cteIndex: Int,
       output: Seq[Attribute],
@@ -32,8 +33,7 @@ class PaimonOptimizationTest extends PaimonOptimizationTestBase {
       ScalarSubquery(
         SparkShimLoader.shim
           .createCTERelationRef(cteIndex, resolved = true, output.toSeq, isStreaming = false)),
-      fieldIndex,
-      None)
+      fieldIndex)
       .as("scalarsubquery()")
   }
 }
