@@ -18,8 +18,6 @@
 
 package org.apache.paimon.spark.procedure;
 
-import org.apache.paimon.fs.FileIO;
-import org.apache.paimon.fs.Path;
 import org.apache.paimon.table.FileStoreTable;
 
 import org.apache.spark.sql.catalyst.InternalRow;
@@ -30,9 +28,6 @@ import org.apache.spark.sql.types.StructField;
 import org.apache.spark.sql.types.StructType;
 import org.apache.spark.unsafe.types.UTF8String;
 
-import java.io.IOException;
-import java.util.Arrays;
-
 import static org.apache.spark.sql.types.DataTypes.StringType;
 
 /** A procedure to purge files for a table. */
@@ -64,36 +59,16 @@ public class PurgeFilesProcedure extends BaseProcedure {
     @Override
     public InternalRow[] call(InternalRow args) {
         Identifier tableIdent = toIdentifier(args.getString(0), PARAMETERS[0].name());
-
         return modifyPaimonTable(
                 tableIdent,
                 table -> {
-                    FileStoreTable fileStoreTable = (FileStoreTable) table;
-                    FileIO fileIO = fileStoreTable.fileIO();
-                    Path tablePath = fileStoreTable.snapshotManager().tablePath();
                     try {
-                        Arrays.stream(fileIO.listStatus(tablePath))
-                                .filter(f -> !f.getPath().getName().contains("schema"))
-                                .forEach(
-                                        fileStatus -> {
-                                            try {
-                                                fileIO.delete(fileStatus.getPath(), true);
-                                            } catch (IOException e) {
-                                                throw new RuntimeException(e);
-                                            }
-                                        });
-                        spark().catalog().refreshTable(table.fullName());
-                    } catch (IOException e) {
+                        ((FileStoreTable) table).purgeFiles();
+                    } catch (Exception e) {
                         throw new RuntimeException(e);
                     }
 
-                    InternalRow outputRow =
-                            newInternalRow(
-                                    UTF8String.fromString(
-                                            String.format(
-                                                    "Success purge files with table: %s.",
-                                                    fileStoreTable.name())));
-                    return new InternalRow[] {outputRow};
+                    return new InternalRow[] {newInternalRow(UTF8String.fromString("Success"))};
                 });
     }
 
