@@ -20,13 +20,16 @@ package org.apache.paimon.spark.sql
 
 import org.apache.paimon.spark.PaimonSparkTestBase
 
-import org.apache.spark.sql.Row
+import org.apache.spark.sql.{Row, SaveMode}
+import org.apache.spark.sql.functions.lit
 import org.apache.spark.sql.types._
 
 import java.sql.{Date, Timestamp}
 
 abstract class InsertOverwriteTableTestBase extends PaimonSparkTestBase {
 
+  import testImplicits._
+
   fileFormats.foreach {
     fileFormat =>
       Seq(true, false).foreach {
@@ -580,30 +583,38 @@ abstract class InsertOverwriteTableTestBase extends PaimonSparkTestBase {
     withTable("my_table") {
       sql("CREATE TABLE my_table (id INT, pt STRING) PARTITIONED BY (pt)")
 
-      for (mode <- Seq("static", "dynamic")) {
-        withSparkSQLConf("spark.sql.sources.partitionOverwriteMode" -> mode) {
-          sql("INSERT OVERWRITE my_table VALUES (1, 'p1'), (2, 'p2')")
-          // INSERT OVERWRITE table
-          sql("INSERT OVERWRITE my_table VALUES (3, 'p1')")
-          if (mode == "dynamic") {
-            checkAnswer(sql("SELECT * FROM my_table ORDER BY id"), Seq(Row(2, "p2"), Row(3, "p1")))
-          } else {
-            checkAnswer(sql("SELECT * FROM my_table ORDER BY id"), Row(3, "p1"))
-          }
+      for (useV2Write <- Seq("true", "false")) {
+        for (mode <- Seq("static", "dynamic")) {
+          withSparkSQLConf(
+            "spark.sql.sources.partitionOverwriteMode" -> mode,
+            "spark.paimon.write.use-v2-write" -> useV2Write) {
+            sql("INSERT OVERWRITE my_table VALUES (1, 'p1'), (2, 'p2')")
+            // INSERT OVERWRITE table
+            sql("INSERT OVERWRITE my_table VALUES (3, 'p1')")
+            if (mode == "dynamic") {
+              checkAnswer(
+                sql("SELECT * FROM my_table ORDER BY id"),
+                Seq(Row(2, "p2"), Row(3, "p1")))
+            } else {
+              checkAnswer(sql("SELECT * FROM my_table ORDER BY id"), Row(3, "p1"))
+            }
 
-          sql("INSERT OVERWRITE my_table VALUES (1, 'p1'), (2, 'p2')")
-          // INSERT OVERWRITE table PARTITION (pt)
-          sql("INSERT OVERWRITE my_table PARTITION (pt) VALUES (3, 'p1')")
-          if (mode == "dynamic") {
-            checkAnswer(sql("SELECT * FROM my_table ORDER BY id"), Seq(Row(2, "p2"), Row(3, "p1")))
-          } else {
-            checkAnswer(sql("SELECT * FROM my_table ORDER BY id"), Row(3, "p1"))
-          }
+            sql("INSERT OVERWRITE my_table VALUES (1, 'p1'), (2, 'p2')")
+            // INSERT OVERWRITE table PARTITION (pt)
+            sql("INSERT OVERWRITE my_table PARTITION (pt) VALUES (3, 'p1')")
+            if (mode == "dynamic") {
+              checkAnswer(
+                sql("SELECT * FROM my_table ORDER BY id"),
+                Seq(Row(2, "p2"), Row(3, "p1")))
+            } else {
+              checkAnswer(sql("SELECT * FROM my_table ORDER BY id"), Row(3, "p1"))
+            }
 
-          sql("INSERT OVERWRITE my_table VALUES (1, 'p1'), (2, 'p2')")
-          // INSERT OVERWRITE table PARTITION (pt='p1')
-          sql("INSERT OVERWRITE my_table PARTITION (pt='p1') VALUES (3)")
-          checkAnswer(sql("SELECT * FROM my_table ORDER BY id"), Seq(Row(2, "p2"), Row(3, "p1")))
+            sql("INSERT OVERWRITE my_table VALUES (1, 'p1'), (2, 'p2')")
+            // INSERT OVERWRITE table PARTITION (pt='p1')
+            sql("INSERT OVERWRITE my_table PARTITION (pt='p1') VALUES (3)")
+            checkAnswer(sql("SELECT * FROM my_table ORDER BY id"), Seq(Row(2, "p2"), Row(3, "p1")))
+          }
         }
       }
     }
@@ -631,4 +642,16 @@ abstract class InsertOverwriteTableTestBase extends PaimonSparkTestBase {
       }
     }
   }
+
+  test("Paimon Insert: insert unsupported type") {
+    val rows = Seq(("1", "2")).toDF("f1", "f2")
+    assert(intercept[Exception] {
+      rows
+        .withColumn("bad", lit(null))
+        .write
+        .format("paimon")
+        .mode(SaveMode.Overwrite)
+        .saveAsTable("badTable")
+    }.getMessage.contains("Not a supported type: void"))
+  }
 }
