@@ -18,10 +18,15 @@
 
 package org.apache.paimon.spark.sql
 
-import org.apache.paimon.spark.{PaimonPrimaryKeyTable, PaimonSparkTestBase, PaimonTableTest}
+import org.apache.paimon.Snapshot
+import org.apache.paimon.spark.{PaimonAppendTable, PaimonPrimaryKeyTable, PaimonSparkTestBase, PaimonTableTest}
 
 import org.apache.spark.sql.Row
 
+import scala.concurrent.{Await, Future}
+import scala.concurrent.ExecutionContext.Implicits.global
+import scala.concurrent.duration.DurationInt
+
 abstract class MergeIntoTableTestBase extends PaimonSparkTestBase with PaimonTableTest {
 
   import testImplicits._
@@ -135,6 +140,53 @@ abstract class MergeIntoTableTestBase extends PaimonSparkTestBase with PaimonTab
     }
   }
 
+  test(s"Paimon MergeInto: partial insert with null") {
+    withTable("source", "target") {
+
+      Seq((1, 100, "c11"), (3, 300, "c33")).toDF("a", "b", "c").createOrReplaceTempView("source")
+
+      createTable("target", "a INT, b INT, c STRING", Seq("a"))
+      spark.sql("INSERT INTO target values (1, 10, 'c1'), (2, 20, 'c2')")
+
+      spark.sql(s"""
+                   |MERGE INTO target
+                   |USING source
+                   |ON target.a = source.a
+                   |WHEN NOT MATCHED
+                   |THEN INSERT (a) values (a)
+                   |""".stripMargin)
+
+      checkAnswer(
+        spark.sql("SELECT * FROM target ORDER BY a, b"),
+        Row(1, 10, "c1") :: Row(2, 20, "c2") :: Row(3, null, null) :: Nil)
+    }
+  }
+
+  test(s"Paimon MergeInto: partial insert with default value") {
+    // The syntax for specifying column default values when creating Paimon tables is only supported in Spark 3.4 and above
+    if (gteqSpark3_4) {
+      withTable("source", "target") {
+
+        Seq((1, 100, "c11"), (3, 300, "c33")).toDF("a", "b", "c").createOrReplaceTempView("source")
+
+        createTable("target", "a INT, b INT DEFAULT 6, c STRING DEFAULT 'test'", Seq("a"))
+        spark.sql("INSERT INTO target values (1, 10, 'c1'), (2, 20, 'c2')")
+
+        spark.sql(s"""
+                     |MERGE INTO target
+                     |USING source
+                     |ON target.a = source.a
+                     |WHEN NOT MATCHED
+                     |THEN INSERT (a) values (a)
+                     |""".stripMargin)
+
+        checkAnswer(
+          spark.sql("SELECT * FROM target ORDER BY a, b"),
+          Row(1, 10, "c1") :: Row(2, 20, "c2") :: Row(3, 6, "test") :: Nil)
+      }
+    }
+  }
+
   test(s"Paimon MergeInto: update + insert with data file path") {
     withTable("source", "target") {
 
@@ -418,7 +470,7 @@ abstract class MergeIntoTableTestBase extends PaimonSparkTestBase with PaimonTab
   test("Paimon MergeInto: fail in case that miss some columns in insert") {
     withTable("source", "target") {
 
-      Seq((1, 100, "c11"), (3, 300, "c33")).toDF("a", "b", "c").createOrReplaceTempView("source")
+      Seq((1, "c11"), (3, "c33")).toDF("a", "c").createOrReplaceTempView("source")
 
       createTable("target", "a INT, b INT, c STRING", Seq("a"))
       spark.sql("INSERT INTO target values (1, 10, 'c1'), (2, 20, 'c2')")
@@ -431,10 +483,10 @@ abstract class MergeIntoTableTestBase extends PaimonSparkTestBase with PaimonTab
                      |WHEN MATCHED THEN
                      |UPDATE SET *
                      |WHEN NOT MATCHED
-                     |THEN INSERT (a, b) VALUES (a, b)
+                     |THEN INSERT *
                      |""".stripMargin)
       }.getMessage
-      assert(error.contains("Can't align the table's columns in insert clause."))
+      assert(error.contains("cannot resolve b from Project"))
     }
   }
 
@@ -485,7 +537,7 @@ abstract class MergeIntoTableTestBase extends PaimonSparkTestBase with PaimonTab
                      |THEN INSERT (a, b, c) values (a, b, c)
                      |""".stripMargin)
       }.getMessage
-      assert(error.contains("match more then one source rows"))
+      assert(error.contains("match more than one source rows"))
     }
   }
 
@@ -606,6 +658,24 @@ abstract class MergeIntoTableTestBase extends PaimonSparkTestBase with PaimonTab
       )
     }
   }
+
+  test(s"Paimon MergeInto: merge into with varchar") {
+    withTable("source", "target") {
+      createTable("source", "a INT, b VARCHAR(32)", Seq("a"))
+      createTable("target", "a INT, b VARCHAR(32)", Seq("a"))
+      sql("INSERT INTO target values (1, 'Alice'), (2, 'Bob')")
+      sql("INSERT INTO source values (1, 'Eve'), (3, 'Cat')")
+
+      sql(s"""
+             |MERGE INTO target
+             |USING source
+             |ON target.a = source.a
+             |WHEN MATCHED THEN
+             |UPDATE SET a = source.a, b = source.b
+             |""".stripMargin)
+      checkAnswer(sql("SELECT * FROM target ORDER BY a, b"), Seq(Row(1, "Eve"), Row(2, "Bob")))
+    }
+  }
 }
 
 trait MergeIntoPrimaryKeyTableTest extends PaimonSparkTestBase with PaimonPrimaryKeyTable {
@@ -649,3 +719,90 @@ trait MergeIntoPrimaryKeyTableTest extends PaimonSparkTestBase with PaimonPrimar
     }
   }
 }
+
+trait MergeIntoAppendTableTest extends PaimonSparkTestBase with PaimonAppendTable {
+
+  test("Paimon MergeInto: non pk table commit kind") {
+    withTable("s", "t") {
+      createTable("s", "id INT, b INT, c INT", Seq("id"))
+      sql("INSERT INTO s VALUES (1, 1, 1)")
+
+      createTable("t", "id INT, b INT, c INT", Seq("id"))
+      sql("INSERT INTO t VALUES (2, 2, 2)")
+
+      sql("""
+            |MERGE INTO t
+            |USING s
+            |ON t.id = s.id
+            |WHEN NOT MATCHED THEN
+            |INSERT (id, b, c) VALUES (s.id, s.b, s.c);
+            |""".stripMargin)
+
+      val table = loadTable("t")
+      var latestSnapshot = table.latestSnapshot().get()
+      assert(latestSnapshot.id == 2)
+      // no old data is deleted, so the commit kind is APPEND
+      assert(latestSnapshot.commitKind.equals(Snapshot.CommitKind.APPEND))
+
+      sql("INSERT INTO s VALUES (2, 22, 22)")
+      sql("""
+            |MERGE INTO t
+            |USING s
+            |ON t.id = s.id
+            |WHEN MATCHED THEN
+            |UPDATE SET id = s.id, b = s.b, c = s.c
+            |""".stripMargin)
+      latestSnapshot = table.latestSnapshot().get()
+      assert(latestSnapshot.id == 3)
+      // new data is updated, so the commit kind is OVERWRITE
+      assert(latestSnapshot.commitKind.equals(Snapshot.CommitKind.OVERWRITE))
+    }
+  }
+
+  test("Paimon MergeInto: concurrent merge and compact") {
+    for (dvEnabled <- Seq("true", "false")) {
+      withTable("s", "t") {
+        sql("CREATE TABLE s (id INT, b INT, c INT)")
+        sql("INSERT INTO s VALUES (1, 1, 1)")
+
+        sql(
+          s"CREATE TABLE t (id INT, b INT, c INT) TBLPROPERTIES ('deletion-vectors.enabled' = '$dvEnabled')")
+        sql("INSERT INTO t VALUES (1, 1, 1)")
+
+        val mergeInto = Future {
+          for (_ <- 1 to 10) {
+            try {
+              sql("""
+                    |MERGE INTO t
+                    |USING s
+                    |ON t.id = s.id
+                    |WHEN MATCHED THEN
+                    |UPDATE SET t.id = s.id, t.b = s.b + t.b, t.c = s.c + t.c
+                    |""".stripMargin)
+            } catch {
+              case a: Throwable =>
+                assert(
+                  a.getMessage.contains("Conflicts during commits") || a.getMessage.contains(
+                    "Missing file"))
+            }
+            checkAnswer(sql("SELECT count(*) FROM t"), Seq(Row(1)))
+          }
+        }
+
+        val compact = Future {
+          for (_ <- 1 to 10) {
+            try {
+              sql("CALL sys.compact(table => 't', order_strategy => 'order', order_by => 'id')")
+            } catch {
+              case a: Throwable => assert(a.getMessage.contains("Conflicts during commits"))
+            }
+            checkAnswer(sql("SELECT count(*) FROM t"), Seq(Row(1)))
+          }
+        }
+
+        Await.result(mergeInto, 60.seconds)
+        Await.result(compact, 60.seconds)
+      }
+    }
+  }
+}
