@@ -24,5 +24,9 @@ import org.apache.paimon.types.RowType
 import org.apache.spark.unsafe.types.VariantVal
 
 class Spark4InternalRow(rowType: RowType) extends AbstractSparkInternalRow(rowType) {
-  override def getVariant(i: Int): VariantVal = throw new UnsupportedOperationException
+
+  override def getVariant(i: Int): VariantVal = {
+    val v = row.getVariant(i)
+    new VariantVal(v.value(), v.metadata())
+  }
 }
